@@ -23,19 +23,23 @@ class SharedArrayBufferBuiltinsAssembler : public CodeStubAssembler {
                                                      Node* base, Node* offset,
                                                      Node* value,
                                                      Node* value_high);
-  void ValidateSharedTypedArray(Node* tagged, Node* context,
-                                TNode<Int32T>* out_elements_kind,
-                                Node** out_backing_store);
-  Node* ConvertTaggedAtomicIndexToWord32(Node* tagged, Node* context,
-                                         Node** number_index);
-  void ValidateAtomicIndex(Node* array, Node* index_word, Node* context);
-#if DEBUG
-  void DebugSanityCheckAtomicIndex(Node* array, Node* index_word,
-                                   Node* context);
-#endif
-  void AtomicBinopBuiltinCommon(Node* array, Node* index, Node* value,
-                                Node* context, AssemblerFunction function,
-                                Runtime::FunctionId runtime_function);
+  TNode<JSArrayBuffer> ValidateIntegerTypedArray(
+      TNode<Object> maybe_array, TNode<Context> context,
+      TNode<Int32T>* out_elements_kind, TNode<RawPtrT>* out_backing_store,
+      Label* detached);
+
+  TNode<UintPtrT> ValidateAtomicAccess(TNode<JSTypedArray> array,
+                                       TNode<Object> index,
+                                       TNode<Context> context);
+
+  inline void DebugSanityCheckAtomicIndex(TNode<JSTypedArray> array,
+                                          TNode<UintPtrT> index);
+
+  void AtomicBinopBuiltinCommon(TNode<Object> maybe_array, TNode<Object> index,
+                                TNode<Object> value, TNode<Context> context,
+                                AssemblerFunction function,
+                                Runtime::FunctionId runtime_function,
+                                const char* method_name);
 
   // Create a BigInt from the result of a 64-bit atomic operation, using
   // projections on 32-bit platforms.
@@ -43,22 +47,28 @@ class SharedArrayBufferBuiltinsAssembler : public CodeStubAssembler {
   TNode<BigInt> BigIntFromUnsigned64(Node* unsigned64);
 };
 
-void SharedArrayBufferBuiltinsAssembler::ValidateSharedTypedArray(
-    Node* tagged, Node* context, TNode<Int32T>* out_elements_kind,
-    Node** out_backing_store) {
+// https://tc39.es/ecma262/#sec-validateintegertypedarray
+TNode<JSArrayBuffer>
+SharedArrayBufferBuiltinsAssembler::ValidateIntegerTypedArray(
+    TNode<Object> maybe_array, TNode<Context> context,
+    TNode<Int32T>* out_elements_kind, TNode<RawPtrT>* out_backing_store,
+    Label* detached) {
   Label not_float_or_clamped(this), invalid(this);
 
+  // The logic of TypedArrayBuiltinsAssembler::ValidateTypedArrayBuffer is
+  // inlined to avoid duplicate error branches.
+
   // Fail if it is not a heap object.
-  GotoIf(TaggedIsSmi(tagged), &invalid);
+  GotoIf(TaggedIsSmi(maybe_array), &invalid);
 
   // Fail if the array's instance type is not JSTypedArray.
-  TNode<Map> tagged_map = LoadMap(tagged);
-  GotoIfNot(IsJSTypedArrayMap(tagged_map), &invalid);
+  TNode<Map> map = LoadMap(CAST(maybe_array));
+  GotoIfNot(IsJSTypedArrayMap(map), &invalid);
+  TNode<JSTypedArray> array = CAST(maybe_array);
 
-  // Fail if the array's JSArrayBuffer is not shared.
-  TNode<JSArrayBuffer> array_buffer = LoadJSArrayBufferViewBuffer(CAST(tagged));
-  TNode<Uint32T> bitfield = LoadJSArrayBufferBitField(array_buffer);
-  GotoIfNot(IsSetWord32<JSArrayBuffer::IsSharedBit>(bitfield), &invalid);
+  // Fail if the array's JSArrayBuffer is detached.
+  TNode<JSArrayBuffer> array_buffer = GetTypedArrayBuffer(context, array);
+  GotoIf(IsDetachedBuffer(array_buffer), detached);
 
   // Fail if the array's element type is float32, float64 or clamped.
   STATIC_ASSERT(INT8_ELEMENTS < FLOAT32_ELEMENTS);
@@ -67,7 +77,7 @@ void SharedArrayBufferBuiltinsAssembler::ValidateSharedTypedArray(
   STATIC_ASSERT(UINT8_ELEMENTS < FLOAT32_ELEMENTS);
   STATIC_ASSERT(UINT16_ELEMENTS < FLOAT32_ELEMENTS);
   STATIC_ASSERT(UINT32_ELEMENTS < FLOAT32_ELEMENTS);
-  TNode<Int32T> elements_kind = LoadMapElementsKind(tagged_map);
+  TNode<Int32T> elements_kind = LoadMapElementsKind(map);
   GotoIf(Int32LessThan(elements_kind, Int32Constant(FLOAT32_ELEMENTS)),
          &not_float_or_clamped);
   STATIC_ASSERT(BIGINT64_ELEMENTS > UINT8_CLAMPED_ELEMENTS);
@@ -77,66 +87,59 @@ void SharedArrayBufferBuiltinsAssembler::ValidateSharedTypedArray(
 
   BIND(&invalid);
   {
-    ThrowTypeError(context, MessageTemplate::kNotIntegerSharedTypedArray,
-                   tagged);
+    ThrowTypeError(context, MessageTemplate::kNotIntegerTypedArray,
+                   maybe_array);
   }
 
   BIND(&not_float_or_clamped);
   *out_elements_kind = elements_kind;
 
-  TNode<RawPtrT> backing_store = LoadJSArrayBufferBackingStore(array_buffer);
-  TNode<UintPtrT> byte_offset = LoadJSArrayBufferViewByteOffset(CAST(tagged));
-  *out_backing_store = IntPtrAdd(backing_store, byte_offset);
+  TNode<RawPtrT> backing_store = LoadJSArrayBufferBackingStorePtr(array_buffer);
+  TNode<UintPtrT> byte_offset = LoadJSArrayBufferViewByteOffset(array);
+  *out_backing_store = RawPtrAdd(backing_store, Signed(byte_offset));
+
+  return array_buffer;
 }
 
 // https://tc39.github.io/ecma262/#sec-validateatomicaccess
-Node* SharedArrayBufferBuiltinsAssembler::ConvertTaggedAtomicIndexToWord32(
-    Node* tagged, Node* context, Node** number_index) {
-  VARIABLE(var_result, MachineRepresentation::kWord32);
+// ValidateAtomicAccess( typedArray, requestIndex )
+TNode<UintPtrT> SharedArrayBufferBuiltinsAssembler::ValidateAtomicAccess(
+    TNode<JSTypedArray> array, TNode<Object> index, TNode<Context> context) {
   Label done(this), range_error(this);
 
-  // Returns word32 since index cannot be longer than a TypedArray length,
-  // which has a uint32 maximum.
-  // The |number_index| output parameter is used only for architectures that
-  // don't currently have a TF implementation and forward to runtime functions
-  // instead; they expect the value has already been coerced to an integer.
-  *number_index = ToSmiIndex(CAST(context), CAST(tagged), &range_error);
-  var_result.Bind(SmiToInt32(*number_index));
-  Goto(&done);
+  // 1. Assert: typedArray is an Object that has a [[ViewedArrayBuffer]]
+  // internal slot.
+  // 2. Let length be typedArray.[[ArrayLength]].
+  TNode<UintPtrT> array_length = LoadJSTypedArrayLength(array);
+
+  // 3. Let accessIndex be ? ToIndex(requestIndex).
+  TNode<UintPtrT> index_uintptr = ToIndex(context, index, &range_error);
+
+  // 4. Assert: accessIndex ≥ 0.
+  // 5. If accessIndex ≥ length, throw a RangeError exception.
+  Branch(UintPtrLessThan(index_uintptr, array_length), &done, &range_error);
 
   BIND(&range_error);
-  { ThrowRangeError(context, MessageTemplate::kInvalidAtomicAccessIndex); }
-
-  BIND(&done);
-  return var_result.value();
-}
-
-void SharedArrayBufferBuiltinsAssembler::ValidateAtomicIndex(Node* array,
-                                                             Node* index,
-                                                             Node* context) {
-  // Check if the index is in bounds. If not, throw RangeError.
-  Label check_passed(this);
-  TNode<UintPtrT> array_length = LoadJSTypedArrayLength(CAST(array));
-  // TODO(v8:4153): Use UintPtr for the {index} as well.
-  GotoIf(UintPtrLessThan(ChangeUint32ToWord(index), array_length),
-         &check_passed);
-
   ThrowRangeError(context, MessageTemplate::kInvalidAtomicAccessIndex);
 
-  BIND(&check_passed);
+  // 6. Return accessIndex.
+  BIND(&done);
+  return index_uintptr;
 }
 
-#if DEBUG
 void SharedArrayBufferBuiltinsAssembler::DebugSanityCheckAtomicIndex(
-    Node* array, Node* index_word, Node* context) {
-  // In Debug mode, we re-validate the index as a sanity check because
-  // ToInteger above calls out to JavaScript. A SharedArrayBuffer can't be
-  // detached and the TypedArray length can't change either, so skipping this
-  // check in Release mode is safe.
-  CSA_ASSERT(this, UintPtrLessThan(ChangeUint32ToWord(index_word),
-                                   LoadJSTypedArrayLength(CAST(array))));
+    TNode<JSTypedArray> array, TNode<UintPtrT> index) {
+  // In Debug mode, we re-validate the index as a sanity check because ToInteger
+  // above calls out to JavaScript. Atomics work on ArrayBuffers, which may be
+  // detached, and detachment state must be checked and throw before this
+  // check. The length cannot change.
+  //
+  // This function must always be called after ValidateIntegerTypedArray, which
+  // will ensure that LoadJSArrayBufferViewBuffer will not be null.
+  CSA_ASSERT(this, Word32BinaryNot(
+                       IsDetachedBuffer(LoadJSArrayBufferViewBuffer(array))));
+  CSA_ASSERT(this, UintPtrLessThan(index, LoadJSTypedArrayLength(array)));
 }
-#endif
 
 TNode<BigInt> SharedArrayBufferBuiltinsAssembler::BigIntFromSigned64(
     Node* signed64) {
@@ -160,21 +163,33 @@ TNode<BigInt> SharedArrayBufferBuiltinsAssembler::BigIntFromUnsigned64(
   }
 }
 
+// https://tc39.es/ecma262/#sec-atomicload
 TF_BUILTIN(AtomicsLoad, SharedArrayBufferBuiltinsAssembler) {
-  Node* array = Parameter(Descriptor::kArray);
-  Node* index = Parameter(Descriptor::kIndex);
-  Node* context = Parameter(Descriptor::kContext);
+  TNode<Object> maybe_array = CAST(Parameter(Descriptor::kArray));
+  TNode<Object> index = CAST(Parameter(Descriptor::kIndex));
+  TNode<Context> context = CAST(Parameter(Descriptor::kContext));
 
+  // 1. Let buffer be ? ValidateIntegerTypedArray(typedArray).
+  Label detached(this);
   TNode<Int32T> elements_kind;
-  Node* backing_store;
-  ValidateSharedTypedArray(array, context, &elements_kind, &backing_store);
+  TNode<RawPtrT> backing_store;
+  TNode<JSArrayBuffer> array_buffer = ValidateIntegerTypedArray(
+      maybe_array, context, &elements_kind, &backing_store, &detached);
+  TNode<JSTypedArray> array = CAST(maybe_array);
 
-  Node* index_integer;
-  Node* index_word32 =
-      ConvertTaggedAtomicIndexToWord32(index, context, &index_integer);
-  ValidateAtomicIndex(array, index_word32, context);
-  TNode<UintPtrT> index_word = ChangeUint32ToWord(index_word32);
+  // 2. Let i be ? ValidateAtomicAccess(typedArray, index).
+  TNode<UintPtrT> index_word = ValidateAtomicAccess(array, index, context);
 
+  // 3. If IsDetachedBuffer(buffer) is true, throw a TypeError exception.
+  // 4. NOTE: The above check is not redundant with the check in
+  // ValidateIntegerTypedArray because the call to ValidateAtomicAccess on the
+  // preceding line can have arbitrary side effects, which could cause the
+  // buffer to become detached.
+  GotoIf(IsDetachedBuffer(array_buffer), &detached);
+
+  // Steps 5-10.
+  //
+  // (Not copied from ecma262 due to the axiomatic nature of the memory model.)
   Label i8(this), u8(this), i16(this), u16(this), i32(this), u32(this),
       i64(this), u64(this), other(this);
   int32_t case_values[] = {
@@ -210,10 +225,13 @@ TF_BUILTIN(AtomicsLoad, SharedArrayBufferBuiltinsAssembler) {
                                          WordShl(index_word, 2))));
 #if V8_TARGET_ARCH_MIPS && !_MIPS_ARCH_MIPS32R6
   BIND(&i64);
-  Return(CallRuntime(Runtime::kAtomicsLoad64, context, array, index_integer));
+  Goto(&u64);
 
   BIND(&u64);
-  Return(CallRuntime(Runtime::kAtomicsLoad64, context, array, index_integer));
+  {
+    TNode<Number> index_number = ChangeUintPtrToTagged(index_word);
+    Return(CallRuntime(Runtime::kAtomicsLoad64, context, array, index_number));
+  }
 #else
   BIND(&i64);
   // This uses Uint64() intentionally: AtomicLoad is not implemented for
@@ -226,39 +244,62 @@ TF_BUILTIN(AtomicsLoad, SharedArrayBufferBuiltinsAssembler) {
   Return(BigIntFromUnsigned64(AtomicLoad(MachineType::Uint64(), backing_store,
                                          WordShl(index_word, 3))));
 #endif
+
   // This shouldn't happen, we've already validated the type.
   BIND(&other);
   Unreachable();
+
+  BIND(&detached);
+  {
+    ThrowTypeError(context, MessageTemplate::kDetachedOperation,
+                   "Atomics.load");
+  }
 }
 
+// https://tc39.es/ecma262/#sec-atomics.store
 TF_BUILTIN(AtomicsStore, SharedArrayBufferBuiltinsAssembler) {
-  Node* array = Parameter(Descriptor::kArray);
-  Node* index = Parameter(Descriptor::kIndex);
-  Node* value = Parameter(Descriptor::kValue);
-  Node* context = Parameter(Descriptor::kContext);
+  TNode<Object> maybe_array = CAST(Parameter(Descriptor::kArray));
+  TNode<Object> index = CAST(Parameter(Descriptor::kIndex));
+  TNode<Object> value = CAST(Parameter(Descriptor::kValue));
+  TNode<Context> context = CAST(Parameter(Descriptor::kContext));
 
+  // 1. Let buffer be ? ValidateIntegerTypedArray(typedArray).
+  Label detached(this);
   TNode<Int32T> elements_kind;
-  Node* backing_store;
-  ValidateSharedTypedArray(array, context, &elements_kind, &backing_store);
+  TNode<RawPtrT> backing_store;
+  TNode<JSArrayBuffer> array_buffer = ValidateIntegerTypedArray(
+      maybe_array, context, &elements_kind, &backing_store, &detached);
+  TNode<JSTypedArray> array = CAST(maybe_array);
 
-  Node* index_integer;
-  Node* index_word32 =
-      ConvertTaggedAtomicIndexToWord32(index, context, &index_integer);
-  ValidateAtomicIndex(array, index_word32, context);
-  TNode<UintPtrT> index_word = ChangeUint32ToWord(index_word32);
+  // 2. Let i be ? ValidateAtomicAccess(typedArray, index).
+  TNode<UintPtrT> index_word = ValidateAtomicAccess(array, index, context);
 
   Label u8(this), u16(this), u32(this), u64(this), other(this);
+
+  // 3. Let arrayTypeName be typedArray.[[TypedArrayName]].
+  // 4. If arrayTypeName is "BigUint64Array" or "BigInt64Array",
+  //    let v be ? ToBigInt(value).
   STATIC_ASSERT(BIGINT64_ELEMENTS > INT32_ELEMENTS);
   STATIC_ASSERT(BIGUINT64_ELEMENTS > INT32_ELEMENTS);
   GotoIf(Int32GreaterThan(elements_kind, Int32Constant(INT32_ELEMENTS)), &u64);
 
-  TNode<Number> value_integer = ToInteger_Inline(CAST(context), CAST(value));
+  // 5. Otherwise, let v be ? ToInteger(value).
+  TNode<Number> value_integer = ToInteger_Inline(context, value);
+
+  // 6. If IsDetachedBuffer(buffer) is true, throw a TypeError exception.
+  // 7. NOTE: The above check is not redundant with the check in
+  // ValidateIntegerTypedArray because the call to ToBigInt or ToInteger on the
+  // preceding lines can have arbitrary side effects, which could cause the
+  // buffer to become detached.
+  GotoIf(IsDetachedBuffer(array_buffer), &detached);
+
   TNode<Word32T> value_word32 = TruncateTaggedToWord32(context, value_integer);
 
-#if DEBUG
-  DebugSanityCheckAtomicIndex(array, index_word32, context);
-#endif
+  DebugSanityCheckAtomicIndex(array, index_word);
 
+  // Steps 8-13.
+  //
+  // (Not copied from ecma262 due to the axiomatic nature of the memory model.)
   int32_t case_values[] = {
       INT8_ELEMENTS,   UINT8_ELEMENTS, INT16_ELEMENTS,
       UINT16_ELEMENTS, INT32_ELEMENTS, UINT32_ELEMENTS,
@@ -284,17 +325,23 @@ TF_BUILTIN(AtomicsStore, SharedArrayBufferBuiltinsAssembler) {
 
   BIND(&u64);
 #if V8_TARGET_ARCH_MIPS && !_MIPS_ARCH_MIPS32R6
-  Return(CallRuntime(Runtime::kAtomicsStore64, context, array, index_integer,
+  TNode<Number> index_number = ChangeUintPtrToTagged(index_word);
+  Return(CallRuntime(Runtime::kAtomicsStore64, context, array, index_number,
                      value));
 #else
-  TNode<BigInt> value_bigint = ToBigInt(CAST(context), CAST(value));
-#if DEBUG
-  DebugSanityCheckAtomicIndex(array, index_word32, context);
-#endif
+  // 4. If arrayTypeName is "BigUint64Array" or "BigInt64Array",
+  //    let v be ? ToBigInt(value).
+  TNode<BigInt> value_bigint = ToBigInt(context, value);
+
+  // 6. If IsDetachedBuffer(buffer) is true, throw a TypeError exception.
+  GotoIf(IsDetachedBuffer(array_buffer), &detached);
+
+  DebugSanityCheckAtomicIndex(array, index_word);
+
   TVARIABLE(UintPtrT, var_low);
   TVARIABLE(UintPtrT, var_high);
   BigIntToRawBytes(value_bigint, &var_low, &var_high);
-  Node* high = Is64() ? nullptr : static_cast<Node*>(var_high.value());
+  TNode<UintPtrT> high = Is64() ? TNode<UintPtrT>() : var_high.value();
   AtomicStore(MachineRepresentation::kWord64, backing_store,
               WordShl(index_word, 3), var_low.value(), high);
   Return(value_bigint);
@@ -303,41 +350,68 @@ TF_BUILTIN(AtomicsStore, SharedArrayBufferBuiltinsAssembler) {
   // This shouldn't happen, we've already validated the type.
   BIND(&other);
   Unreachable();
+
+  BIND(&detached);
+  {
+    ThrowTypeError(context, MessageTemplate::kDetachedOperation,
+                   "Atomics.store");
+  }
 }
 
+// https://tc39.es/ecma262/#sec-atomics.exchange
 TF_BUILTIN(AtomicsExchange, SharedArrayBufferBuiltinsAssembler) {
-  Node* array = Parameter(Descriptor::kArray);
-  Node* index = Parameter(Descriptor::kIndex);
-  Node* value = Parameter(Descriptor::kValue);
-  Node* context = Parameter(Descriptor::kContext);
+  TNode<Object> maybe_array = CAST(Parameter(Descriptor::kArray));
+  TNode<Object> index = CAST(Parameter(Descriptor::kIndex));
+  TNode<Object> value = CAST(Parameter(Descriptor::kValue));
+  TNode<Context> context = CAST(Parameter(Descriptor::kContext));
 
+  // Inlines AtomicReadModifyWrite
+  // https://tc39.es/ecma262/#sec-atomicreadmodifywrite
+
+  // 1. Let buffer be ? ValidateIntegerTypedArray(typedArray).
+  Label detached(this);
   TNode<Int32T> elements_kind;
-  Node* backing_store;
-  ValidateSharedTypedArray(array, context, &elements_kind, &backing_store);
+  TNode<RawPtrT> backing_store;
+  TNode<JSArrayBuffer> array_buffer = ValidateIntegerTypedArray(
+      maybe_array, context, &elements_kind, &backing_store, &detached);
+  TNode<JSTypedArray> array = CAST(maybe_array);
 
-  Node* index_integer;
-  Node* index_word32 =
-      ConvertTaggedAtomicIndexToWord32(index, context, &index_integer);
-  ValidateAtomicIndex(array, index_word32, context);
+  // 2. Let i be ? ValidateAtomicAccess(typedArray, index).
+  TNode<UintPtrT> index_word = ValidateAtomicAccess(array, index, context);
 
 #if V8_TARGET_ARCH_MIPS || V8_TARGET_ARCH_MIPS64
-  Return(CallRuntime(Runtime::kAtomicsExchange, context, array, index_integer,
+  USE(array_buffer);
+  TNode<Number> index_number = ChangeUintPtrToTagged(index_word);
+  Return(CallRuntime(Runtime::kAtomicsExchange, context, array, index_number,
                      value));
 #else
-  TNode<UintPtrT> index_word = ChangeUint32ToWord(index_word32);
 
   Label i8(this), u8(this), i16(this), u16(this), i32(this), u32(this),
       i64(this), u64(this), big(this), other(this);
+
+  // 3. Let arrayTypeName be typedArray.[[TypedArrayName]].
+  // 4. If typedArray.[[ContentType]] is BigInt, let v be ? ToBigInt(value).
   STATIC_ASSERT(BIGINT64_ELEMENTS > INT32_ELEMENTS);
   STATIC_ASSERT(BIGUINT64_ELEMENTS > INT32_ELEMENTS);
   GotoIf(Int32GreaterThan(elements_kind, Int32Constant(INT32_ELEMENTS)), &big);
 
-  TNode<Number> value_integer = ToInteger_Inline(CAST(context), CAST(value));
-#if DEBUG
-  DebugSanityCheckAtomicIndex(array, index_word32, context);
-#endif
+  // 5. Otherwise, let v be ? ToInteger(value).
+  TNode<Number> value_integer = ToInteger_Inline(context, value);
+
+  // 6. If IsDetachedBuffer(buffer) is true, throw a TypeError exception.
+  // 7. NOTE: The above check is not redundant with the check in
+  // ValidateIntegerTypedArray because the call to ToBigInt or ToInteger on the
+  // preceding lines can have arbitrary side effects, which could cause the
+  // buffer to become detached.
+  GotoIf(IsDetachedBuffer(array_buffer), &detached);
+
+  DebugSanityCheckAtomicIndex(array, index_word);
+
   TNode<Word32T> value_word32 = TruncateTaggedToWord32(context, value_integer);
 
+  // Steps 8-12.
+  //
+  // (Not copied from ecma262 due to the axiomatic nature of the memory model.)
   int32_t case_values[] = {
       INT8_ELEMENTS,   UINT8_ELEMENTS, INT16_ELEMENTS,
       UINT16_ELEMENTS, INT32_ELEMENTS, UINT32_ELEMENTS,
@@ -375,14 +449,18 @@ TF_BUILTIN(AtomicsExchange, SharedArrayBufferBuiltinsAssembler) {
                      WordShl(index_word, 2), value_word32)));
 
   BIND(&big);
-  TNode<BigInt> value_bigint = ToBigInt(CAST(context), CAST(value));
-#if DEBUG
-  DebugSanityCheckAtomicIndex(array, index_word32, context);
-#endif
+  // 4. If typedArray.[[ContentType]] is BigInt, let v be ? ToBigInt(value).
+  TNode<BigInt> value_bigint = ToBigInt(context, value);
+
+  // 6. If IsDetachedBuffer(buffer) is true, throw a TypeError exception.
+  GotoIf(IsDetachedBuffer(array_buffer), &detached);
+
+  DebugSanityCheckAtomicIndex(array, index_word);
+
   TVARIABLE(UintPtrT, var_low);
   TVARIABLE(UintPtrT, var_high);
   BigIntToRawBytes(value_bigint, &var_low, &var_high);
-  Node* high = Is64() ? nullptr : static_cast<Node*>(var_high.value());
+  TNode<UintPtrT> high = Is64() ? TNode<UintPtrT>() : var_high.value();
   GotoIf(Word32Equal(elements_kind, Int32Constant(BIGINT64_ELEMENTS)), &i64);
   GotoIf(Word32Equal(elements_kind, Int32Constant(BIGUINT64_ELEMENTS)), &u64);
   Unreachable();
@@ -404,49 +482,74 @@ TF_BUILTIN(AtomicsExchange, SharedArrayBufferBuiltinsAssembler) {
   BIND(&other);
   Unreachable();
 #endif  // V8_TARGET_ARCH_MIPS || V8_TARGET_ARCH_MIPS64
+
+  BIND(&detached);
+  {
+    ThrowTypeError(context, MessageTemplate::kDetachedOperation,
+                   "Atomics.exchange");
+  }
 }
 
+// https://tc39.es/ecma262/#sec-atomics.compareexchange
 TF_BUILTIN(AtomicsCompareExchange, SharedArrayBufferBuiltinsAssembler) {
-  Node* array = Parameter(Descriptor::kArray);
-  Node* index = Parameter(Descriptor::kIndex);
-  Node* old_value = Parameter(Descriptor::kOldValue);
-  Node* new_value = Parameter(Descriptor::kNewValue);
-  Node* context = Parameter(Descriptor::kContext);
+  TNode<Object> maybe_array = CAST(Parameter(Descriptor::kArray));
+  TNode<Object> index = CAST(Parameter(Descriptor::kIndex));
+  TNode<Object> old_value = CAST(Parameter(Descriptor::kOldValue));
+  TNode<Object> new_value = CAST(Parameter(Descriptor::kNewValue));
+  TNode<Context> context = CAST(Parameter(Descriptor::kContext));
 
+  // 1. Let buffer be ? ValidateIntegerTypedArray(typedArray).
+  Label detached(this);
   TNode<Int32T> elements_kind;
-  Node* backing_store;
-  ValidateSharedTypedArray(array, context, &elements_kind, &backing_store);
+  TNode<RawPtrT> backing_store;
+  TNode<JSArrayBuffer> array_buffer = ValidateIntegerTypedArray(
+      maybe_array, context, &elements_kind, &backing_store, &detached);
+  TNode<JSTypedArray> array = CAST(maybe_array);
 
-  Node* index_integer;
-  Node* index_word32 =
-      ConvertTaggedAtomicIndexToWord32(index, context, &index_integer);
-  ValidateAtomicIndex(array, index_word32, context);
+  // 2. Let i be ? ValidateAtomicAccess(typedArray, index).
+  TNode<UintPtrT> index_word = ValidateAtomicAccess(array, index, context);
 
 #if V8_TARGET_ARCH_MIPS || V8_TARGET_ARCH_MIPS64 || V8_TARGET_ARCH_PPC64 || \
     V8_TARGET_ARCH_PPC || V8_TARGET_ARCH_S390 || V8_TARGET_ARCH_S390X
+  USE(array_buffer);
+  TNode<Number> index_number = ChangeUintPtrToTagged(index_word);
   Return(CallRuntime(Runtime::kAtomicsCompareExchange, context, array,
-                     index_integer, old_value, new_value));
+                     index_number, old_value, new_value));
 #else
-  TNode<UintPtrT> index_word = ChangeUint32ToWord(index_word32);
-
   Label i8(this), u8(this), i16(this), u16(this), i32(this), u32(this),
       i64(this), u64(this), big(this), other(this);
+
+  // 3. Let arrayTypeName be typedArray.[[TypedArrayName]].
+  // 4. If typedArray.[[ContentType]] is BigInt, then
+  //   a. Let expected be ? ToBigInt(expectedValue).
+  //   b. Let replacement be ? ToBigInt(replacementValue).
   STATIC_ASSERT(BIGINT64_ELEMENTS > INT32_ELEMENTS);
   STATIC_ASSERT(BIGUINT64_ELEMENTS > INT32_ELEMENTS);
   GotoIf(Int32GreaterThan(elements_kind, Int32Constant(INT32_ELEMENTS)), &big);
 
-  TNode<Number> old_value_integer =
-      ToInteger_Inline(CAST(context), CAST(old_value));
-  TNode<Number> new_value_integer =
-      ToInteger_Inline(CAST(context), CAST(new_value));
-#if DEBUG
-  DebugSanityCheckAtomicIndex(array, index_word32, context);
-#endif
+  // 5. Else,
+  //   a. Let expected be ? ToInteger(expectedValue).
+  //   b. Let replacement be ? ToInteger(replacementValue).
+  TNode<Number> old_value_integer = ToInteger_Inline(context, old_value);
+  TNode<Number> new_value_integer = ToInteger_Inline(context, new_value);
+
+  // 6. If IsDetachedBuffer(buffer) is true, throw a TypeError exception.
+  // 7. NOTE: The above check is not redundant with the check in
+  // ValidateIntegerTypedArray because the call to ToBigInt or ToInteger on the
+  // preceding lines can have arbitrary side effects, which could cause the
+  // buffer to become detached.
+  GotoIf(IsDetachedBuffer(array_buffer), &detached);
+
+  DebugSanityCheckAtomicIndex(array, index_word);
+
   TNode<Word32T> old_value_word32 =
       TruncateTaggedToWord32(context, old_value_integer);
   TNode<Word32T> new_value_word32 =
       TruncateTaggedToWord32(context, new_value_integer);
 
+  // Steps 8-14.
+  //
+  // (Not copied from ecma262 due to the axiomatic nature of the memory model.)
   int32_t case_values[] = {
       INT8_ELEMENTS,   UINT8_ELEMENTS, INT16_ELEMENTS,
       UINT16_ELEMENTS, INT32_ELEMENTS, UINT32_ELEMENTS,
@@ -488,19 +591,25 @@ TF_BUILTIN(AtomicsCompareExchange, SharedArrayBufferBuiltinsAssembler) {
       old_value_word32, new_value_word32)));
 
   BIND(&big);
-  TNode<BigInt> old_value_bigint = ToBigInt(CAST(context), CAST(old_value));
-  TNode<BigInt> new_value_bigint = ToBigInt(CAST(context), CAST(new_value));
-#if DEBUG
-  DebugSanityCheckAtomicIndex(array, index_word32, context);
-#endif
+  // 4. If typedArray.[[ContentType]] is BigInt, then
+  //   a. Let expected be ? ToBigInt(expectedValue).
+  //   b. Let replacement be ? ToBigInt(replacementValue).
+  TNode<BigInt> old_value_bigint = ToBigInt(context, old_value);
+  TNode<BigInt> new_value_bigint = ToBigInt(context, new_value);
+
+  // 6. If IsDetachedBuffer(buffer) is true, throw a TypeError exception.
+  GotoIf(IsDetachedBuffer(array_buffer), &detached);
+
+  DebugSanityCheckAtomicIndex(array, index_word);
+
   TVARIABLE(UintPtrT, var_old_low);
   TVARIABLE(UintPtrT, var_old_high);
   TVARIABLE(UintPtrT, var_new_low);
   TVARIABLE(UintPtrT, var_new_high);
   BigIntToRawBytes(old_value_bigint, &var_old_low, &var_old_high);
   BigIntToRawBytes(new_value_bigint, &var_new_low, &var_new_high);
-  Node* old_high = Is64() ? nullptr : static_cast<Node*>(var_old_high.value());
-  Node* new_high = Is64() ? nullptr : static_cast<Node*>(var_new_high.value());
+  TNode<UintPtrT> old_high = Is64() ? TNode<UintPtrT>() : var_old_high.value();
+  TNode<UintPtrT> new_high = Is64() ? TNode<UintPtrT>() : var_new_high.value();
   GotoIf(Word32Equal(elements_kind, Int32Constant(BIGINT64_ELEMENTS)), &i64);
   GotoIf(Word32Equal(elements_kind, Int32Constant(BIGUINT64_ELEMENTS)), &u64);
   Unreachable();
@@ -523,56 +632,84 @@ TF_BUILTIN(AtomicsCompareExchange, SharedArrayBufferBuiltinsAssembler) {
   Unreachable();
 #endif  // V8_TARGET_ARCH_MIPS || V8_TARGET_ARCH_MIPS64 || V8_TARGET_ARCH_PPC64
         // || V8_TARGET_ARCH_PPC || V8_TARGET_ARCH_S390 || V8_TARGET_ARCH_S390X
+
+  BIND(&detached);
+  {
+    ThrowTypeError(context, MessageTemplate::kDetachedOperation,
+                   "Atomics.store");
+  }
 }
 
-#define BINOP_BUILTIN(op)                                       \
-  TF_BUILTIN(Atomics##op, SharedArrayBufferBuiltinsAssembler) { \
-    Node* array = Parameter(Descriptor::kArray);                \
-    Node* index = Parameter(Descriptor::kIndex);                \
-    Node* value = Parameter(Descriptor::kValue);                \
-    Node* context = Parameter(Descriptor::kContext);            \
-    AtomicBinopBuiltinCommon(array, index, value, context,      \
-                             &CodeAssembler::Atomic##op,        \
-                             Runtime::kAtomics##op);            \
+#define BINOP_BUILTIN(op, method_name)                              \
+  TF_BUILTIN(Atomics##op, SharedArrayBufferBuiltinsAssembler) {     \
+    TNode<Object> array = CAST(Parameter(Descriptor::kArray));      \
+    TNode<Object> index = CAST(Parameter(Descriptor::kIndex));      \
+    TNode<Object> value = CAST(Parameter(Descriptor::kValue));      \
+    TNode<Context> context = CAST(Parameter(Descriptor::kContext)); \
+    AtomicBinopBuiltinCommon(array, index, value, context,          \
+                             &CodeAssembler::Atomic##op,            \
+                             Runtime::kAtomics##op, method_name);   \
   }
-BINOP_BUILTIN(Add)
-BINOP_BUILTIN(Sub)
-BINOP_BUILTIN(And)
-BINOP_BUILTIN(Or)
-BINOP_BUILTIN(Xor)
+// https://tc39.es/ecma262/#sec-atomics.add
+BINOP_BUILTIN(Add, "Atomics.add")
+// https://tc39.es/ecma262/#sec-atomics.sub
+BINOP_BUILTIN(Sub, "Atomics.sub")
+// https://tc39.es/ecma262/#sec-atomics.and
+BINOP_BUILTIN(And, "Atomics.and")
+// https://tc39.es/ecma262/#sec-atomics.or
+BINOP_BUILTIN(Or, "Atomics.or")
+// https://tc39.es/ecma262/#sec-atomics.xor
+BINOP_BUILTIN(Xor, "Atomics.xor")
 #undef BINOP_BUILTIN
 
+// https://tc39.es/ecma262/#sec-atomicreadmodifywrite
 void SharedArrayBufferBuiltinsAssembler::AtomicBinopBuiltinCommon(
-    Node* array, Node* index, Node* value, Node* context,
-    AssemblerFunction function, Runtime::FunctionId runtime_function) {
+    TNode<Object> maybe_array, TNode<Object> index, TNode<Object> value,
+    TNode<Context> context, AssemblerFunction function,
+    Runtime::FunctionId runtime_function, const char* method_name) {
+  // 1. Let buffer be ? ValidateIntegerTypedArray(typedArray).
+  Label detached(this);
   TNode<Int32T> elements_kind;
-  Node* backing_store;
-  ValidateSharedTypedArray(array, context, &elements_kind, &backing_store);
+  TNode<RawPtrT> backing_store;
+  TNode<JSArrayBuffer> array_buffer = ValidateIntegerTypedArray(
+      maybe_array, context, &elements_kind, &backing_store, &detached);
+  TNode<JSTypedArray> array = CAST(maybe_array);
 
-  Node* index_integer;
-  Node* index_word32 =
-      ConvertTaggedAtomicIndexToWord32(index, context, &index_integer);
-  ValidateAtomicIndex(array, index_word32, context);
+  // 2. Let i be ? ValidateAtomicAccess(typedArray, index).
+  TNode<UintPtrT> index_word = ValidateAtomicAccess(array, index, context);
 
 #if V8_TARGET_ARCH_MIPS || V8_TARGET_ARCH_MIPS64 || V8_TARGET_ARCH_PPC64 || \
     V8_TARGET_ARCH_PPC || V8_TARGET_ARCH_S390 || V8_TARGET_ARCH_S390X
-  Return(CallRuntime(runtime_function, context, array, index_integer, value));
+  USE(array_buffer);
+  TNode<Number> index_number = ChangeUintPtrToTagged(index_word);
+  Return(CallRuntime(runtime_function, context, array, index_number, value));
 #else
-  TNode<UintPtrT> index_word = ChangeUint32ToWord(index_word32);
-
   Label i8(this), u8(this), i16(this), u16(this), i32(this), u32(this),
       i64(this), u64(this), big(this), other(this);
 
+  // 3. Let arrayTypeName be typedArray.[[TypedArrayName]].
+  // 4. If typedArray.[[ContentType]] is BigInt, let v be ? ToBigInt(value).
   STATIC_ASSERT(BIGINT64_ELEMENTS > INT32_ELEMENTS);
   STATIC_ASSERT(BIGUINT64_ELEMENTS > INT32_ELEMENTS);
   GotoIf(Int32GreaterThan(elements_kind, Int32Constant(INT32_ELEMENTS)), &big);
 
-  TNode<Number> value_integer = ToInteger_Inline(CAST(context), CAST(value));
-#if DEBUG
-  DebugSanityCheckAtomicIndex(array, index_word32, context);
-#endif
+  // 5. Otherwise, let v be ? ToInteger(value).
+  TNode<Number> value_integer = ToInteger_Inline(context, value);
+
+  // 6. If IsDetachedBuffer(buffer) is true, throw a TypeError exception.
+  // 7. NOTE: The above check is not redundant with the check in
+  // ValidateIntegerTypedArray because the call to ToBigInt or ToInteger on the
+  // preceding lines can have arbitrary side effects, which could cause the
+  // buffer to become detached.
+  GotoIf(IsDetachedBuffer(array_buffer), &detached);
+
+  DebugSanityCheckAtomicIndex(array, index_word);
+
   TNode<Word32T> value_word32 = TruncateTaggedToWord32(context, value_integer);
 
+  // Steps 8-12.
+  //
+  // (Not copied from ecma262 due to the axiomatic nature of the memory model.)
   int32_t case_values[] = {
       INT8_ELEMENTS,   UINT8_ELEMENTS, INT16_ELEMENTS,
       UINT16_ELEMENTS, INT32_ELEMENTS, UINT32_ELEMENTS,
@@ -612,14 +749,18 @@ void SharedArrayBufferBuiltinsAssembler::AtomicBinopBuiltinCommon(
                         WordShl(index_word, 2), value_word32, nullptr)));
 
   BIND(&big);
-  TNode<BigInt> value_bigint = ToBigInt(CAST(context), CAST(value));
-#if DEBUG
-  DebugSanityCheckAtomicIndex(array, index_word32, context);
-#endif
+  // 4. If typedArray.[[ContentType]] is BigInt, let v be ? ToBigInt(value).
+  TNode<BigInt> value_bigint = ToBigInt(context, value);
+
+  // 6. If IsDetachedBuffer(buffer) is true, throw a TypeError exception.
+  GotoIf(IsDetachedBuffer(array_buffer), &detached);
+
+  DebugSanityCheckAtomicIndex(array, index_word);
+
   TVARIABLE(UintPtrT, var_low);
   TVARIABLE(UintPtrT, var_high);
   BigIntToRawBytes(value_bigint, &var_low, &var_high);
-  Node* high = Is64() ? nullptr : static_cast<Node*>(var_high.value());
+  TNode<UintPtrT> high = Is64() ? TNode<UintPtrT>() : var_high.value();
   GotoIf(Word32Equal(elements_kind, Int32Constant(BIGINT64_ELEMENTS)), &i64);
   GotoIf(Word32Equal(elements_kind, Int32Constant(BIGUINT64_ELEMENTS)), &u64);
   Unreachable();
@@ -642,6 +783,9 @@ void SharedArrayBufferBuiltinsAssembler::AtomicBinopBuiltinCommon(
   Unreachable();
 #endif  // V8_TARGET_ARCH_MIPS || V8_TARGET_ARCH_MIPS64 || V8_TARGET_ARCH_PPC64
         // || V8_TARGET_ARCH_PPC || V8_TARGET_ARCH_S390 || V8_TARGET_ARCH_S390X
+
+  BIND(&detached);
+  ThrowTypeError(context, MessageTemplate::kDetachedOperation, method_name);
 }
 
 }  // namespace internal
