@@ -38,7 +38,7 @@ namespace internal {
 //     // Tells whether key matches other.
 //     static bool IsMatch(Key key, Object other);
 //     // Returns the hash value for key.
-//     static uint32_t Hash(Isolate* isolate, Key key);
+//     static uint32_t Hash(ReadOnlyRoots roots, Key key);
 //     // Returns the hash value for object.
 //     static uint32_t HashForObject(ReadOnlyRoots roots, Object object);
 //     // Convert key to an object.
@@ -50,21 +50,17 @@ namespace internal {
 //     static const int kEntrySize = ..;
 //     // Indicates whether IsMatch can deal with other being the_hole (a
 //     // deleted entry).
-//     static const bool kNeedsHoleCheck = ..;
+//     static const bool kMatchNeedsHoleCheck = ..;
 //   };
 // The prefix size indicates an amount of memory in the
 // beginning of the backing storage that can be used for non-element
 // information by subclasses.
 
 template <typename KeyT>
-class BaseShape {
+class V8_EXPORT_PRIVATE BaseShape {
  public:
   using Key = KeyT;
-  static inline RootIndex GetMapRootIndex();
-  static const bool kNeedsHoleCheck = true;
   static Object Unwrap(Object key) { return key; }
-  static inline bool IsKey(ReadOnlyRoots roots, Object key);
-  static inline bool IsLive(ReadOnlyRoots roots, Object key);
 };
 
 class V8_EXPORT_PRIVATE HashTableBase : public NON_EXPORTED_BASE(FixedArray) {
@@ -78,6 +74,8 @@ class V8_EXPORT_PRIVATE HashTableBase : public NON_EXPORTED_BASE(FixedArray) {
   // Returns the capacity of the hash table.
   inline int Capacity() const;
 
+  inline InternalIndex::Range IterateEntries() const;
+
   // ElementAdded should be called whenever an element is added to a
   // hash table.
   inline void ElementAdded();
@@ -91,19 +89,11 @@ class V8_EXPORT_PRIVATE HashTableBase : public NON_EXPORTED_BASE(FixedArray) {
   // number of elements. May be more than HashTable::kMaxCapacity.
   static inline int ComputeCapacity(int at_least_space_for);
 
-  // Compute the probe offset (quadratic probing).
-  V8_INLINE static uint32_t GetProbeOffset(uint32_t n) {
-    return (n + n * n) >> 1;
-  }
-
   static const int kNumberOfElementsIndex = 0;
   static const int kNumberOfDeletedElementsIndex = 1;
   static const int kCapacityIndex = 2;
   static const int kPrefixStartIndex = 3;
 
-  // Constant used for denoting a absent entry.
-  static const int kNotFound = -1;
-
   // Minimum capacity for newly created hash tables.
   static const int kMinCapacity = 4;
 
@@ -115,18 +105,13 @@ class V8_EXPORT_PRIVATE HashTableBase : public NON_EXPORTED_BASE(FixedArray) {
   inline void SetNumberOfDeletedElements(int nod);
 
   // Returns probe entry.
-  static uint32_t GetProbe(uint32_t hash, uint32_t number, uint32_t size) {
-    DCHECK(base::bits::IsPowerOfTwo(size));
-    return (hash + GetProbeOffset(number)) & (size - 1);
+  inline static InternalIndex FirstProbe(uint32_t hash, uint32_t size) {
+    return InternalIndex(hash & (size - 1));
   }
 
-  inline static uint32_t FirstProbe(uint32_t hash, uint32_t size) {
-    return hash & (size - 1);
-  }
-
-  inline static uint32_t NextProbe(uint32_t last, uint32_t number,
-                                   uint32_t size) {
-    return (last + number) & (size - 1);
+  inline static InternalIndex NextProbe(InternalIndex last, uint32_t number,
+                                        uint32_t size) {
+    return InternalIndex((last.as_uint32() + number) & (size - 1));
   }
 
   OBJECT_CONSTRUCTORS(HashTableBase, FixedArray);
@@ -140,32 +125,39 @@ class EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) HashTable
   using Key = typename Shape::Key;
 
   // Returns a new HashTable object.
+  template <typename LocalIsolate>
   V8_WARN_UNUSED_RESULT static Handle<Derived> New(
-      Isolate* isolate, int at_least_space_for,
+      LocalIsolate* isolate, int at_least_space_for,
       AllocationType allocation = AllocationType::kYoung,
       MinimumCapacity capacity_option = USE_DEFAULT_MINIMUM_CAPACITY);
 
+  static inline Handle<Map> GetMap(ReadOnlyRoots roots);
+
   // Garbage collection support.
   void IteratePrefix(ObjectVisitor* visitor);
   void IterateElements(ObjectVisitor* visitor);
 
   // Find entry for key otherwise return kNotFound.
-  inline int FindEntry(ReadOnlyRoots roots, Key key, int32_t hash);
-  inline int FindEntry(Isolate* isolate, Key key);
+  template <typename LocalIsolate>
+  inline InternalIndex FindEntry(const LocalIsolate* isolate,
+                                 ReadOnlyRoots roots, Key key, int32_t hash);
+  template <typename LocalIsolate>
+  inline InternalIndex FindEntry(LocalIsolate* isolate, Key key);
 
   // Rehashes the table in-place.
-  void Rehash(ReadOnlyRoots roots);
+  void Rehash(const Isolate* isolate);
 
-  // Tells whether k is a real key.  The hole and undefined are not allowed
-  // as keys and can be used to indicate missing or deleted elements.
-  static bool IsKey(ReadOnlyRoots roots, Object k);
+  // Returns whether k is a real key.  The hole and undefined are not allowed as
+  // keys and can be used to indicate missing or deleted elements.
+  static inline bool IsKey(ReadOnlyRoots roots, Object k);
 
-  inline bool ToKey(ReadOnlyRoots roots, int entry, Object* out_k);
-  inline bool ToKey(Isolate* isolate, int entry, Object* out_k);
+  inline bool ToKey(ReadOnlyRoots roots, InternalIndex entry, Object* out_k);
+  inline bool ToKey(const Isolate* isolate, InternalIndex entry, Object* out_k);
 
   // Returns the key at entry.
-  inline Object KeyAt(int entry);
-  inline Object KeyAt(Isolate* isolate, int entry);
+  inline Object KeyAt(InternalIndex entry);
+  template <typename LocalIsolate>
+  inline Object KeyAt(const LocalIsolate* isolate, InternalIndex entry);
 
   static const int kElementsStartIndex = kPrefixStartIndex + Shape::kPrefixSize;
   static const int kEntrySize = Shape::kEntrySize;
@@ -182,16 +174,19 @@ class EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) HashTable
   // Don't shrink a HashTable below this capacity.
   static const int kMinShrinkCapacity = 16;
 
+  // Pretenure hashtables above this capacity.
+  static const int kMinCapacityForPretenure = 256;
+
   static const int kMaxRegularCapacity = kMaxRegularHeapObjectSize / 32;
 
   // Returns the index for an entry (of the key)
-  static constexpr inline int EntryToIndex(int entry) {
-    return (entry * kEntrySize) + kElementsStartIndex;
+  static constexpr inline int EntryToIndex(InternalIndex entry) {
+    return (entry.as_int() * kEntrySize) + kElementsStartIndex;
   }
 
-  // Returns the index for an entry (of the key)
-  static constexpr inline int IndexToEntry(int index) {
-    return (index - kElementsStartIndex) / kEntrySize;
+  // Returns the entry for an index (of the key)
+  static constexpr inline InternalIndex IndexToEntry(int index) {
+    return InternalIndex((index - kElementsStartIndex) / kEntrySize);
   }
 
   // Returns the index for a slot address in the object.
@@ -200,36 +195,57 @@ class EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) HashTable
   }
 
   // Ensure enough space for n additional elements.
+  template <typename LocalIsolate>
   V8_WARN_UNUSED_RESULT static Handle<Derived> EnsureCapacity(
-      Isolate* isolate, Handle<Derived> table, int n = 1,
+      LocalIsolate* isolate, Handle<Derived> table, int n = 1,
       AllocationType allocation = AllocationType::kYoung);
 
   // Returns true if this table has sufficient capacity for adding n elements.
   bool HasSufficientCapacityToAdd(int number_of_additional_elements);
 
+  // Returns true if a table with the given parameters has sufficient capacity
+  // for adding n elements. Can be used to check hypothetical capacities without
+  // actually allocating a table with that capacity.
+  static bool HasSufficientCapacityToAdd(int capacity, int number_of_elements,
+                                         int number_of_deleted_elements,
+                                         int number_of_additional_elements);
+
  protected:
   friend class ObjectHashTable;
 
+  template <typename LocalIsolate>
   V8_WARN_UNUSED_RESULT static Handle<Derived> NewInternal(
-      Isolate* isolate, int capacity, AllocationType allocation);
+      LocalIsolate* isolate, int capacity, AllocationType allocation);
 
   // Find the entry at which to insert element with the given key that
   // has the given hash value.
-  uint32_t FindInsertionEntry(uint32_t hash);
+  InternalIndex FindInsertionEntry(const Isolate* isolate, ReadOnlyRoots roots,
+                                   uint32_t hash);
+  InternalIndex FindInsertionEntry(Isolate* isolate, uint32_t hash);
 
-  // Attempt to shrink hash table after removal of key.
+  // Computes the capacity a table with the given capacity would need to have
+  // room for the given number of elements, also allowing it to shrink.
+  static int ComputeCapacityWithShrink(int current_capacity,
+                                       int at_least_room_for);
+
+  // Shrink the hash table.
   V8_WARN_UNUSED_RESULT static Handle<Derived> Shrink(
       Isolate* isolate, Handle<Derived> table, int additionalCapacity = 0);
 
+  // Rehashes this hash-table into the new table.
+  void Rehash(const Isolate* isolate, Derived new_table);
+
   inline void set_key(int index, Object value);
   inline void set_key(int index, Object value, WriteBarrierMode mode);
 
  private:
   // Ensure that kMaxRegularCapacity yields a non-large object dictionary.
-  STATIC_ASSERT(EntryToIndex(kMaxRegularCapacity) < kMaxRegularLength);
+  STATIC_ASSERT(EntryToIndex(InternalIndex(kMaxRegularCapacity)) <
+                kMaxRegularLength);
   STATIC_ASSERT(v8::base::bits::IsPowerOfTwo(kMaxRegularCapacity));
   static const int kMaxRegularEntry = kMaxRegularCapacity / kEntrySize;
-  static const int kMaxRegularIndex = EntryToIndex(kMaxRegularEntry);
+  static const int kMaxRegularIndex =
+      EntryToIndex(InternalIndex(kMaxRegularEntry));
   STATIC_ASSERT(OffsetOfElementAt(kMaxRegularIndex) <
                 kMaxRegularHeapObjectSize);
 
@@ -239,17 +255,32 @@ class EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) HashTable
   // Returns _expected_ if one of entries given by the first _probe_ probes is
   // equal to  _expected_. Otherwise, returns the entry given by the probe
   // number _probe_.
-  uint32_t EntryForProbe(ReadOnlyRoots roots, Object k, int probe,
-                         uint32_t expected);
+  InternalIndex EntryForProbe(ReadOnlyRoots roots, Object k, int probe,
+                              InternalIndex expected);
 
-  void Swap(uint32_t entry1, uint32_t entry2, WriteBarrierMode mode);
-
-  // Rehashes this hash-table into the new table.
-  void Rehash(ReadOnlyRoots roots, Derived new_table);
+  void Swap(InternalIndex entry1, InternalIndex entry2, WriteBarrierMode mode);
 
   OBJECT_CONSTRUCTORS(HashTable, HashTableBase);
 };
 
+#define EXTERN_DECLARE_HASH_TABLE(DERIVED, SHAPE)                            \
+  extern template class EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE)           \
+      HashTable<class DERIVED, SHAPE>;                                       \
+                                                                             \
+  extern template EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) Handle<DERIVED> \
+  HashTable<DERIVED, SHAPE>::New(Isolate*, int, AllocationType,              \
+                                 MinimumCapacity);                           \
+  extern template EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) Handle<DERIVED> \
+  HashTable<DERIVED, SHAPE>::New(LocalIsolate*, int, AllocationType,         \
+                                 MinimumCapacity);                           \
+                                                                             \
+  extern template EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) Handle<DERIVED> \
+  HashTable<DERIVED, SHAPE>::EnsureCapacity(Isolate*, Handle<DERIVED>, int,  \
+                                            AllocationType);                 \
+  extern template EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) Handle<DERIVED> \
+  HashTable<DERIVED, SHAPE>::EnsureCapacity(LocalIsolate*, Handle<DERIVED>,  \
+                                            int, AllocationType);
+
 // HashTableKey is an abstract superclass for virtual key behavior.
 class HashTableKey {
  public:
@@ -276,13 +307,13 @@ class HashTableKey {
 class ObjectHashTableShape : public BaseShape<Handle<Object>> {
  public:
   static inline bool IsMatch(Handle<Object> key, Object other);
-  static inline uint32_t Hash(Isolate* isolate, Handle<Object> key);
+  static inline uint32_t Hash(ReadOnlyRoots roots, Handle<Object> key);
   static inline uint32_t HashForObject(ReadOnlyRoots roots, Object object);
   static inline Handle<Object> AsHandle(Handle<Object> key);
   static const int kPrefixSize = 0;
   static const int kEntryValueIndex = 1;
   static const int kEntrySize = 2;
-  static const bool kNeedsHoleCheck = false;
+  static const bool kMatchNeedsHoleCheck = false;
 };
 
 template <typename Derived, typename Shape>
@@ -293,10 +324,10 @@ class EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) ObjectHashTableBase
   // returned in case the key is not present.
   Object Lookup(Handle<Object> key);
   Object Lookup(Handle<Object> key, int32_t hash);
-  Object Lookup(ReadOnlyRoots roots, Handle<Object> key, int32_t hash);
+  Object Lookup(const Isolate* isolate, Handle<Object> key, int32_t hash);
 
   // Returns the value at entry.
-  Object ValueAt(int entry);
+  Object ValueAt(InternalIndex entry);
 
   // Overwrite all keys and values with the hole value.
   static void FillEntriesWithHoles(Handle<Derived>);
@@ -316,24 +347,24 @@ class EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) ObjectHashTableBase
                                 int32_t hash);
 
   // Returns the index to the value of an entry.
-  static inline int EntryToValueIndex(int entry) {
+  static inline int EntryToValueIndex(InternalIndex entry) {
     return HashTable<Derived, Shape>::EntryToIndex(entry) +
            Shape::kEntryValueIndex;
   }
 
  protected:
-  void AddEntry(int entry, Object key, Object value);
-  void RemoveEntry(int entry);
+  void AddEntry(InternalIndex entry, Object key, Object value);
+  void RemoveEntry(InternalIndex entry);
 
   OBJECT_CONSTRUCTORS(ObjectHashTableBase, HashTable<Derived, Shape>);
 };
 
-class ObjectHashTable;
+#define EXTERN_DECLARE_OBJECT_BASE_HASH_TABLE(DERIVED, SHAPE)      \
+  EXTERN_DECLARE_HASH_TABLE(DERIVED, SHAPE)                        \
+  extern template class EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE) \
+      ObjectHashTableBase<class DERIVED, SHAPE>;
 
-extern template class EXPORT_TEMPLATE_DECLARE(
-    V8_EXPORT_PRIVATE) HashTable<ObjectHashTable, ObjectHashTableShape>;
-extern template class EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE)
-    ObjectHashTableBase<ObjectHashTable, ObjectHashTableShape>;
+EXTERN_DECLARE_OBJECT_BASE_HASH_TABLE(ObjectHashTable, ObjectHashTableShape)
 
 // ObjectHashTable maps keys that are arbitrary objects to object values by
 // using the identity hash of the key for hashing purposes.
@@ -348,25 +379,17 @@ class V8_EXPORT_PRIVATE ObjectHashTable
       ObjectHashTableBase<ObjectHashTable, ObjectHashTableShape>);
 };
 
-class EphemeronHashTableShape : public ObjectHashTableShape {
- public:
-  static inline RootIndex GetMapRootIndex();
-};
-
-class EphemeronHashTable;
-
-extern template class EXPORT_TEMPLATE_DECLARE(
-    V8_EXPORT_PRIVATE) HashTable<EphemeronHashTable, EphemeronHashTableShape>;
-extern template class EXPORT_TEMPLATE_DECLARE(V8_EXPORT_PRIVATE)
-    ObjectHashTableBase<EphemeronHashTable, EphemeronHashTableShape>;
+EXTERN_DECLARE_OBJECT_BASE_HASH_TABLE(EphemeronHashTable, ObjectHashTableShape)
 
 // EphemeronHashTable is similar to ObjectHashTable but gets special treatment
 // by the GC. The GC treats its entries as ephemerons: both key and value are
 // weak references, however if the key is strongly reachable its corresponding
 // value is also kept alive.
 class V8_EXPORT_PRIVATE EphemeronHashTable
-    : public ObjectHashTableBase<EphemeronHashTable, EphemeronHashTableShape> {
+    : public ObjectHashTableBase<EphemeronHashTable, ObjectHashTableShape> {
  public:
+  static inline Handle<Map> GetMap(ReadOnlyRoots roots);
+
   DECL_CAST(EphemeronHashTable)
   DECL_PRINTER(EphemeronHashTable)
   class BodyDescriptor;
@@ -374,14 +397,14 @@ class V8_EXPORT_PRIVATE EphemeronHashTable
  protected:
   friend class MarkCompactCollector;
   friend class ScavengerCollector;
-  friend class HashTable<EphemeronHashTable, EphemeronHashTableShape>;
-  friend class ObjectHashTableBase<EphemeronHashTable, EphemeronHashTableShape>;
+  friend class HashTable<EphemeronHashTable, ObjectHashTableShape>;
+  friend class ObjectHashTableBase<EphemeronHashTable, ObjectHashTableShape>;
   inline void set_key(int index, Object value);
   inline void set_key(int index, Object value, WriteBarrierMode mode);
 
   OBJECT_CONSTRUCTORS(
       EphemeronHashTable,
-      ObjectHashTableBase<EphemeronHashTable, EphemeronHashTableShape>);
+      ObjectHashTableBase<EphemeronHashTable, ObjectHashTableShape>);
 };
 
 class ObjectHashSetShape : public ObjectHashTableShape {
@@ -390,9 +413,7 @@ class ObjectHashSetShape : public ObjectHashTableShape {
   static const int kEntrySize = 1;
 };
 
-class ObjectHashSet;
-extern template class EXPORT_TEMPLATE_DECLARE(
-    V8_EXPORT_PRIVATE) HashTable<ObjectHashSet, ObjectHashSetShape>;
+EXTERN_DECLARE_HASH_TABLE(ObjectHashSet, ObjectHashSetShape)
 
 class V8_EXPORT_PRIVATE ObjectHashSet
     : public HashTable<ObjectHashSet, ObjectHashSetShape> {
