@@ -58,14 +58,19 @@ const {
   getOrSetAsyncId
 } = require('internal/async_hooks');
 const { IncomingMessage } = require('_http_incoming');
+const {
+  connResetException,
+  codes
+} = require('internal/errors');
 const {
   ERR_HTTP_REQUEST_TIMEOUT,
   ERR_HTTP_HEADERS_SENT,
   ERR_HTTP_INVALID_STATUS_CODE,
+  ERR_HTTP_SOCKET_ENCODING,
   ERR_INVALID_ARG_TYPE,
   ERR_INVALID_ARG_VALUE,
   ERR_INVALID_CHAR
-} = require('internal/errors').codes;
+} = codes;
 const {
   validateInteger,
   validateBoolean
@@ -79,10 +84,6 @@ const { observerCounts, constants } = internalBinding('performance');
 const { setTimeout, clearTimeout } = require('timers');
 const { NODE_PERFORMANCE_ENTRY_TYPE_HTTP } = constants;
 
-const dc = require('diagnostics_channel');
-const onRequestStartChannel = dc.channel('http.server.request.start');
-const onResponseFinishChannel = dc.channel('http.server.response.finish');
-
 const kServerResponse = Symbol('ServerResponse');
 const kServerResponseStatistics = Symbol('ServerResponseStatistics');
 
@@ -218,7 +219,11 @@ function onServerResponseClose() {
   // Ergo, we need to deal with stale 'close' events and handle the case
   // where the ServerResponse object has already been deconstructed.
   // Fortunately, that requires only a single if check. :-)
-  if (this._httpMessage) this._httpMessage.emit('close');
+  if (this._httpMessage) {
+    this._httpMessage.destroyed = true;
+    this._httpMessage._closed = true;
+    this._httpMessage.emit('close');
+  }
 }
 
 ServerResponse.prototype.assignSocket = function assignSocket(socket) {
@@ -487,6 +492,7 @@ function connectionListenerInternal(server, socket) {
   socket.on = generateSocketListenerWrapper('on');
   socket.addListener = generateSocketListenerWrapper('addListener');
   socket.prependListener = generateSocketListenerWrapper('prependListener');
+  socket.setEncoding = socketSetEncoding;
 
   // We only consume the socket if it has never been consumed before.
   if (socket._handle && socket._handle.isStreamBase &&
@@ -514,6 +520,10 @@ function connectionListenerInternal(server, socket) {
   socket._paused = false;
 }
 
+function socketSetEncoding() {
+  throw new ERR_HTTP_SOCKET_ENCODING();
+}
+
 function updateOutgoingData(socket, state, delta) {
   state.outgoingData += delta;
   socketOnDrain(socket, state);
@@ -561,8 +571,13 @@ function socketOnClose(socket, state) {
 function abortIncoming(incoming) {
   while (incoming.length) {
     const req = incoming.shift();
+    // TODO(ronag): req.destroy(err)
     req.aborted = true;
+    req.destroyed = true;
     req.emit('aborted');
+    if (req.listenerCount('error') > 0) {
+      req.emit('error', connResetException('aborted'));
+    }
     req.emit('close');
   }
   // Abort socket._httpMessage ?
@@ -722,11 +737,13 @@ function clearIncoming(req) {
   if (parser && parser.incoming === req) {
     if (req.readableEnded) {
       parser.incoming = null;
+      req.destroyed = true;
       req.emit('close');
     } else {
       req.on('end', clearIncoming);
     }
   } else {
+    req.destroyed = true;
     req.emit('close');
   }
 }
@@ -758,15 +775,6 @@ function clearRequestTimeout(req) {
 }
 
 function resOnFinish(req, res, socket, state, server) {
-  if (onResponseFinishChannel.hasSubscribers) {
-    onResponseFinishChannel.publish({
-      request: req,
-      response: res,
-      socket,
-      server
-    });
-  }
-
   // Usually the first incoming element should be our request.  it may
   // be that in the case abortIncoming() was called that the incoming
   // array will be empty.
@@ -782,7 +790,7 @@ function resOnFinish(req, res, socket, state, server) {
 
   // Make sure the requestTimeout is cleared before finishing.
   // This might occur if the application has sent a response
-  // without consuming the request body, which would have already
+  // without consuming the request body, which would have alredy
   // cleared the timer.
   // clearRequestTimeout can be executed even if the timer is not active,
   // so this is safe.
@@ -813,6 +821,8 @@ function resOnFinish(req, res, socket, state, server) {
 }
 
 function emitCloseNT(self) {
+  self.destroyed = true;
+  self._closed = true;
   self.emit('close');
 }
 
@@ -852,15 +862,6 @@ function parserOnIncoming(server, socket, state, req, keepAlive) {
   res.shouldKeepAlive = keepAlive;
   DTRACE_HTTP_SERVER_REQUEST(req, socket);
 
-  if (onRequestStartChannel.hasSubscribers) {
-    onRequestStartChannel.publish({
-      request: req,
-      response: res,
-      socket,
-      server
-    });
-  }
-
   if (socket._httpMessage) {
     // There are already pending outgoing res, append.
     state.outgoing.push(res);
