@@ -24,7 +24,6 @@
 const {
   Array,
   ArrayIsArray,
-  ArrayPrototypeForEach,
   Error,
   MathFloor,
   MathMin,
@@ -35,17 +34,11 @@ const {
   ObjectCreate,
   ObjectDefineProperties,
   ObjectDefineProperty,
+  ObjectGetOwnPropertyDescriptor,
+  ObjectGetPrototypeOf,
   ObjectSetPrototypeOf,
-  StringPrototypeCharCodeAt,
-  StringPrototypeReplace,
-  StringPrototypeSlice,
-  StringPrototypeToLowerCase,
-  StringPrototypeTrim,
   SymbolSpecies,
   SymbolToPrimitive,
-  TypedArrayPrototypeGetByteLength,
-  TypedArrayPrototypeFill,
-  TypedArrayPrototypeSet,
   Uint8Array,
   Uint8ArrayPrototype,
 } = primordials;
@@ -63,8 +56,7 @@ const {
   swap32: _swap32,
   swap64: _swap64,
   kMaxLength,
-  kStringMaxLength,
-  zeroFill: bindingZeroFill
+  kStringMaxLength
 } = internalBinding('buffer');
 const {
   getOwnNonIndexProperties,
@@ -95,7 +87,6 @@ const {
     ERR_INVALID_ARG_TYPE,
     ERR_INVALID_ARG_VALUE,
     ERR_INVALID_BUFFER_SIZE,
-    ERR_INVALID_OPT_VALUE,
     ERR_OUT_OF_RANGE,
     ERR_UNKNOWN_ENCODING
   },
@@ -113,9 +104,17 @@ const validateOffset = (value, name, min = 0, max = kMaxLength) =>
 const {
   FastBuffer,
   markAsUntransferable,
-  addBufferPrototypeMethods
+  addBufferPrototypeMethods,
+  createUnsafeBuffer
 } = require('internal/buffer');
 
+const TypedArrayPrototype = ObjectGetPrototypeOf(Uint8ArrayPrototype);
+
+const TypedArrayProto_byteLength =
+      ObjectGetOwnPropertyDescriptor(TypedArrayPrototype,
+                                     'byteLength').get;
+const TypedArrayFill = TypedArrayPrototype.fill;
+
 FastBuffer.prototype.constructor = Buffer;
 Buffer.prototype = FastBuffer.prototype;
 addBufferPrototypeMethods(Buffer.prototype);
@@ -136,25 +135,10 @@ const constants = ObjectDefineProperties({}, {
 Buffer.poolSize = 8 * 1024;
 let poolSize, poolOffset, allocPool;
 
-// A toggle used to access the zero fill setting of the array buffer allocator
-// in C++.
-// |zeroFill| can be undefined when running inside an isolate where we
-// do not own the ArrayBuffer allocator.  Zero fill is always on in that case.
-const zeroFill = bindingZeroFill || [0];
-
 const encodingsMap = ObjectCreate(null);
 for (let i = 0; i < encodings.length; ++i)
   encodingsMap[encodings[i]] = i;
 
-function createUnsafeBuffer(size) {
-  zeroFill[0] = 0;
-  try {
-    return new FastBuffer(size);
-  } finally {
-    zeroFill[0] = 1;
-  }
-}
-
 function createPool() {
   poolSize = Buffer.poolSize;
   allocPool = createUnsafeBuffer(poolSize).buffer;
@@ -252,14 +236,17 @@ function _copyActual(source, target, targetStart, sourceStart, sourceEnd) {
     sourceEnd = sourceStart + target.length - targetStart;
 
   let nb = sourceEnd - sourceStart;
+  const targetLen = target.length - targetStart;
   const sourceLen = source.length - sourceStart;
+  if (nb > targetLen)
+    nb = targetLen;
   if (nb > sourceLen)
     nb = sourceLen;
 
   if (sourceStart !== 0 || sourceEnd < source.length)
     source = new Uint8Array(source.buffer, source.byteOffset + sourceStart, nb);
 
-  TypedArrayPrototypeSet(target, source, targetStart);
+  target.set(source, targetStart);
 
   return nb;
 }
@@ -358,7 +345,7 @@ const assertSize = hideStackFrames((size) => {
     throw new ERR_INVALID_ARG_TYPE('size', 'number', size);
   }
   if (!(size >= 0 && size <= kMaxLength)) {
-    throw new ERR_INVALID_OPT_VALUE.RangeError('size', size);
+    throw new ERR_INVALID_ARG_VALUE.RangeError('size', size);
   }
 });
 
@@ -493,7 +480,7 @@ function fromArrayLike(obj) {
     if (obj.length > (poolSize - poolOffset))
       createPool();
     const b = new FastBuffer(allocPool, poolOffset, obj.length);
-    TypedArrayPrototypeSet(b, obj, 0);
+    b.set(obj, 0);
     poolOffset += obj.length;
     alignPool();
     return b;
@@ -579,7 +566,7 @@ Buffer.concat = function concat(list, length) {
     // Zero-fill the remaining bytes if the specified `length` was more than
     // the actual total length, i.e. if we have some remaining allocated bytes
     // there were not initialized.
-    TypedArrayPrototypeFill(buffer, 0, pos, length);
+    TypedArrayFill.call(buffer, 0, pos, length);
   }
 
   return buffer;
@@ -587,9 +574,9 @@ Buffer.concat = function concat(list, length) {
 
 function base64ByteLength(str, bytes) {
   // Handle padding
-  if (StringPrototypeCharCodeAt(str, bytes - 1) === 0x3D)
+  if (str.charCodeAt(bytes - 1) === 0x3D)
     bytes--;
-  if (bytes > 1 && StringPrototypeCharCodeAt(str, bytes - 1) === 0x3D)
+  if (bytes > 1 && str.charCodeAt(bytes - 1) === 0x3D)
     bytes--;
 
   // Base64 ratio: 3/4
@@ -679,7 +666,7 @@ function getEncodingOps(encoding) {
     case 4:
       if (encoding === 'utf8') return encodingOps.utf8;
       if (encoding === 'ucs2') return encodingOps.ucs2;
-      encoding = StringPrototypeToLowerCase(encoding);
+      encoding = encoding.toLowerCase();
       if (encoding === 'utf8') return encodingOps.utf8;
       if (encoding === 'ucs2') return encodingOps.ucs2;
       break;
@@ -687,32 +674,30 @@ function getEncodingOps(encoding) {
       if (encoding === 'utf-8') return encodingOps.utf8;
       if (encoding === 'ascii') return encodingOps.ascii;
       if (encoding === 'ucs-2') return encodingOps.ucs2;
-      encoding = StringPrototypeToLowerCase(encoding);
+      encoding = encoding.toLowerCase();
       if (encoding === 'utf-8') return encodingOps.utf8;
       if (encoding === 'ascii') return encodingOps.ascii;
       if (encoding === 'ucs-2') return encodingOps.ucs2;
       break;
     case 7:
-      if (encoding === 'utf16le' ||
-          StringPrototypeToLowerCase(encoding) === 'utf16le')
+      if (encoding === 'utf16le' || encoding.toLowerCase() === 'utf16le')
         return encodingOps.utf16le;
       break;
     case 8:
-      if (encoding === 'utf-16le' ||
-          StringPrototypeToLowerCase(encoding) === 'utf-16le')
+      if (encoding === 'utf-16le' || encoding.toLowerCase() === 'utf-16le')
         return encodingOps.utf16le;
       break;
     case 6:
       if (encoding === 'latin1' || encoding === 'binary')
         return encodingOps.latin1;
       if (encoding === 'base64') return encodingOps.base64;
-      encoding = StringPrototypeToLowerCase(encoding);
+      encoding = encoding.toLowerCase();
       if (encoding === 'latin1' || encoding === 'binary')
         return encodingOps.latin1;
       if (encoding === 'base64') return encodingOps.base64;
       break;
     case 3:
-      if (encoding === 'hex' || StringPrototypeToLowerCase(encoding) === 'hex')
+      if (encoding === 'hex' || encoding.toLowerCase() === 'hex')
         return encodingOps.hex;
       break;
   }
@@ -825,30 +810,28 @@ Buffer.prototype[customInspectSymbol] = function inspect(recurseTimes, ctx) {
   const max = INSPECT_MAX_BYTES;
   const actualMax = MathMin(max, this.length);
   const remaining = this.length - max;
-  let str = StringPrototypeTrim(StringPrototypeReplace(
-    this.hexSlice(0, actualMax), /(.{2})/g, '$1 '));
+  let str = this.hexSlice(0, actualMax).replace(/(.{2})/g, '$1 ').trim();
   if (remaining > 0)
     str += ` ... ${remaining} more byte${remaining > 1 ? 's' : ''}`;
   // Inspect special properties as well, if possible.
   if (ctx) {
     let extras = false;
     const filter = ctx.showHidden ? ALL_PROPERTIES : ONLY_ENUMERABLE;
-    const obj = ObjectCreate(null);
-    ArrayPrototypeForEach(getOwnNonIndexProperties(this, filter),
-                          (key) => {
-                            extras = true;
-                            obj[key] = this[key];
-                          });
+    const obj = getOwnNonIndexProperties(this, filter).reduce((obj, key) => {
+      extras = true;
+      obj[key] = this[key];
+      return obj;
+    }, ObjectCreate(null));
     if (extras) {
       if (this.length !== 0)
         str += ', ';
       // '[Object: null prototype] {'.length === 26
       // This is guarded with a test.
-      str += StringPrototypeSlice(utilInspect(obj, {
+      str += utilInspect(obj, {
         ...ctx,
         breakLength: Infinity,
         compact: true
-      }), 27, -2);
+      }).slice(27, -2);
     }
   }
   return `<${this.constructor.name} ${str}>`;
@@ -992,12 +975,12 @@ function _fill(buf, value, offset, end, encoding) {
     } else if (value.length === 1) {
       // Fast path: If `value` fits into a single byte, use that numeric value.
       if (normalizedEncoding === 'utf8') {
-        const code = StringPrototypeCharCodeAt(value, 0);
+        const code = value.charCodeAt(0);
         if (code < 128) {
           value = code;
         }
       } else if (normalizedEncoding === 'latin1') {
-        value = StringPrototypeCharCodeAt(value, 0);
+        value = value.charCodeAt(0);
       }
     }
   } else {
@@ -1022,12 +1005,12 @@ function _fill(buf, value, offset, end, encoding) {
 
   if (typeof value === 'number') {
     // OOB check
-    const byteLen = TypedArrayPrototypeGetByteLength(buf);
+    const byteLen = TypedArrayProto_byteLength.call(buf);
     const fillLength = end - offset;
     if (offset > end || fillLength + offset > byteLen)
       throw new ERR_BUFFER_OUT_OF_BOUNDS();
 
-    TypedArrayPrototypeFill(buf, value, offset, end);
+    TypedArrayFill.call(buf, value, offset, end);
   } else {
     const res = bindingFill(buf, value, offset, end, encoding);
     if (res < 0) {
@@ -1203,51 +1186,13 @@ if (internalBinding('config').hasIntl) {
   };
 }
 
-let DOMException;
-
-const lazyInvalidCharError = hideStackFrames((message, name) => {
-  if (DOMException === undefined)
-    DOMException = internalBinding('messaging').DOMException;
-  throw new DOMException('Invalid character', 'InvalidCharacterError');
-});
-
-function btoa(input) {
-  // The implementation here has not been performance optimized in any way and
-  // should not be.
-  // Refs: https://github.com/nodejs/node/pull/38433#issuecomment-828426932
-  input = `${input}`;
-  for (let n = 0; n < input.length; n++) {
-    if (input[n].charCodeAt(0) > 0xff)
-      lazyInvalidCharError();
-  }
-  const buf = Buffer.from(input, 'latin1');
-  return buf.toString('base64');
-}
-
-const kBase64Digits =
-  'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
-
-function atob(input) {
-  // The implementation here has not been performance optimized in any way and
-  // should not be.
-  // Refs: https://github.com/nodejs/node/pull/38433#issuecomment-828426932
-  input = `${input}`;
-  for (let n = 0; n < input.length; n++) {
-    if (!kBase64Digits.includes(input[n]))
-      lazyInvalidCharError();
-  }
-  return Buffer.from(input, 'base64').toString('latin1');
-}
-
 module.exports = {
   Buffer,
   SlowBuffer,
   transcode,
   // Legacy
   kMaxLength,
-  kStringMaxLength,
-  btoa,
-  atob,
+  kStringMaxLength
 };
 
 ObjectDefineProperties(module.exports, {
