@@ -22,20 +22,18 @@
 'use strict';
 
 const {
-  ArrayPrototypeSlice,
   Boolean,
   Error,
-  ErrorCaptureStackTrace,
   MathMin,
   NumberIsNaN,
   ObjectCreate,
   ObjectDefineProperty,
-  ObjectDefineProperties,
   ObjectGetPrototypeOf,
   ObjectSetPrototypeOf,
   Promise,
   PromiseReject,
   PromiseResolve,
+  ReflectApply,
   ReflectOwnKeys,
   String,
   Symbol,
@@ -67,9 +65,6 @@ const {
 
 const kCapture = Symbol('kCapture');
 const kErrorMonitor = Symbol('events.errorMonitor');
-const kMaxEventTargetListeners = Symbol('events.maxEventTargetListeners');
-const kMaxEventTargetListenersWarned =
-  Symbol('events.maxEventTargetListenersWarned');
 
 let DOMException;
 const lazyDOMException = hideStackFrames((message, name) => {
@@ -78,19 +73,13 @@ const lazyDOMException = hideStackFrames((message, name) => {
   return new DOMException(message, name);
 });
 
-
-/**
- * Creates a new `EventEmitter` instance.
- * @param {{ captureRejections?: boolean; }} [opts]
- * @returns {EventEmitter}
- */
 function EventEmitter(opts) {
   EventEmitter.init.call(this, opts);
 }
 module.exports = EventEmitter;
 module.exports.once = once;
 module.exports.on = on;
-module.exports.getEventListeners = getEventListeners;
+
 // Backwards-compat with node 0.10.x
 EventEmitter.EventEmitter = EventEmitter;
 
@@ -128,7 +117,6 @@ EventEmitter.prototype._maxListeners = undefined;
 // By default EventEmitters will print a warning if more than 10 listeners are
 // added to it. This is a useful default which helps finding memory leaks.
 let defaultMaxListeners = 10;
-let isEventTarget;
 
 function checkListener(listener) {
   if (typeof listener !== 'function') {
@@ -151,54 +139,6 @@ ObjectDefineProperty(EventEmitter, 'defaultMaxListeners', {
   }
 });
 
-ObjectDefineProperties(EventEmitter, {
-  kMaxEventTargetListeners: {
-    value: kMaxEventTargetListeners,
-    enumerable: false,
-    configurable: false,
-    writable: false,
-  },
-  kMaxEventTargetListenersWarned: {
-    value: kMaxEventTargetListenersWarned,
-    enumerable: false,
-    configurable: false,
-    writable: false,
-  }
-});
-
-/**
- * Sets the max listeners.
- * @param {number} n
- * @param {EventTarget[] | EventEmitter[]} [eventTargets]
- * @returns {void}
- */
-EventEmitter.setMaxListeners =
-  function(n = defaultMaxListeners, ...eventTargets) {
-    if (typeof n !== 'number' || n < 0 || NumberIsNaN(n))
-      throw new ERR_OUT_OF_RANGE('n', 'a non-negative number', n);
-    if (eventTargets.length === 0) {
-      defaultMaxListeners = n;
-    } else {
-      if (isEventTarget === undefined)
-        isEventTarget = require('internal/event_target').isEventTarget;
-
-      for (let i = 0; i < eventTargets.length; i++) {
-        const target = eventTargets[i];
-        if (isEventTarget(target)) {
-          target[kMaxEventTargetListeners] = n;
-          target[kMaxEventTargetListenersWarned] = false;
-        } else if (typeof target.setMaxListeners === 'function') {
-          target.setMaxListeners(n);
-        } else {
-          throw new ERR_INVALID_ARG_TYPE(
-            'eventTargets',
-            ['EventEmitter', 'EventTarget'],
-            target);
-        }
-      }
-    }
-  };
-
 EventEmitter.init = function(opts) {
 
   if (this._events === undefined ||
@@ -210,7 +150,7 @@ EventEmitter.init = function(opts) {
   this._maxListeners = this._maxListeners || undefined;
 
 
-  if (opts?.captureRejections) {
+  if (opts && opts.captureRejections) {
     if (typeof opts.captureRejections !== 'boolean') {
       throw new ERR_INVALID_ARG_TYPE('options.captureRejections',
                                      'boolean', opts.captureRejections);
@@ -266,11 +206,8 @@ function emitUnhandledRejectionOrErr(ee, err, type, args) {
   }
 }
 
-/**
- * Increases the max listeners of the event emitter.
- * @param {number} n
- * @returns {EventEmitter}
- */
+// Obviously not all Emitters should be limited to 10. This function allows
+// that to be increased. Set to zero for unlimited.
 EventEmitter.prototype.setMaxListeners = function setMaxListeners(n) {
   if (typeof n !== 'number' || n < 0 || NumberIsNaN(n)) {
     throw new ERR_OUT_OF_RANGE('n', 'a non-negative number', n);
@@ -285,10 +222,6 @@ function _getMaxListeners(that) {
   return that._maxListeners;
 }
 
-/**
- * Returns the current max listener value for the event emitter.
- * @returns {number}
- */
 EventEmitter.prototype.getMaxListeners = function getMaxListeners() {
   return _getMaxListeners(this);
 };
@@ -330,7 +263,7 @@ function enhanceStackTrace(err, own) {
   const errStack = err.stack.split('\n').slice(1);
   const ownStack = own.stack.split('\n').slice(1);
 
-  const { 0: len, 1: off } = identicalSequenceRange(ownStack, errStack);
+  const [ len, off ] = identicalSequenceRange(ownStack, errStack);
   if (len > 0) {
     ownStack.splice(off + 1, len - 2,
                     '    [... lines matching original stack trace ...]');
@@ -339,13 +272,6 @@ function enhanceStackTrace(err, own) {
   return err.stack + sep + ownStack.join('\n');
 }
 
-/**
- * Synchronously calls each of the listeners registered
- * for the event.
- * @param {string | symbol} type
- * @param {...any} [args]
- * @returns {boolean}
- */
 EventEmitter.prototype.emit = function emit(type, ...args) {
   let doError = (type === 'error');
 
@@ -365,7 +291,8 @@ EventEmitter.prototype.emit = function emit(type, ...args) {
     if (er instanceof Error) {
       try {
         const capture = {};
-        ErrorCaptureStackTrace(capture, EventEmitter.prototype.emit);
+        // eslint-disable-next-line no-restricted-syntax
+        Error.captureStackTrace(capture, EventEmitter.prototype.emit);
         ObjectDefineProperty(er, kEnhanceStackBeforeInspector, {
           value: enhanceStackTrace.bind(this, er, capture),
           configurable: true
@@ -397,7 +324,7 @@ EventEmitter.prototype.emit = function emit(type, ...args) {
     return false;
 
   if (typeof handler === 'function') {
-    const result = handler.apply(this, args);
+    const result = ReflectApply(handler, this, args);
 
     // We check if result is undefined first because that
     // is the most common case so we do not pay any perf
@@ -409,7 +336,7 @@ EventEmitter.prototype.emit = function emit(type, ...args) {
     const len = handler.length;
     const listeners = arrayClone(handler);
     for (let i = 0; i < len; ++i) {
-      const result = listeners[i].apply(this, args);
+      const result = ReflectApply(listeners[i], this, args);
 
       // We check if result is undefined first because that
       // is the most common case so we do not pay any perf
@@ -487,25 +414,12 @@ function _addListener(target, type, listener, prepend) {
   return target;
 }
 
-/**
- * Adds a listener to the event emitter.
- * @param {string | symbol} type
- * @param {Function} listener
- * @returns {EventEmitter}
- */
 EventEmitter.prototype.addListener = function addListener(type, listener) {
   return _addListener(this, type, listener, false);
 };
 
 EventEmitter.prototype.on = EventEmitter.prototype.addListener;
 
-/**
- * Adds the `listener` function to the beginning of
- * the listeners array.
- * @param {string | symbol} type
- * @param {Function} listener
- * @returns {EventEmitter}
- */
 EventEmitter.prototype.prependListener =
     function prependListener(type, listener) {
       return _addListener(this, type, listener, true);
@@ -529,12 +443,6 @@ function _onceWrap(target, type, listener) {
   return wrapped;
 }
 
-/**
- * Adds a one-time `listener` function to the event emitter.
- * @param {string | symbol} type
- * @param {Function} listener
- * @returns {EventEmitter}
- */
 EventEmitter.prototype.once = function once(type, listener) {
   checkListener(listener);
 
@@ -542,13 +450,6 @@ EventEmitter.prototype.once = function once(type, listener) {
   return this;
 };
 
-/**
- * Adds a one-time `listener` function to the beginning of
- * the listeners array.
- * @param {string | symbol} type
- * @param {Function} listener
- * @returns {EventEmitter}
- */
 EventEmitter.prototype.prependOnceListener =
     function prependOnceListener(type, listener) {
       checkListener(listener);
@@ -557,12 +458,7 @@ EventEmitter.prototype.prependOnceListener =
       return this;
     };
 
-/**
- * Removes the specified `listener` from the listeners array.
- * @param {string | symbol} type
- * @param {Function} listener
- * @returns {EventEmitter}
- */
+// Emits a 'removeListener' event if and only if the listener was removed.
 EventEmitter.prototype.removeListener =
     function removeListener(type, listener) {
       checkListener(listener);
@@ -616,13 +512,6 @@ EventEmitter.prototype.removeListener =
 
 EventEmitter.prototype.off = EventEmitter.prototype.removeListener;
 
-/**
- * Removes all listeners from the event emitter. (Only
- * removes listeners for a specific event name if specified
- * as `type`).
- * @param {string | symbol} [type]
- * @returns {EventEmitter}
- */
 EventEmitter.prototype.removeAllListeners =
     function removeAllListeners(type) {
       const events = this._events;
@@ -686,34 +575,14 @@ function _listeners(target, type, unwrap) {
     unwrapListeners(evlistener) : arrayClone(evlistener);
 }
 
-/**
- * Returns a copy of the array of listeners for the event name
- * specified as `type`.
- * @param {string | symbol} type
- * @returns {Function[]}
- */
 EventEmitter.prototype.listeners = function listeners(type) {
   return _listeners(this, type, true);
 };
 
-/**
- * Returns a copy of the array of listeners and wrappers for
- * the event name specified as `type`.
- * @param {string | symbol} type
- * @returns {Function[]}
- */
 EventEmitter.prototype.rawListeners = function rawListeners(type) {
   return _listeners(this, type, false);
 };
 
-/**
- * Returns the number of listeners listening to the event name
- * specified as `type`.
- * @deprecated since v3.2.0
- * @param {EventEmitter} emitter
- * @param {string | symbol} type
- * @returns {number}
- */
 EventEmitter.listenerCount = function(emitter, type) {
   if (typeof emitter.listenerCount === 'function') {
     return emitter.listenerCount(type);
@@ -722,13 +591,6 @@ EventEmitter.listenerCount = function(emitter, type) {
 };
 
 EventEmitter.prototype.listenerCount = listenerCount;
-
-/**
- * Returns the number of listeners listening to event name
- * specified as `type`.
- * @param {string | symbol} type
- * @returns {number}
- */
 function listenerCount(type) {
   const events = this._events;
 
@@ -745,11 +607,6 @@ function listenerCount(type) {
   return 0;
 }
 
-/**
- * Returns an array listing the events for which
- * the emitter has registered listeners.
- * @returns {any[]}
- */
 EventEmitter.prototype.eventNames = function eventNames() {
   return this._eventsCount > 0 ? ReflectOwnKeys(this._events) : [];
 };
@@ -764,7 +621,7 @@ function arrayClone(arr) {
     case 5: return [arr[0], arr[1], arr[2], arr[3], arr[4]];
     case 6: return [arr[0], arr[1], arr[2], arr[3], arr[4], arr[5]];
   }
-  return ArrayPrototypeSlice(arr);
+  return arr.slice();
 }
 
 function unwrapListeners(arr) {
@@ -777,57 +634,16 @@ function unwrapListeners(arr) {
   return ret;
 }
 
-/**
- * Returns a copy of the array of listeners for the event name
- * specified as `type`.
- * @param {EventEmitter | EventTarget} emitterOrTarget
- * @param {string | symbol} type
- * @returns {Function[]}
- */
-function getEventListeners(emitterOrTarget, type) {
-  // First check if EventEmitter
-  if (typeof emitterOrTarget.listeners === 'function') {
-    return emitterOrTarget.listeners(type);
-  }
-  // Require event target lazily to avoid always loading it
-  const { isEventTarget, kEvents } = require('internal/event_target');
-  if (isEventTarget(emitterOrTarget)) {
-    const root = emitterOrTarget[kEvents].get(type);
-    const listeners = [];
-    let handler = root?.next;
-    while (handler?.listener !== undefined) {
-      listeners.push(handler.listener);
-      handler = handler.next;
-    }
-    return listeners;
-  }
-  throw new ERR_INVALID_ARG_TYPE('emitter',
-                                 ['EventEmitter', 'EventTarget'],
-                                 emitterOrTarget);
-}
-
-/**
- * Creates a `Promise` that is fulfilled when the emitter
- * emits the given event.
- * @param {EventEmitter} emitter
- * @param {string} name
- * @param {{ signal: AbortSignal; }} [options]
- * @returns {Promise}
- */
 async function once(emitter, name, options = {}) {
-  const signal = options?.signal;
+  const signal = options ? options.signal : undefined;
   validateAbortSignal(signal, 'options.signal');
-  if (signal?.aborted)
+  if (signal && signal.aborted)
     throw lazyDOMException('The operation was aborted', 'AbortError');
   return new Promise((resolve, reject) => {
     const errorListener = (err) => {
       emitter.removeListener(name, resolver);
       if (signal != null) {
-        eventTargetAgnosticRemoveListener(
-          signal,
-          'abort',
-          abortListener,
-          { once: true });
+        eventTargetAgnosticRemoveListener(signal, 'abort', abortListener);
       }
       reject(err);
     };
@@ -836,38 +652,22 @@ async function once(emitter, name, options = {}) {
         emitter.removeListener('error', errorListener);
       }
       if (signal != null) {
-        eventTargetAgnosticRemoveListener(
-          signal,
-          'abort',
-          abortListener,
-          { once: true });
+        eventTargetAgnosticRemoveListener(signal, 'abort', abortListener);
       }
       resolve(args);
     };
     eventTargetAgnosticAddListener(emitter, name, resolver, { once: true });
-    if (name !== 'error') {
-      addErrorHandlerIfEventEmitter(emitter, errorListener, { once: true });
+    if (name !== 'error' && typeof emitter.once === 'function') {
+      emitter.once('error', errorListener);
     }
     function abortListener() {
-      if (typeof emitter.removeListener === 'function') {
-        emitter.removeListener(name, resolver);
-        emitter.removeListener('error', errorListener);
-      } else {
-        eventTargetAgnosticRemoveListener(
-          emitter,
-          name,
-          resolver,
-          { once: true });
-        eventTargetAgnosticRemoveListener(
-          emitter,
-          'error',
-          errorListener,
-          { once: true });
-      }
+      eventTargetAgnosticRemoveListener(emitter, name, resolver);
+      eventTargetAgnosticRemoveListener(emitter, 'error', resolver);
       reject(lazyDOMException('The operation was aborted', 'AbortError'));
     }
     if (signal != null) {
-      signal.addEventListener('abort', abortListener, { once: true });
+      eventTargetAgnosticAddListener(
+        signal, 'abort', abortListener, { once: true });
     }
   });
 }
@@ -879,12 +679,6 @@ function createIterResult(value, done) {
   return { value, done };
 }
 
-function addErrorHandlerIfEventEmitter(emitter, handler, flags) {
-  if (typeof emitter.on === 'function') {
-    eventTargetAgnosticAddListener(emitter, 'error', handler, flags);
-  }
-}
-
 function eventTargetAgnosticRemoveListener(emitter, name, listener, flags) {
   if (typeof emitter.removeListener === 'function') {
     emitter.removeListener(name, listener);
@@ -897,7 +691,7 @@ function eventTargetAgnosticRemoveListener(emitter, name, listener, flags) {
 
 function eventTargetAgnosticAddListener(emitter, name, listener, flags) {
   if (typeof emitter.on === 'function') {
-    if (flags?.once) {
+    if (flags && flags.once) {
       emitter.once(name, listener);
     } else {
       emitter.on(name, listener);
@@ -911,17 +705,10 @@ function eventTargetAgnosticAddListener(emitter, name, listener, flags) {
   }
 }
 
-/**
- * Returns an `AsyncIterator` that iterates `event` events.
- * @param {EventEmitter} emitter
- * @param {string | symbol} event
- * @param {{ signal: AbortSignal; }} [options]
- * @returns {AsyncIterator}
- */
 function on(emitter, event, options) {
-  const signal = options?.signal;
+  const { signal } = { ...options };
   validateAbortSignal(signal, 'options.signal');
-  if (signal?.aborted) {
+  if (signal && signal.aborted) {
     throw lazyDOMException('The operation was aborted', 'AbortError');
   }
 
@@ -996,8 +783,8 @@ function on(emitter, event, options) {
   }, AsyncIteratorPrototype);
 
   eventTargetAgnosticAddListener(emitter, event, eventHandler);
-  if (event !== 'error') {
-    addErrorHandlerIfEventEmitter(emitter, errorHandler);
+  if (event !== 'error' && typeof emitter.on === 'function') {
+    emitter.on('error', errorHandler);
   }
 
   if (signal) {
