@@ -24,6 +24,10 @@
 
 'use strict';
 
+// Most platforms don't allow reads or writes >= 2 GB.
+// See https://github.com/libuv/libuv/pull/1501.
+const kIoMaxLength = 2 ** 31 - 1;
+
 // When using FSReqCallback, make sure to create the object only *after* all
 // parameter validation has happened, so that the objects are not kept in memory
 // in case they are created but never used due to an exception.
@@ -32,6 +36,7 @@ const {
   Map,
   MathMax,
   Number,
+  NumberIsSafeInteger,
   ObjectCreate,
   ObjectDefineProperties,
   ObjectDefineProperty,
@@ -64,10 +69,8 @@ const {
     ERR_INVALID_ARG_VALUE,
     ERR_INVALID_ARG_TYPE,
     ERR_INVALID_CALLBACK,
-    ERR_FEATURE_UNAVAILABLE_ON_PLATFORM,
+    ERR_FEATURE_UNAVAILABLE_ON_PLATFORM
   },
-  hideStackFrames,
-  uvErrmapGet,
   uvException
 } = require('internal/errors');
 
@@ -75,10 +78,6 @@ const { FSReqCallback, statValues } = binding;
 const { toPathIfFileURL } = require('internal/url');
 const internalUtil = require('internal/util');
 const {
-  constants: {
-    kIoMaxLength,
-    kMaxUserId,
-  },
   copyObject,
   Dirent,
   getDirents,
@@ -98,7 +97,6 @@ const {
   validateOffsetLengthRead,
   validateOffsetLengthWrite,
   validatePath,
-  validatePosition,
   validateRmOptions,
   validateRmOptionsSync,
   validateRmdirOptions,
@@ -119,9 +117,10 @@ const {
   parseFileMode,
   validateBuffer,
   validateInteger,
-  validateInt32,
-  validateString,
+  validateInt32
 } = require('internal/validators');
+// 2 ** 32 - 1
+const kMaxUserId = 4294967295;
 
 let truncateWarn = true;
 let fs;
@@ -134,13 +133,6 @@ let ReadStream;
 let WriteStream;
 let rimraf;
 let rimrafSync;
-let DOMException;
-
-const lazyDOMException = hideStackFrames((message, name) => {
-  if (DOMException === undefined)
-    DOMException = internalBinding('messaging').DOMException;
-  return new DOMException(message, name);
-});
 
 // These have to be separate because of how graceful-fs happens to do it's
 // monkeypatching.
@@ -323,9 +315,6 @@ function readFile(path, options, callback) {
   const context = new ReadFileContext(callback, options.encoding);
   context.isUserFd = isFd(path); // File descriptor ownership
 
-  if (options.signal) {
-    context.signal = options.signal;
-  }
   if (context.isUserFd) {
     process.nextTick(function tick(context) {
       readFileAfterOpen.call({ context }, null, path);
@@ -333,11 +322,6 @@ function readFile(path, options, callback) {
     return;
   }
 
-  if (options.signal?.aborted) {
-    callback(lazyDOMException('The operation was aborted', 'AbortError'));
-    return;
-  }
-
   const flagsNumber = stringToFlags(options.flag);
   path = getValidatedPath(path);
 
@@ -438,14 +422,9 @@ function readFileSync(path, options) {
   return buffer;
 }
 
-function defaultCloseCallback(err) {
-  if (err != null) throw err;
-}
-
-function close(fd, callback = defaultCloseCallback) {
+function close(fd, callback) {
   validateInt32(fd, 'fd', 0);
-  if (callback !== defaultCloseCallback)
-    callback = makeCallback(callback);
+  callback = makeCallback(callback);
 
   const req = new FSReqCallback();
   req.oncomplete = callback;
@@ -523,7 +502,7 @@ function read(fd, buffer, offset, length, position, callback) {
     ({
       buffer = Buffer.alloc(16384),
       offset = 0,
-      length = buffer.byteLength,
+      length = buffer.length,
       position
     } = options);
   }
@@ -534,7 +513,7 @@ function read(fd, buffer, offset, length, position, callback) {
   if (offset == null) {
     offset = 0;
   } else {
-    validateInteger(offset, 'offset', 0);
+    validateInteger(offset, 'offset');
   }
 
   length |= 0;
@@ -552,11 +531,9 @@ function read(fd, buffer, offset, length, position, callback) {
 
   validateOffsetLengthRead(offset, length, buffer.byteLength);
 
-  if (position == null)
+  if (!NumberIsSafeInteger(position))
     position = -1;
 
-  validatePosition(position, 'position');
-
   function wrapper(err, bytesRead) {
     // Retain a reference to buffer so that it can't be GC'ed too soon.
     callback(err, bytesRead || 0, buffer);
@@ -578,19 +555,19 @@ ObjectDefineProperty(read, internalUtil.customPromisifyArgs,
 function readSync(fd, buffer, offset, length, position) {
   validateInt32(fd, 'fd', 0);
 
-  validateBuffer(buffer);
-
   if (arguments.length <= 3) {
     // Assume fs.read(fd, buffer, options)
     const options = offset || {};
 
-    ({ offset = 0, length = buffer.byteLength, position } = options);
+    ({ offset = 0, length = buffer.length, position } = options);
   }
 
+  validateBuffer(buffer);
+
   if (offset == null) {
     offset = 0;
   } else {
-    validateInteger(offset, 'offset', 0);
+    validateInteger(offset, 'offset');
   }
 
   length |= 0;
@@ -606,11 +583,9 @@ function readSync(fd, buffer, offset, length, position) {
 
   validateOffsetLengthRead(offset, length, buffer.byteLength);
 
-  if (position == null)
+  if (!NumberIsSafeInteger(position))
     position = -1;
 
-  validatePosition(position, 'position');
-
   const ctx = {};
   const result = binding.read(fd, buffer, offset, length, position,
                               undefined, ctx);
@@ -670,10 +645,10 @@ function write(fd, buffer, offset, length, position, callback) {
     if (offset == null || typeof offset === 'function') {
       offset = 0;
     } else {
-      validateInteger(offset, 'offset', 0);
+      validateInteger(offset, 'offset');
     }
     if (typeof length !== 'number')
-      length = buffer.byteLength - offset;
+      length = buffer.length - offset;
     if (typeof position !== 'number')
       position = null;
     validateOffsetLengthWrite(offset, length, buffer.byteLength);
@@ -718,7 +693,7 @@ function writeSync(fd, buffer, offset, length, position) {
     if (offset == null) {
       offset = 0;
     } else {
-      validateInteger(offset, 'offset', 0);
+      validateInteger(offset, 'offset');
     }
     if (typeof length !== 'number')
       length = buffer.byteLength - offset;
@@ -810,7 +785,6 @@ function truncate(path, len, callback) {
   }
 
   validateInteger(len, 'len');
-  len = MathMax(0, len);
   callback = maybeCallback(callback);
   fs.open(path, 'r+', (er, fd) => {
     if (er) return callback(er);
@@ -885,14 +859,18 @@ function rmdir(path, options, callback) {
   path = pathModule.toNamespacedPath(getValidatedPath(path));
 
   if (options && options.recursive) {
-    validateRmOptions(path, { ...options, force: true }, (err, options) => {
-      if (err) {
-        return callback(err);
-      }
+    options = validateRmOptions(
+      path,
+      { ...options, force: true },
+      true,
+      (err, options) => {
+        if (err) {
+          return callback(err);
+        }
 
-      lazyLoadRimraf();
-      return rimraf(path, options, callback);
-    });
+        lazyLoadRimraf();
+        return rimraf(path, options, callback);
+      });
   } else {
     validateRmdirOptions(options);
     const req = new FSReqCallback();
@@ -905,7 +883,7 @@ function rmdirSync(path, options) {
   path = getValidatedPath(path);
 
   if (options && options.recursive) {
-    options = validateRmOptionsSync(path, { ...options, force: true });
+    options = validateRmOptionsSync(path, { ...options, force: true }, true);
     lazyLoadRimraf();
     return rimrafSync(pathModule.toNamespacedPath(path), options);
   }
@@ -922,7 +900,7 @@ function rm(path, options, callback) {
     options = undefined;
   }
 
-  validateRmOptions(path, options, (err, options) => {
+  validateRmOptions(path, options, false, (err, options) => {
     if (err) {
       return callback(err);
     }
@@ -932,7 +910,7 @@ function rm(path, options, callback) {
 }
 
 function rmSync(path, options) {
-  options = validateRmOptionsSync(path, options);
+  options = validateRmOptionsSync(path, options, false);
 
   lazyLoadRimraf();
   return rimrafSync(pathModule.toNamespacedPath(path), options);
@@ -1087,20 +1065,7 @@ function stat(path, options = { bigint: false }, callback) {
   binding.stat(pathModule.toNamespacedPath(path), options.bigint, req);
 }
 
-function hasNoEntryError(ctx) {
-  if (ctx.errno) {
-    const uvErr = uvErrmapGet(ctx.errno);
-    return uvErr && uvErr[0] === 'ENOENT';
-  }
-
-  if (ctx.error) {
-    return ctx.error.code === 'ENOENT';
-  }
-
-  return false;
-}
-
-function fstatSync(fd, options = { bigint: false, throwIfNoEntry: true }) {
+function fstatSync(fd, options = { bigint: false }) {
   validateInt32(fd, 'fd', 0);
   const ctx = { fd };
   const stats = binding.fstat(fd, options.bigint, undefined, ctx);
@@ -1108,26 +1073,20 @@ function fstatSync(fd, options = { bigint: false, throwIfNoEntry: true }) {
   return getStatsFromBinding(stats);
 }
 
-function lstatSync(path, options = { bigint: false, throwIfNoEntry: true }) {
+function lstatSync(path, options = { bigint: false }) {
   path = getValidatedPath(path);
   const ctx = { path };
   const stats = binding.lstat(pathModule.toNamespacedPath(path),
                               options.bigint, undefined, ctx);
-  if (options.throwIfNoEntry === false && hasNoEntryError(ctx)) {
-    return undefined;
-  }
   handleErrorFromBinding(ctx);
   return getStatsFromBinding(stats);
 }
 
-function statSync(path, options = { bigint: false, throwIfNoEntry: true }) {
+function statSync(path, options = { bigint: false }) {
   path = getValidatedPath(path);
   const ctx = { path };
   const stats = binding.stat(pathModule.toNamespacedPath(path),
                              options.bigint, undefined, ctx);
-  if (options.throwIfNoEntry === false && hasNoEntryError(ctx)) {
-    return undefined;
-  }
   handleErrorFromBinding(ctx);
   return getStatsFromBinding(stats);
 }
@@ -1447,17 +1406,7 @@ function lutimesSync(path, atime, mtime) {
   handleErrorFromBinding(ctx);
 }
 
-function writeAll(fd, isUserFd, buffer, offset, length, signal, callback) {
-  if (signal?.aborted) {
-    if (isUserFd) {
-      callback(lazyDOMException('The operation was aborted', 'AbortError'));
-    } else {
-      fs.close(fd, function() {
-        callback(lazyDOMException('The operation was aborted', 'AbortError'));
-      });
-    }
-    return;
-  }
+function writeAll(fd, isUserFd, buffer, offset, length, callback) {
   // write(fd, buffer, offset, length, position, callback)
   fs.write(fd, buffer, offset, length, null, (writeErr, written) => {
     if (writeErr) {
@@ -1477,7 +1426,7 @@ function writeAll(fd, isUserFd, buffer, offset, length, signal, callback) {
     } else {
       offset += written;
       length -= written;
-      writeAll(fd, isUserFd, buffer, offset, length, signal, callback);
+      writeAll(fd, isUserFd, buffer, offset, length, callback);
     }
   });
 }
@@ -1494,22 +1443,16 @@ function writeFile(path, data, options, callback) {
 
   if (isFd(path)) {
     const isUserFd = true;
-    const signal = options.signal;
-    writeAll(path, isUserFd, data, 0, data.byteLength, signal, callback);
+    writeAll(path, isUserFd, data, 0, data.byteLength, callback);
     return;
   }
 
-  if (options.signal?.aborted) {
-    callback(lazyDOMException('The operation was aborted', 'AbortError'));
-    return;
-  }
   fs.open(path, flag, options.mode, (openErr, fd) => {
     if (openErr) {
       callback(openErr);
     } else {
       const isUserFd = false;
-      const signal = options.signal;
-      writeAll(fd, isUserFd, data, 0, data.byteLength, signal, callback);
+      writeAll(fd, isUserFd, data, 0, data.byteLength, callback);
     }
   });
 }
@@ -1591,17 +1534,6 @@ function watch(filename, options, listener) {
   if (listener) {
     watcher.addListener('change', listener);
   }
-  if (options.signal) {
-    if (options.signal.aborted) {
-      process.nextTick(() => watcher.close());
-    } else {
-      const listener = () => watcher.close();
-      options.signal.addEventListener('abort', listener);
-      watcher.once('close', () => {
-        options.signal.removeEventListener('abort', listener);
-      });
-    }
-  }
 
   return watcher;
 }
@@ -2001,8 +1933,9 @@ realpath.native = (path, options, callback) => {
 function mkdtemp(prefix, options, callback) {
   callback = makeCallback(typeof options === 'function' ? options : callback);
   options = getOptions(options, {});
-
-  validateString(prefix, 'prefix');
+  if (!prefix || typeof prefix !== 'string') {
+    throw new ERR_INVALID_ARG_TYPE('prefix', 'string', prefix);
+  }
   nullCheck(prefix, 'prefix');
   warnOnNonPortableTemplate(prefix);
   const req = new FSReqCallback();
@@ -2013,8 +1946,9 @@ function mkdtemp(prefix, options, callback) {
 
 function mkdtempSync(prefix, options) {
   options = getOptions(options, {});
-
-  validateString(prefix, 'prefix');
+  if (!prefix || typeof prefix !== 'string') {
+    throw new ERR_INVALID_ARG_TYPE('prefix', 'string', prefix);
+  }
   nullCheck(prefix, 'prefix');
   warnOnNonPortableTemplate(prefix);
   const path = `${prefix}XXXXXX`;
@@ -2064,8 +1998,7 @@ function copyFileSync(src, dest, mode) {
 function lazyLoadStreams() {
   if (!ReadStream) {
     ({ ReadStream, WriteStream } = require('internal/fs/streams'));
-    FileReadStream = ReadStream;
-    FileWriteStream = WriteStream;
+    [ FileReadStream, FileWriteStream ] = [ ReadStream, WriteStream ];
   }
 }
 
