@@ -7,7 +7,7 @@
 // This file is expected not to perform any asynchronous operations itself
 // when being executed - those should be done in either
 // `lib/internal/bootstrap/pre_execution.js` or in main scripts. The majority
-// of the code here focus on setting up the global proxy and the process
+// of the code here focuses on setting up the global proxy and the process
 // object in a synchronous manner.
 // As special caution is given to the performance of the startup process,
 // many dependencies are invoked lazily.
@@ -34,7 +34,7 @@
 
 // This file is compiled as if it's wrapped in a function with arguments
 // passed by node::RunBootstrapping()
-/* global process, require, internalBinding */
+/* global process, require, internalBinding, primordials */
 
 setupPrepareStackTrace();
 
@@ -57,7 +57,9 @@ process.domain = null;
 process._exiting = false;
 
 // process.config is serialized config.gypi
-process.config = JSONParse(internalBinding('native_module').config);
+const nativeModule = internalBinding('native_module');
+process.config = JSONParse(nativeModule.config);
+require('internal/worker/js_transferable').setup();
 
 // Bootstrappers for all threads, including worker threads and main thread
 const perThreadSetup = require('internal/process/per_thread');
@@ -78,8 +80,6 @@ const rawMethods = internalBinding('process_methods');
 
   const wrapped = perThreadSetup.wrapProcessMethods(rawMethods);
   process._rawDebug = wrapped._rawDebug;
-  process.hrtime = wrapped.hrtime;
-  process.hrtime.bigint = wrapped.hrtimeBigInt;
   process.cpuUsage = wrapped.cpuUsage;
   process.resourceUsage = wrapped.resourceUsage;
   process.memoryUsage = wrapped.memoryUsage;
@@ -133,6 +133,28 @@ if (!config.noBrowserGlobals) {
   // https://encoding.spec.whatwg.org/#textdecoder
   exposeInterface(global, 'TextDecoder', TextDecoder);
 
+  const {
+    AbortController,
+    AbortSignal,
+  } = require('internal/abort_controller');
+  exposeInterface(global, 'AbortController', AbortController);
+  exposeInterface(global, 'AbortSignal', AbortSignal);
+
+  const {
+    EventTarget,
+    Event,
+  } = require('internal/event_target');
+  exposeInterface(global, 'EventTarget', EventTarget);
+  exposeInterface(global, 'Event', Event);
+  const {
+    MessageChannel,
+    MessagePort,
+    MessageEvent,
+  } = require('internal/worker/io');
+  exposeInterface(global, 'MessageChannel', MessageChannel);
+  exposeInterface(global, 'MessagePort', MessagePort);
+  exposeInterface(global, 'MessageEvent', MessageEvent);
+
   // https://html.spec.whatwg.org/multipage/webappapis.html#windoworworkerglobalscope
   const timers = require('timers');
   defineOperation(global, 'clearInterval', timers.clearInterval);
@@ -184,21 +206,28 @@ process.assert = deprecate(
 // TODO(joyeecheung): this property has not been well-maintained, should we
 // deprecate it in favor of a better API?
 const { isDebugBuild, hasOpenSSL, hasInspector } = config;
+const features = {
+  inspector: hasInspector,
+  debug: isDebugBuild,
+  uv: true,
+  ipv6: true,  // TODO(bnoordhuis) ping libuv
+  tls_alpn: hasOpenSSL,
+  tls_sni: hasOpenSSL,
+  tls_ocsp: hasOpenSSL,
+  tls: hasOpenSSL,
+  // This needs to be dynamic because snapshot is built without code cache.
+  // TODO(joyeecheung): https://github.com/nodejs/node/issues/31074
+  // Make it possible to build snapshot with code cache
+  get cached_builtins() {
+    return nativeModule.hasCachedBuiltins();
+  }
+};
+
 ObjectDefineProperty(process, 'features', {
   enumerable: true,
   writable: false,
   configurable: false,
-  value: {
-    inspector: hasInspector,
-    debug: isDebugBuild,
-    uv: true,
-    ipv6: true,  // TODO(bnoordhuis) ping libuv
-    tls_alpn: hasOpenSSL,
-    tls_sni: hasOpenSSL,
-    tls_ocsp: hasOpenSSL,
-    tls: hasOpenSSL,
-    cached_builtins: config.hasCachedBuiltins,
-  }
+  value: features
 });
 
 {
