@@ -4,40 +4,56 @@ const {
   ObjectSetPrototypeOf,
 } = primordials;
 
-const {
-  RSA_PKCS1_OAEP_PADDING,
-  RSA_PKCS1_PADDING
-} = internalBinding('constants').crypto;
-
-const {
-  ERR_CRYPTO_INVALID_STATE,
-  ERR_INVALID_ARG_TYPE,
-  ERR_INVALID_OPT_VALUE
-} = require('internal/errors').codes;
-const { validateEncoding, validateString } = require('internal/validators');
-
-const {
-  preparePrivateKey,
-  preparePublicOrPrivateKey,
-  prepareSecretKey
-} = require('internal/crypto/keys');
-const {
-  getDefaultEncoding,
-  kHandle,
-  getArrayBufferView
-} = require('internal/crypto/util');
-
-const { isArrayBufferView } = require('internal/util/types');
-
 const {
   CipherBase,
   privateDecrypt: _privateDecrypt,
   privateEncrypt: _privateEncrypt,
   publicDecrypt: _publicDecrypt,
-  publicEncrypt: _publicEncrypt
+  publicEncrypt: _publicEncrypt,
+  getCipherInfo: _getCipherInfo,
 } = internalBinding('crypto');
 
+const {
+  crypto: {
+    RSA_PKCS1_OAEP_PADDING,
+    RSA_PKCS1_PADDING,
+  }
+} = internalBinding('constants');
+
+const {
+  codes: {
+    ERR_CRYPTO_INVALID_STATE,
+    ERR_INVALID_ARG_TYPE,
+    ERR_INVALID_ARG_VALUE,
+  }
+} = require('internal/errors');
+
+const {
+  validateEncoding,
+  validateInt32,
+  validateObject,
+  validateString,
+} = require('internal/validators');
+
+const {
+  preparePrivateKey,
+  preparePublicOrPrivateKey,
+  prepareSecretKey,
+} = require('internal/crypto/keys');
+
+const {
+  getDefaultEncoding,
+  getArrayBufferOrView,
+  getStringOption,
+  kHandle,
+} = require('internal/crypto/util');
+
+const {
+  isArrayBufferView,
+} = require('internal/util/types');
+
 const assert = require('internal/assert');
+
 const LazyTransform = require('internal/streams/lazy_transform');
 
 const { normalizeEncoding } = require('internal/util');
@@ -52,14 +68,13 @@ function rsaFunctionFor(method, defaultPadding, keyType) {
         preparePrivateKey(options) :
         preparePublicOrPrivateKey(options);
     const padding = options.padding || defaultPadding;
-    const { oaepHash, oaepLabel } = options;
-    if (oaepHash !== undefined && typeof oaepHash !== 'string')
-      throw new ERR_INVALID_ARG_TYPE('options.oaepHash', 'string', oaepHash);
-    if (oaepLabel !== undefined && !isArrayBufferView(oaepLabel)) {
-      throw new ERR_INVALID_ARG_TYPE('options.oaepLabel',
-                                     ['Buffer', 'TypedArray', 'DataView'],
-                                     oaepLabel);
-    }
+    const { oaepHash, encoding } = options;
+    let { oaepLabel } = options;
+    if (oaepHash !== undefined)
+      validateString(oaepHash, 'key.oaepHash');
+    if (oaepLabel !== undefined)
+      oaepLabel = getArrayBufferOrView(oaepLabel, 'key.oaepLabel', encoding);
+    buffer = getArrayBufferOrView(buffer, 'buffer', encoding);
     return method(data, format, type, passphrase, buffer, padding, oaepHash,
                   oaepLabel);
   };
@@ -87,7 +102,7 @@ function getUIntOption(options, key) {
   let value;
   if (options && (value = options[key]) != null) {
     if (value >>> 0 !== value)
-      throw new ERR_INVALID_OPT_VALUE(key, value);
+      throw new ERR_INVALID_ARG_VALUE(`options.${key}`, value);
     return value;
   }
   return -1;
@@ -95,7 +110,6 @@ function getUIntOption(options, key) {
 
 function createCipherBase(cipher, credential, options, decipher, iv) {
   const authTagLength = getUIntOption(options, 'authTagLength');
-
   this[kHandle] = new CipherBase(decipher);
   if (iv === undefined) {
     this[kHandle].init(cipher, credential, authTagLength);
@@ -109,18 +123,24 @@ function createCipherBase(cipher, credential, options, decipher, iv) {
 
 function createCipher(cipher, password, options, decipher) {
   validateString(cipher, 'cipher');
-  password = getArrayBufferView(password, 'password');
+  password = getArrayBufferOrView(password, 'password');
 
   createCipherBase.call(this, cipher, password, options, decipher);
 }
 
 function createCipherWithIV(cipher, key, options, decipher, iv) {
   validateString(cipher, 'cipher');
-  key = prepareSecretKey(key);
-  iv = iv === null ? null : getArrayBufferView(iv, 'iv');
+  const encoding = getStringOption(options, 'encoding');
+  key = prepareSecretKey(key, encoding);
+  iv = iv === null ? null : getArrayBufferOrView(iv, 'iv');
   createCipherBase.call(this, cipher, key, options, decipher, iv);
 }
 
+// The Cipher class is part of the legacy Node.js crypto API. It exposes
+// a stream-based encryption/decryption model. For backwards compatibility
+// the Cipher class is defined using the legacy function syntax rather than
+// ES6 classes.
+
 function Cipher(cipher, password, options) {
   if (!(this instanceof Cipher))
     return new Cipher(cipher, password, options);
@@ -151,13 +171,13 @@ Cipher.prototype.update = function update(data, inputEncoding, outputEncoding) {
   inputEncoding = inputEncoding || encoding;
   outputEncoding = outputEncoding || encoding;
 
-  if (typeof data !== 'string' && !isArrayBufferView(data)) {
+  if (typeof data === 'string') {
+    validateEncoding(data, inputEncoding);
+  } else if (!isArrayBufferView(data)) {
     throw new ERR_INVALID_ARG_TYPE(
       'data', ['string', 'Buffer', 'TypedArray', 'DataView'], data);
   }
 
-  validateEncoding(data, inputEncoding);
-
   const ret = this[kHandle].update(data, inputEncoding);
 
   if (outputEncoding && outputEncoding !== 'buffer') {
@@ -196,30 +216,27 @@ Cipher.prototype.getAuthTag = function getAuthTag() {
 };
 
 
-function setAuthTag(tagbuf) {
-  if (!isArrayBufferView(tagbuf)) {
-    throw new ERR_INVALID_ARG_TYPE('buffer',
-                                   ['Buffer', 'TypedArray', 'DataView'],
-                                   tagbuf);
-  }
+function setAuthTag(tagbuf, encoding) {
+  tagbuf = getArrayBufferOrView(tagbuf, 'buffer', encoding);
   if (!this[kHandle].setAuthTag(tagbuf))
     throw new ERR_CRYPTO_INVALID_STATE('setAuthTag');
   return this;
 }
 
 Cipher.prototype.setAAD = function setAAD(aadbuf, options) {
-  if (!isArrayBufferView(aadbuf)) {
-    throw new ERR_INVALID_ARG_TYPE('buffer',
-                                   ['Buffer', 'TypedArray', 'DataView'],
-                                   aadbuf);
-  }
-
+  const encoding = getStringOption(options, 'encoding');
   const plaintextLength = getUIntOption(options, 'plaintextLength');
+  aadbuf = getArrayBufferOrView(aadbuf, 'aadbuf', encoding);
   if (!this[kHandle].setAAD(aadbuf, plaintextLength))
     throw new ERR_CRYPTO_INVALID_STATE('setAAD');
   return this;
 };
 
+// The Cipheriv class is part of the legacy Node.js crypto API. It exposes
+// a stream-based encryption/decryption model. For backwards compatibility
+// the Cipheriv class is defined using the legacy function syntax rather than
+// ES6 classes.
+
 function Cipheriv(cipher, key, iv, options) {
   if (!(this instanceof Cipheriv))
     return new Cipheriv(cipher, key, iv, options);
@@ -245,6 +262,11 @@ ObjectSetPrototypeOf(Cipheriv.prototype, LazyTransform.prototype);
 ObjectSetPrototypeOf(Cipheriv, LazyTransform);
 addCipherPrototypeFunctions(Cipheriv);
 
+// The Decipher class is part of the legacy Node.js crypto API. It exposes
+// a stream-based encryption/decryption model. For backwards compatibility
+// the Decipher class is defined using the legacy function syntax rather than
+// ES6 classes.
+
 function Decipher(cipher, password, options) {
   if (!(this instanceof Decipher))
     return new Decipher(cipher, password, options);
@@ -256,6 +278,10 @@ ObjectSetPrototypeOf(Decipher.prototype, LazyTransform.prototype);
 ObjectSetPrototypeOf(Decipher, LazyTransform);
 addCipherPrototypeFunctions(Decipher);
 
+// The Decipheriv class is part of the legacy Node.js crypto API. It exposes
+// a stream-based encryption/decryption model. For backwards compatibility
+// the Decipheriv class is defined using the legacy function syntax rather than
+// ES6 classes.
 
 function Decipheriv(cipher, key, iv, options) {
   if (!(this instanceof Decipheriv))
@@ -268,6 +294,33 @@ ObjectSetPrototypeOf(Decipheriv.prototype, LazyTransform.prototype);
 ObjectSetPrototypeOf(Decipheriv, LazyTransform);
 addCipherPrototypeFunctions(Decipheriv);
 
+function getCipherInfo(nameOrNid, options) {
+  if (typeof nameOrNid !== 'string' && typeof nameOrNid !== 'number') {
+    throw new ERR_INVALID_ARG_TYPE(
+      'nameOrNid',
+      ['string', 'number'],
+      nameOrNid);
+  }
+  if (typeof nameOrNid === 'number')
+    validateInt32(nameOrNid, 'nameOrNid');
+  let keyLength, ivLength;
+  if (options !== undefined) {
+    validateObject(options, 'options');
+    ({ keyLength, ivLength } = options);
+    if (keyLength !== undefined)
+      validateInt32(keyLength, 'options.keyLength');
+    if (ivLength !== undefined)
+      validateInt32(ivLength, 'options.ivLength');
+  }
+
+  const ret = _getCipherInfo({}, nameOrNid, keyLength, ivLength);
+  if (ret !== undefined) {
+    if (ret.name) ret.name = ret.name.toLowerCase();
+    if (ret.type) ret.type = ret.type.toLowerCase();
+  }
+  return ret;
+}
+
 module.exports = {
   Cipher,
   Cipheriv,
@@ -277,4 +330,5 @@ module.exports = {
   privateEncrypt,
   publicDecrypt,
   publicEncrypt,
+  getCipherInfo,
 };
