@@ -1,11 +1,9 @@
 'use strict';
 
 const {
-  ArrayPrototypeMap,
   ObjectCreate,
   ObjectDefineProperty,
   Promise,
-  ReflectApply,
 } = primordials;
 
 const {
@@ -28,7 +26,7 @@ const {
 } = internalBinding('cares_wrap');
 const {
   ERR_INVALID_ARG_TYPE,
-  ERR_INVALID_OPT_VALUE,
+  ERR_INVALID_ARG_VALUE,
   ERR_MISSING_ARGS,
 } = codes;
 const {
@@ -156,7 +154,7 @@ function lookupService(address, port) {
     throw new ERR_MISSING_ARGS('address', 'port');
 
   if (isIP(address) === 0)
-    throw new ERR_INVALID_OPT_VALUE('address', address);
+    throw new ERR_INVALID_ARG_VALUE('address', address);
 
   validatePort(port);
 
@@ -171,8 +169,7 @@ function onresolve(err, result, ttls) {
   }
 
   if (ttls && this.ttl)
-    result = ArrayPrototypeMap(
-      result, (address, index) => ({ address, ttl: ttls[index] }));
+    result = result.map((address, index) => ({ address, ttl: ttls[index] }));
 
   this.resolve(result);
 }
@@ -220,8 +217,6 @@ class Resolver {
 
 Resolver.prototype.getServers = CallbackResolver.prototype.getServers;
 Resolver.prototype.setServers = CallbackResolver.prototype.setServers;
-Resolver.prototype.cancel = CallbackResolver.prototype.cancel;
-Resolver.prototype.setLocalAddress = CallbackResolver.prototype.setLocalAddress;
 Resolver.prototype.resolveAny = resolveMap.ANY = resolver('queryAny');
 Resolver.prototype.resolve4 = resolveMap.A = resolver('queryA');
 Resolver.prototype.resolve6 = resolveMap.AAAA = resolver('queryAaaa');
@@ -242,14 +237,14 @@ Resolver.prototype.resolve = function resolve(hostname, rrtype) {
     resolver = resolveMap[rrtype];
 
     if (typeof resolver !== 'function')
-      throw new ERR_INVALID_OPT_VALUE('rrtype', rrtype);
+      throw new ERR_INVALID_ARG_VALUE('rrtype', rrtype);
   } else if (rrtype === undefined) {
     resolver = resolveMap.A;
   } else {
     throw new ERR_INVALID_ARG_TYPE('rrtype', 'string', rrtype);
   }
 
-  return ReflectApply(resolver, this, [hostname]);
+  return resolver.call(this, hostname);
 };
 
 
