@@ -23,8 +23,8 @@ const addrSplitRE = /(^.+?)(?::(\d+))?$/;
 const {
   ERR_DNS_SET_SERVERS_FAILED,
   ERR_INVALID_ARG_TYPE,
+  ERR_INVALID_ARG_VALUE,
   ERR_INVALID_IP_ADDRESS,
-  ERR_INVALID_OPT_VALUE
 } = errors.codes;
 
 function validateTimeout(options) {
@@ -114,18 +114,6 @@ class Resolver {
       throw new ERR_DNS_SET_SERVERS_FAILED(err, servers);
     }
   }
-
-  setLocalAddress(ipv4, ipv6) {
-    if (typeof ipv4 !== 'string') {
-      throw new ERR_INVALID_ARG_TYPE('ipv4', 'String', ipv4);
-    }
-
-    if (typeof ipv6 !== 'string' && ipv6 !== undefined) {
-      throw new ERR_INVALID_ARG_TYPE('ipv6', ['String', 'undefined'], ipv6);
-    }
-
-    this._handle.setLocalAddress(ipv4, ipv6);
-  }
 }
 
 let defaultResolver = new Resolver();
@@ -163,7 +151,7 @@ function bindDefaultResolver(target, source) {
 
 function validateHints(hints) {
   if ((hints & ~(AI_ADDRCONFIG | AI_ALL | AI_V4MAPPED)) !== 0) {
-    throw new ERR_INVALID_OPT_VALUE('hints', hints);
+    throw new ERR_INVALID_ARG_VALUE('hints', hints);
   }
 }
 
