@@ -4,17 +4,15 @@ const {
   ArrayFrom,
   Boolean,
   Error,
+  Map,
   NumberIsInteger,
-  Object,
+  ObjectAssign,
+  ObjectDefineProperties,
   ObjectDefineProperty,
-  ObjectGetOwnPropertyDescriptor,
-  ReflectApply,
-  SafeMap,
   String,
   Symbol,
   SymbolFor,
   SymbolToStringTag,
-  SafeWeakSet,
 } = primordials;
 
 const {
@@ -25,30 +23,24 @@ const {
     ERR_INVALID_THIS,
   }
 } = require('internal/errors');
-const { validateObject } = require('internal/validators');
+const { validateInteger, validateObject } = require('internal/validators');
 
 const { customInspectSymbol } = require('internal/util');
 const { inspect } = require('util');
 
 const kIsEventTarget = SymbolFor('nodejs.event_target');
 
-const EventEmitter = require('events');
-const {
-  kMaxEventTargetListeners,
-  kMaxEventTargetListenersWarned,
-} = EventEmitter;
-
 const kEvents = Symbol('kEvents');
 const kStop = Symbol('kStop');
 const kTarget = Symbol('kTarget');
-const kHandlers = Symbol('khandlers');
 
 const kHybridDispatch = Symbol.for('nodejs.internal.kHybridDispatch');
 const kCreateEvent = Symbol('kCreateEvent');
 const kNewListener = Symbol('kNewListener');
 const kRemoveListener = Symbol('kRemoveListener');
 const kIsNodeStyleListener = Symbol('kIsNodeStyleListener');
-const kTrustEvent = Symbol('kTrustEvent');
+const kMaxListeners = Symbol('kMaxListeners');
+const kMaxListenersWarned = Symbol('kMaxListenersWarned');
 
 // Lazy load perf_hooks to avoid the additional overhead on startup
 let perf_hooks;
@@ -69,18 +61,13 @@ const kBubbles = Symbol('bubbles');
 const kComposed = Symbol('composed');
 const kPropagationStopped = Symbol('propagationStopped');
 
-const isTrustedSet = new SafeWeakSet();
-const isTrusted = ObjectGetOwnPropertyDescriptor({
-  get isTrusted() {
-    return isTrustedSet.has(this);
-  }
-}, 'isTrusted').get;
+const isTrusted = () => false;
 
 class Event {
-  constructor(type, options = null) {
+  constructor(type, options) {
     if (arguments.length === 0)
       throw new ERR_MISSING_ARGS('type');
-    if (options !== null)
+    if (options != null)
       validateObject(options, 'options');
     const { cancelable, bubbles, composed } = { ...options };
     this[kCancelable] = !!cancelable;
@@ -90,10 +77,6 @@ class Event {
     this[kDefaultPrevented] = false;
     this[kTimestamp] = lazyNow();
     this[kPropagationStopped] = false;
-    if (options != null && options[kTrustEvent]) {
-      isTrustedSet.add(this);
-    }
-
     // isTrusted is special (LegacyUnforgeable)
     ObjectDefineProperty(this, 'isTrusted', {
       get: isTrusted,
@@ -108,7 +91,7 @@ class Event {
     if (depth < 0)
       return name;
 
-    const opts = Object.assign({}, options, {
+    const opts = ObjectAssign({}, options, {
       depth: NumberIsInteger(options.depth) ? options.depth - 1 : options.depth
     });
 
@@ -170,21 +153,13 @@ class Event {
   static BUBBLING_PHASE = 3;
 }
 
-Object.defineProperty(Event.prototype, SymbolToStringTag, {
+ObjectDefineProperty(Event.prototype, SymbolToStringTag, {
   writable: false,
   enumerable: false,
   configurable: true,
   value: 'Event',
 });
 
-class NodeCustomEvent extends Event {
-  constructor(type, options) {
-    super(type, options);
-    if (options && options.detail) {
-      this.detail = options.detail;
-    }
-  }
-}
 // The listeners for an EventTarget are maintained as a linked list.
 // Unfortunately, the way EventTarget is defined, listeners are accounted
 // using the tuple [handler,capture], and even if we don't actually make
@@ -223,9 +198,7 @@ class Listener {
 }
 
 function initEventTarget(self) {
-  self[kEvents] = new SafeMap();
-  self[kMaxEventTargetListeners] = EventEmitter.defaultMaxListeners;
-  self[kMaxEventTargetListenersWarned] = false;
+  self[kEvents] = new Map();
 }
 
 class EventTarget {
@@ -238,24 +211,7 @@ class EventTarget {
     initEventTarget(this);
   }
 
-  [kNewListener](size, type, listener, once, capture, passive) {
-    if (this[kMaxEventTargetListeners] > 0 &&
-        size > this[kMaxEventTargetListeners] &&
-        !this[kMaxEventTargetListenersWarned]) {
-      this[kMaxEventTargetListenersWarned] = true;
-      // No error code for this since it is a Warning
-      // eslint-disable-next-line no-restricted-syntax
-      const w = new Error('Possible EventTarget memory leak detected. ' +
-                          `${size} ${type} listeners ` +
-                          `added to ${inspect(this, { depth: -1 })}. Use ` +
-                          'events.setMaxListeners() to increase limit');
-      w.name = 'MaxListenersExceededWarning';
-      w.target = this;
-      w.type = type;
-      w.count = size;
-      process.emitWarning(w);
-    }
-  }
+  [kNewListener](size, type, listener, once, capture, passive) {}
   [kRemoveListener](size, type, listener, capture) {}
 
   addEventListener(type, listener, options = {}) {
@@ -366,13 +322,14 @@ class EventTarget {
       }
       return event;
     };
-    if (event !== undefined)
-      event[kTarget] = this;
 
     const root = this[kEvents].get(type);
     if (root === undefined || root.next === undefined)
       return true;
 
+    if (event !== undefined)
+      event[kTarget] = this;
+
     let handler = root.next;
     let next;
 
@@ -409,15 +366,12 @@ class EventTarget {
       event[kTarget] = undefined;
   }
 
-  [kCreateEvent](nodeValue, type) {
-    return new NodeCustomEvent(type, { detail: nodeValue });
-  }
   [customInspectSymbol](depth, options) {
     const name = this.constructor.name;
     if (depth < 0)
       return name;
 
-    const opts = Object.assign({}, options, {
+    const opts = ObjectAssign({}, options, {
       depth: NumberIsInteger(options.depth) ? options.depth - 1 : options.depth
     });
 
@@ -425,12 +379,12 @@ class EventTarget {
   }
 }
 
-Object.defineProperties(EventTarget.prototype, {
+ObjectDefineProperties(EventTarget.prototype, {
   addEventListener: { enumerable: true },
   removeEventListener: { enumerable: true },
   dispatchEvent: { enumerable: true }
 });
-Object.defineProperty(EventTarget.prototype, SymbolToStringTag, {
+ObjectDefineProperty(EventTarget.prototype, SymbolToStringTag, {
   writable: false,
   enumerable: false,
   configurable: true,
@@ -439,6 +393,9 @@ Object.defineProperty(EventTarget.prototype, SymbolToStringTag, {
 
 function initNodeEventTarget(self) {
   initEventTarget(self);
+  // eslint-disable-next-line no-use-before-define
+  self[kMaxListeners] = NodeEventTarget.defaultMaxListeners;
+  self[kMaxListenersWarned] = false;
 }
 
 class NodeEventTarget extends EventTarget {
@@ -449,12 +406,33 @@ class NodeEventTarget extends EventTarget {
     initNodeEventTarget(this);
   }
 
+  [kNewListener](size, type, listener, once, capture, passive) {
+    if (this[kMaxListeners] > 0 &&
+        size > this[kMaxListeners] &&
+        !this[kMaxListenersWarned]) {
+      this[kMaxListenersWarned] = true;
+      // No error code for this since it is a Warning
+      // eslint-disable-next-line no-restricted-syntax
+      const w = new Error('Possible EventTarget memory leak detected. ' +
+                          `${size} ${type} listeners ` +
+                          `added to ${inspect(this, { depth: -1 })}. Use ` +
+                          'setMaxListeners() to increase limit');
+      w.name = 'MaxListenersExceededWarning';
+      w.target = this;
+      w.type = type;
+      w.count = size;
+      process.emitWarning(w);
+    }
+  }
+
   setMaxListeners(n) {
-    EventEmitter.setMaxListeners(n, this);
+    validateInteger(n, 'n', 0);
+    this[kMaxListeners] = n;
+    return this;
   }
 
   getMaxListeners() {
-    return this[kMaxEventTargetListeners];
+    return this[kMaxListeners];
   }
 
   eventNames() {
@@ -485,14 +463,6 @@ class NodeEventTarget extends EventTarget {
     this.addEventListener(type, listener, { [kIsNodeStyleListener]: true });
     return this;
   }
-  emit(type, arg) {
-    if (typeof type !== 'string') {
-      throw new ERR_INVALID_ARG_TYPE('type', 'string', type);
-    }
-    const hadListeners = this.listenerCount(type) > 0;
-    this[kHybridDispatch](arg, type);
-    return hadListeners;
-  }
 
   once(type, listener) {
     this.addEventListener(type, listener,
@@ -506,12 +476,10 @@ class NodeEventTarget extends EventTarget {
     } else {
       this[kEvents].clear();
     }
-
-    return this;
   }
 }
 
-Object.defineProperties(NodeEventTarget.prototype, {
+ObjectDefineProperties(NodeEventTarget.prototype, {
   setMaxListeners: { enumerable: true },
   getMaxListeners: { enumerable: true },
   eventNames: { enumerable: true },
@@ -521,7 +489,6 @@ Object.defineProperties(NodeEventTarget.prototype, {
   on: { enumerable: true },
   addListener: { enumerable: true },
   once: { enumerable: true },
-  emit: { enumerable: true },
   removeAllListeners: { enumerable: true },
 });
 
@@ -577,50 +544,22 @@ function emitUnhandledRejectionOrErr(that, err, event) {
   process.emit('error', err, event);
 }
 
-function makeEventHandler(handler) {
-  // Event handlers are dispatched in the order they were first set
-  // See https://github.com/nodejs/node/pull/35949#issuecomment-722496598
-  function eventHandler(...args) {
-    if (typeof eventHandler.handler !== 'function') {
-      return;
-    }
-    return ReflectApply(eventHandler.handler, this, args);
-  }
-  eventHandler.handler = handler;
-  return eventHandler;
-}
-
 function defineEventHandler(emitter, name) {
   // 8.1.5.1 Event handlers - basically `on[eventName]` attributes
+  let eventHandlerValue;
   ObjectDefineProperty(emitter, `on${name}`, {
     get() {
-      return this[kHandlers]?.get(name)?.handler;
+      return eventHandlerValue;
     },
     set(value) {
-      if (!this[kHandlers]) {
-        this[kHandlers] = new SafeMap();
+      if (eventHandlerValue) {
+        emitter.removeEventListener(name, eventHandlerValue);
       }
-      let wrappedHandler = this[kHandlers]?.get(name);
-      if (wrappedHandler) {
-        if (typeof wrappedHandler.handler === 'function') {
-          this[kEvents].get(name).size--;
-          const size = this[kEvents].get(name).size;
-          this[kRemoveListener](size, name, wrappedHandler.handler, false);
-        }
-        wrappedHandler.handler = value;
-        if (typeof wrappedHandler.handler === 'function') {
-          this[kEvents].get(name).size++;
-          const size = this[kEvents].get(name).size;
-          this[kNewListener](size, name, value, false, false, false);
-        }
-      } else {
-        wrappedHandler = makeEventHandler(value);
-        this.addEventListener(name, wrappedHandler);
+      if (typeof value === 'function') {
+        emitter.addEventListener(name, value);
       }
-      this[kHandlers].set(name, wrappedHandler);
-    },
-    configurable: true,
-    enumerable: true
+      eventHandlerValue = value;
+    }
   });
 }
 module.exports = {
@@ -632,8 +571,5 @@ module.exports = {
   initNodeEventTarget,
   kCreateEvent,
   kNewListener,
-  kTrustEvent,
   kRemoveListener,
-  kEvents,
-  isEventTarget,
 };
