@@ -3,13 +3,11 @@
 const {
   ArrayIsArray,
   BigInt,
-  Date,
   DateNow,
-  ErrorCaptureStackTrace,
+  Error,
   ObjectPrototypeHasOwnProperty,
   Number,
   NumberIsFinite,
-  NumberIsInteger,
   MathMin,
   ObjectSetPrototypeOf,
   ReflectOwnKeys,
@@ -23,8 +21,6 @@ const {
     ERR_FS_INVALID_SYMLINK_TYPE,
     ERR_INVALID_ARG_TYPE,
     ERR_INVALID_ARG_VALUE,
-    ERR_INVALID_OPT_VALUE,
-    ERR_INVALID_OPT_VALUE_ENCODING,
     ERR_OUT_OF_RANGE
   },
   hideStackFrames,
@@ -39,11 +35,9 @@ const {
 const { once } = require('internal/util');
 const { toPathIfFileURL } = require('internal/url');
 const {
-  validateAbortSignal,
   validateBoolean,
   validateInt32,
-  validateInteger,
-  validateUint32,
+  validateUint32
 } = require('internal/validators');
 const pathModule = require('path');
 const kType = Symbol('type');
@@ -113,23 +107,6 @@ const kMaximumCopyMode = COPYFILE_EXCL |
                          COPYFILE_FICLONE |
                          COPYFILE_FICLONE_FORCE;
 
-// Most platforms don't allow reads or writes >= 2 GB.
-// See https://github.com/libuv/libuv/pull/1501.
-const kIoMaxLength = 2 ** 31 - 1;
-
-// Use 64kb in case the file type is not a regular file and thus do not know the
-// actual file size. Increasing the value further results in more frequent over
-// allocation for small files and consumes CPU time and memory that should be
-// used else wise.
-// Use up to 512kb per read otherwise to partition reading big files to prevent
-// blocking other threads in case the available threads are all in use.
-const kReadFileUnknownBufferLength = 64 * 1024;
-const kReadFileBufferLength = 512 * 1024;
-
-const kWriteFileMaxChunkSize = 512 * 1024;
-
-const kMaxUserId = 2 ** 32 - 1;
-
 const isWindows = process.platform === 'win32';
 
 let fs;
@@ -142,7 +119,8 @@ function lazyLoadFs() {
 
 function assertEncoding(encoding) {
   if (encoding && !Buffer.isEncoding(encoding)) {
-    throw new ERR_INVALID_OPT_VALUE_ENCODING(encoding);
+    const reason = 'is invalid encoding';
+    throw new ERR_INVALID_ARG_VALUE(encoding, 'encoding', reason);
   }
 }
 
@@ -318,24 +296,22 @@ function getOptions(options, defaultOptions) {
 
   if (options.encoding !== 'buffer')
     assertEncoding(options.encoding);
-
-  if (options.signal !== undefined) {
-    validateAbortSignal(options.signal, 'options.signal');
-  }
   return options;
 }
 
 function handleErrorFromBinding(ctx) {
   if (ctx.errno !== undefined) {  // libuv error numbers
     const err = uvException(ctx);
-    ErrorCaptureStackTrace(err, handleErrorFromBinding);
+    // eslint-disable-next-line no-restricted-syntax
+    Error.captureStackTrace(err, handleErrorFromBinding);
     throw err;
   }
   if (ctx.error !== undefined) {  // Errors created in C++ land.
     // TODO(joyeecheung): currently, ctx.error are encoding errors
     // usually caused by memory problems. We need to figure out proper error
     // code(s) for this.
-    ErrorCaptureStackTrace(ctx.error, handleErrorFromBinding);
+    // eslint-disable-next-line no-restricted-syntax
+    Error.captureStackTrace(ctx.error, handleErrorFromBinding);
     throw ctx.error;
   }
 }
@@ -573,7 +549,7 @@ function stringToFlags(flags) {
     case 'sa+': return O_APPEND | O_CREAT | O_RDWR | O_SYNC;
   }
 
-  throw new ERR_INVALID_OPT_VALUE('flags', flags);
+  throw new ERR_INVALID_ARG_VALUE('flags', flags);
 }
 
 const stringToSymlinkType = hideStackFrames((type) => {
@@ -638,12 +614,6 @@ const validateOffsetLengthWrite = hideStackFrames(
     if (length > byteLength - offset) {
       throw new ERR_OUT_OF_RANGE('length', `<= ${byteLength - offset}`, length);
     }
-
-    if (length < 0) {
-      throw new ERR_OUT_OF_RANGE('length', '>= 0', length);
-    }
-
-    validateInt32(length, 'length', 0);
   }
 );
 
@@ -702,18 +672,25 @@ const defaultRmdirOptions = {
   recursive: false,
 };
 
-const validateRmOptions = hideStackFrames((path, options, callback) => {
+const validateRmOptions = hideStackFrames((path, options, warn, callback) => {
   options = validateRmdirOptions(options, defaultRmOptions);
   validateBoolean(options.force, 'options.force');
 
   lazyLoadFs().stat(path, (err, stats) => {
     if (err) {
       if (options.force && err.code === 'ENOENT') {
+        if (warn) {
+          emitPermissiveRmdirWarning();
+        }
         return callback(null, options);
       }
       return callback(err, options);
     }
 
+    if (warn && !stats.isDirectory()) {
+      emitPermissiveRmdirWarning();
+    }
+
     if (stats.isDirectory() && !options.recursive) {
       return callback(new ERR_FS_EISDIR({
         code: 'EISDIR',
@@ -727,13 +704,17 @@ const validateRmOptions = hideStackFrames((path, options, callback) => {
   });
 });
 
-const validateRmOptionsSync = hideStackFrames((path, options) => {
+const validateRmOptionsSync = hideStackFrames((path, options, warn) => {
   options = validateRmdirOptions(options, defaultRmOptions);
   validateBoolean(options.force, 'options.force');
 
   try {
     const stats = lazyLoadFs().statSync(path);
 
+    if (warn && !stats.isDirectory()) {
+      emitPermissiveRmdirWarning();
+    }
+
     if (stats.isDirectory() && !options.recursive) {
       throw new ERR_FS_EISDIR({
         code: 'EISDIR',
@@ -748,12 +729,29 @@ const validateRmOptionsSync = hideStackFrames((path, options) => {
       throw err;
     } else if (err.code === 'ENOENT' && !options.force) {
       throw err;
+    } else if (warn && err.code === 'ENOENT') {
+      emitPermissiveRmdirWarning();
     }
   }
 
   return options;
 });
 
+let permissiveRmdirWarned = false;
+
+function emitPermissiveRmdirWarning() {
+  if (!permissiveRmdirWarned) {
+    process.emitWarning(
+      'In future versions of Node.js, fs.rmdir(path, { recursive: true }) ' +
+      'will throw if path does not exist or is a file. Use fs.rm(path, ' +
+      '{ recursive: true, force: true }) instead',
+      'DeprecationWarning',
+      'DEP0147'
+    );
+    permissiveRmdirWarned = true;
+  }
+}
+
 const validateRmdirOptions = hideStackFrames(
   (options, defaults = defaultRmdirOptions) => {
     if (options === undefined)
@@ -784,7 +782,7 @@ const getValidMode = hideStackFrames((mode, type) => {
   if (mode == null) {
     return def;
   }
-  if (NumberIsInteger(mode) && mode >= min && mode <= max) {
+  if (Number.isInteger(mode) && mode >= min && mode <= max) {
     return mode;
   }
   if (typeof mode !== 'number') {
@@ -815,30 +813,7 @@ const validateStringAfterArrayBufferView = hideStackFrames((buffer, name) => {
   );
 });
 
-const validatePosition = hideStackFrames((position, name) => {
-  if (typeof position === 'number') {
-    validateInteger(position, 'position');
-  } else if (typeof position === 'bigint') {
-    if (!(position >= -(2n ** 63n) && position <= 2n ** 63n - 1n)) {
-      throw new ERR_OUT_OF_RANGE('position',
-                                 `>= ${-(2n ** 63n)} && <= ${2n ** 63n - 1n}`,
-                                 position);
-    }
-  } else {
-    throw new ERR_INVALID_ARG_TYPE('position',
-                                   ['integer', 'bigint'],
-                                   position);
-  }
-});
-
 module.exports = {
-  constants: {
-    kIoMaxLength,
-    kMaxUserId,
-    kReadFileBufferLength,
-    kReadFileUnknownBufferLength,
-    kWriteFileMaxChunkSize,
-  },
   assertEncoding,
   BigIntStats,  // for testing
   copyObject,
@@ -861,7 +836,6 @@ module.exports = {
   validateOffsetLengthRead,
   validateOffsetLengthWrite,
   validatePath,
-  validatePosition,
   validateRmOptions,
   validateRmOptionsSync,
   validateRmdirOptions,
