@@ -34,7 +34,7 @@ const { getOptionValue } = require('internal/options');
 const policy = getOptionValue('--experimental-policy') ?
   require('internal/process/policy') :
   null;
-const { sep, relative, resolve } = require('path');
+const { sep, relative } = require('path');
 const preserveSymlinks = getOptionValue('--preserve-symlinks');
 const preserveSymlinksMain = getOptionValue('--preserve-symlinks-main');
 const typeFlag = getOptionValue('--input-type');
@@ -59,37 +59,6 @@ const userConditions = getOptionValue('--conditions');
 const DEFAULT_CONDITIONS = ObjectFreeze(['node', 'import', ...userConditions]);
 const DEFAULT_CONDITIONS_SET = new SafeSet(DEFAULT_CONDITIONS);
 
-const pendingDeprecation = getOptionValue('--pending-deprecation');
-
-function emitLegacyIndexDeprecation(url, packageJSONUrl, base, main) {
-  if (!pendingDeprecation)
-    return;
-  const { format } = defaultGetFormat(url);
-  if (format !== 'module')
-    return;
-  const path = fileURLToPath(url);
-  const pkgPath = fileURLToPath(new URL('.', packageJSONUrl));
-  const basePath = fileURLToPath(base);
-  if (main)
-    process.emitWarning(
-      `Package ${pkgPath} has a "main" field set to ${JSONStringify(main)}, ` +
-      `excluding the full filename and extension to the resolved file at "${
-        StringPrototypeSlice(path, pkgPath.length)}", imported from ${
-        basePath}.\n Automatic extension resolution of the "main" field is` +
-      'deprecated for ES modules.',
-      'DeprecationWarning',
-      'DEP0151'
-    );
-  else
-    process.emitWarning(
-      `No "main" or "exports" field defined in the package.json for ${pkgPath
-      } resolving the main entry point "${
-        StringPrototypeSlice(path, pkgPath.length)}", imported from ${basePath
-      }.\nDefault "index" lookups for the main are deprecated for ES modules.`,
-      'DeprecationWarning',
-      'DEP0151'
-    );
-}
 
 function getConditionsSet(conditions) {
   if (conditions !== undefined && conditions !== DEFAULT_CONDITIONS) {
@@ -105,8 +74,13 @@ function getConditionsSet(conditions) {
 const realpathCache = new SafeMap();
 const packageJSONCache = new SafeMap();  /* string -> PackageConfig */
 
-const tryStatSync =
-  (path) => statSync(path, { throwIfNoEntry: false }) ?? new Stats();
+function tryStatSync(path) {
+  try {
+    return statSync(path);
+  } catch {
+    return new Stats();
+  }
+}
 
 function getPackageConfig(path, specifier, base) {
   const existing = packageJSONCache.get(path);
@@ -191,18 +165,16 @@ function getPackageScopeConfig(resolved) {
   return packageConfig;
 }
 
-/**
+/*
  * Legacy CommonJS main resolution:
  * 1. let M = pkg_url + (json main field)
  * 2. TRY(M, M.js, M.json, M.node)
  * 3. TRY(M/index.js, M/index.json, M/index.node)
  * 4. TRY(pkg_url/index.js, pkg_url/index.json, pkg_url/index.node)
  * 5. NOT_FOUND
- * @param {string | URL} url
- * @returns {boolean}
  */
 function fileExists(url) {
-  return statSync(url, { throwIfNoEntry: false })?.isFile() ?? false;
+  return tryStatSync(fileURLToPath(url)).isFile();
 }
 
 function legacyMainResolve(packageJSONUrl, packageConfig, base) {
@@ -212,33 +184,41 @@ function legacyMainResolve(packageJSONUrl, packageConfig, base) {
     if (fileExists(guess = new URL(`./${packageConfig.main}`,
                                    packageJSONUrl))) {
       return guess;
-    } else if (fileExists(guess = new URL(`./${packageConfig.main}.js`,
-                                          packageJSONUrl)));
-    else if (fileExists(guess = new URL(`./${packageConfig.main}.json`,
-                                        packageJSONUrl)));
-    else if (fileExists(guess = new URL(`./${packageConfig.main}.node`,
-                                        packageJSONUrl)));
-    else if (fileExists(guess = new URL(`./${packageConfig.main}/index.js`,
-                                        packageJSONUrl)));
-    else if (fileExists(guess = new URL(`./${packageConfig.main}/index.json`,
-                                        packageJSONUrl)));
-    else if (fileExists(guess = new URL(`./${packageConfig.main}/index.node`,
-                                        packageJSONUrl)));
-    else guess = undefined;
-    if (guess) {
-      emitLegacyIndexDeprecation(guess, packageJSONUrl, base,
-                                 packageConfig.main);
+    }
+    if (fileExists(guess = new URL(`./${packageConfig.main}.js`,
+                                   packageJSONUrl))) {
+      return guess;
+    }
+    if (fileExists(guess = new URL(`./${packageConfig.main}.json`,
+                                   packageJSONUrl))) {
+      return guess;
+    }
+    if (fileExists(guess = new URL(`./${packageConfig.main}.node`,
+                                   packageJSONUrl))) {
+      return guess;
+    }
+    if (fileExists(guess = new URL(`./${packageConfig.main}/index.js`,
+                                   packageJSONUrl))) {
+      return guess;
+    }
+    if (fileExists(guess = new URL(`./${packageConfig.main}/index.json`,
+                                   packageJSONUrl))) {
+      return guess;
+    }
+    if (fileExists(guess = new URL(`./${packageConfig.main}/index.node`,
+                                   packageJSONUrl))) {
       return guess;
     }
     // Fallthrough.
   }
-  if (fileExists(guess = new URL('./index.js', packageJSONUrl)));
+  if (fileExists(guess = new URL('./index.js', packageJSONUrl))) {
+    return guess;
+  }
   // So fs.
-  else if (fileExists(guess = new URL('./index.json', packageJSONUrl)));
-  else if (fileExists(guess = new URL('./index.node', packageJSONUrl)));
-  else guess = undefined;
-  if (guess) {
-    emitLegacyIndexDeprecation(guess, packageJSONUrl, base, packageConfig.main);
+  if (fileExists(guess = new URL('./index.json', packageJSONUrl))) {
+    return guess;
+  }
+  if (fileExists(guess = new URL('./index.node', packageJSONUrl))) {
     return guess;
   }
   // Not found.
@@ -261,19 +241,7 @@ function resolveExtensions(search) {
   return undefined;
 }
 
-function resolveDirectoryEntry(search) {
-  const dirPath = fileURLToPath(search);
-  const pkgJsonPath = resolve(dirPath, 'package.json');
-  if (fileExists(pkgJsonPath)) {
-    const pkgJson = packageJsonReader.read(pkgJsonPath);
-    if (pkgJson.containsKeys) {
-      const { main } = JSONParse(pkgJson.string);
-      if (main != null) {
-        const mainUrl = pathToFileURL(resolve(dirPath, main));
-        return resolveExtensionsWithTryExactName(mainUrl);
-      }
-    }
-  }
+function resolveIndex(search) {
   return resolveExtensions(new URL('index', search));
 }
 
@@ -289,10 +257,10 @@ function finalizeResolution(resolved, base) {
     let file = resolveExtensionsWithTryExactName(resolved);
     if (file !== undefined) return file;
     if (!StringPrototypeEndsWith(path, '/')) {
-      file = resolveDirectoryEntry(new URL(`${resolved}/`));
+      file = resolveIndex(new URL(`${resolved}/`));
       if (file !== undefined) return file;
     } else {
-      return resolveDirectoryEntry(resolved) || resolved;
+      return resolveIndex(resolved) || resolved;
     }
     throw new ERR_MODULE_NOT_FOUND(
       resolved.pathname, fileURLToPath(base), 'module');
@@ -869,8 +837,7 @@ function defaultResolve(specifier, context = {}, defaultResolveUnused) {
       [internalFS.realpathCacheKey]: realpathCache
     });
     const old = url;
-    url = pathToFileURL(
-      real + (StringPrototypeEndsWith(urlPath, sep) ? '/' : ''));
+    url = pathToFileURL(real + (urlPath.endsWith(sep) ? '/' : ''));
     url.search = old.search;
     url.hash = old.hash;
   }
@@ -882,11 +849,7 @@ module.exports = {
   DEFAULT_CONDITIONS,
   defaultResolve,
   encodedSepRegEx,
-  getPackageScopeConfig,
   getPackageType,
   packageExportsResolve,
   packageImportsResolve
 };
-
-// cycle
-const { defaultGetFormat } = require('internal/modules/esm/get_format');
