@@ -5,6 +5,7 @@
 const {
   Boolean,
   JSONParse,
+  ObjectGetPrototypeOf,
   ObjectPrototypeHasOwnProperty,
   ObjectKeys,
   PromisePrototypeCatch,
@@ -15,6 +16,7 @@ const {
   StringPrototypeReplace,
   StringPrototypeSplit,
   StringPrototypeStartsWith,
+  SyntaxErrorPrototype,
 } = primordials;
 
 let _TYPES = null;
@@ -56,9 +58,20 @@ const { getOptionValue } = require('internal/options');
 const experimentalImportMetaResolve =
     getOptionValue('--experimental-import-meta-resolve');
 const asyncESM = require('internal/process/esm_loader');
-const cjsParse = require('internal/deps/cjs-module-lexer/lexer');
 const { emitWarningSync } = require('internal/process/warning');
 
+let cjsParse;
+async function initCJSParse() {
+  if (typeof WebAssembly === 'undefined') {
+    cjsParse = require('internal/deps/cjs-module-lexer/lexer');
+  } else {
+    const { parse, init } =
+        require('internal/deps/cjs-module-lexer/dist/lexer');
+    await init();
+    cjsParse = parse;
+  }
+}
+
 const translators = new SafeMap();
 exports.translators = translators;
 exports.enrichCJSError = enrichCJSError;
@@ -136,6 +149,9 @@ translators.set('module', async function moduleStrategy(url) {
 });
 
 function enrichCJSError(err) {
+  if (err == null || ObjectGetPrototypeOf(err) !== SyntaxErrorPrototype) {
+    return;
+  }
   const stack = StringPrototypeSplit(err.stack, '\n');
   /*
   * The regular expression below targets the most common import statement
@@ -164,6 +180,7 @@ translators.set('commonjs', async function commonjsStrategy(url, isMain) {
   if (isWindows)
     filename = StringPrototypeReplace(filename, winSepRegEx, '\\');
 
+  if (!cjsParse) await initCJSParse();
   const { module, exportNames } = cjsPreparseModuleExports(filename);
   const namesWithDefault = exportNames.has('default') ?
     [...exportNames] : ['default', ...exportNames];
