@@ -101,9 +101,9 @@ primordials.SafeSet = makeSafe(
   Set,
   class SafeSet extends Set {}
 );
-primordials.SafeWeakSet = makeSafe(
-  WeakSet,
-  class SafeWeakSet extends WeakSet {}
+primordials.SafePromise = makeSafe(
+  Promise,
+  class SafePromise extends Promise {}
 );
 
 // Create copies of the namespace objects
@@ -123,6 +123,7 @@ primordials.SafeWeakSet = makeSafe(
   'BigInt64Array',
   'BigUint64Array',
   'Boolean',
+  'DataView',
   'Date',
   'Error',
   'EvalError',
