@@ -25,9 +25,8 @@ const {
   errnoException,
   codes: {
     ERR_ASSERTION,
-    ERR_CPU_USAGE,
     ERR_INVALID_ARG_TYPE,
-    ERR_INVALID_OPT_VALUE,
+    ERR_INVALID_ARG_VALUE,
     ERR_OUT_OF_RANGE,
     ERR_UNKNOWN_SIGNAL
   }
@@ -39,11 +38,56 @@ function assert(x, msg) {
   if (!x) throw new ERR_ASSERTION(msg || 'assertion error');
 }
 
+function getFastAPIs(binding) {
+  const {
+    hrtime: _hrtime
+  } = binding.getFastAPIs();
+
+  // The 3 entries filled in by the original process.hrtime contains
+  // the upper/lower 32 bits of the second part of the value,
+  // and the remaining nanoseconds of the value.
+  const hrValues = new Uint32Array(_hrtime.buffer);
+
+  function hrtime(time) {
+    _hrtime.hrtime();
+
+    if (time !== undefined) {
+      if (!ArrayIsArray(time)) {
+        throw new ERR_INVALID_ARG_TYPE('time', 'Array', time);
+      }
+      if (time.length !== 2) {
+        throw new ERR_OUT_OF_RANGE('time', 2, time.length);
+      }
+
+      const sec = (hrValues[0] * 0x100000000 + hrValues[1]) - time[0];
+      const nsec = hrValues[2] - time[1];
+      const needsBorrow = nsec < 0;
+      return [needsBorrow ? sec - 1 : sec, needsBorrow ? nsec + 1e9 : nsec];
+    }
+
+    return [
+      hrValues[0] * 0x100000000 + hrValues[1],
+      hrValues[2]
+    ];
+  }
+
+  // Use a BigUint64Array in the closure because this is actually a bit
+  // faster than simply returning a BigInt from C++ in V8 7.1.
+  const hrBigintValues = new BigUint64Array(_hrtime.buffer, 0, 1);
+  function hrtimeBigInt() {
+    _hrtime.hrtimeBigInt();
+    return hrBigintValues[0];
+  }
+
+  return {
+    hrtime,
+    hrtimeBigInt,
+  };
+}
+
 // The execution of this function itself should not cause any side effects.
 function wrapProcessMethods(binding) {
   const {
-    hrtime: _hrtime,
-    hrtimeBigInt: _hrtimeBigInt,
     cpuUsage: _cpuUsage,
     memoryUsage: _memoryUsage,
     resourceUsage: _resourceUsage
@@ -69,7 +113,7 @@ function wrapProcessMethods(binding) {
           throw new ERR_INVALID_ARG_TYPE('prevValue.user',
                                          'number', prevValue.user);
         }
-        throw new ERR_INVALID_OPT_VALUE.RangeError('prevValue.user',
+        throw new ERR_INVALID_ARG_VALUE.RangeError('prevValue.user',
                                                    prevValue.user);
       }
 
@@ -78,16 +122,13 @@ function wrapProcessMethods(binding) {
           throw new ERR_INVALID_ARG_TYPE('prevValue.system',
                                          'number', prevValue.system);
         }
-        throw new ERR_INVALID_OPT_VALUE.RangeError('prevValue.system',
+        throw new ERR_INVALID_ARG_VALUE.RangeError('prevValue.system',
                                                    prevValue.system);
       }
     }
 
     // Call the native function to get the current values.
-    const errmsg = _cpuUsage(cpuValues);
-    if (errmsg) {
-      throw new ERR_CPU_USAGE(errmsg);
-    }
+    _cpuUsage(cpuValues);
 
     // If a previous value was passed in, return diff of current from previous.
     if (prevValue) {
@@ -112,42 +153,6 @@ function wrapProcessMethods(binding) {
         num >= 0;
   }
 
-  // The 3 entries filled in by the original process.hrtime contains
-  // the upper/lower 32 bits of the second part of the value,
-  // and the remaining nanoseconds of the value.
-  const hrValues = new Uint32Array(3);
-
-  function hrtime(time) {
-    _hrtime(hrValues);
-
-    if (time !== undefined) {
-      if (!ArrayIsArray(time)) {
-        throw new ERR_INVALID_ARG_TYPE('time', 'Array', time);
-      }
-      if (time.length !== 2) {
-        throw new ERR_OUT_OF_RANGE('time', 2, time.length);
-      }
-
-      const sec = (hrValues[0] * 0x100000000 + hrValues[1]) - time[0];
-      const nsec = hrValues[2] - time[1];
-      const needsBorrow = nsec < 0;
-      return [needsBorrow ? sec - 1 : sec, needsBorrow ? nsec + 1e9 : nsec];
-    }
-
-    return [
-      hrValues[0] * 0x100000000 + hrValues[1],
-      hrValues[2]
-    ];
-  }
-
-  // Use a BigUint64Array in the closure because this is actually a bit
-  // faster than simply returning a BigInt from C++ in V8 7.1.
-  const hrBigintValues = new BigUint64Array(1);
-  function hrtimeBigInt() {
-    _hrtimeBigInt(hrBigintValues);
-    return hrBigintValues[0];
-  }
-
   const memValues = new Float64Array(5);
   function memoryUsage() {
     _memoryUsage(memValues);
@@ -228,8 +233,6 @@ function wrapProcessMethods(binding) {
 
   return {
     _rawDebug,
-    hrtime,
-    hrtimeBigInt,
     cpuUsage,
     resourceUsage,
     memoryUsage,
@@ -359,6 +362,7 @@ function toggleTraceCategoryState(asyncHooksEnabled) {
 
 module.exports = {
   toggleTraceCategoryState,
+  getFastAPIs,
   assert,
   buildAllowedFlags,
   wrapProcessMethods
