@@ -3,8 +3,6 @@
 const {
   ArrayIsArray,
   Error,
-  ErrorPrototypeToString,
-  ErrorCaptureStackTrace,
   String,
 } = primordials;
 
@@ -82,10 +80,10 @@ function onWarning(warning) {
   if (trace && warning.stack) {
     msg += `${warning.stack}`;
   } else {
-    msg +=
+    const toString =
       typeof warning.toString === 'function' ?
-        `${warning.toString()}` :
-        ErrorPrototypeToString(warning);
+        warning.toString : Error.prototype.toString;
+    msg += `${toString.apply(warning)}`;
   }
   if (typeof warning.detail === 'string') {
     msg += `\n${warning.detail}`;
@@ -137,8 +135,13 @@ function emitWarning(warning, type, code, ctor) {
   if (warning.name === 'DeprecationWarning') {
     if (process.noDeprecation)
       return;
-    if (process.throwDeprecation)
-      throw warning;
+    if (process.throwDeprecation) {
+      // Delay throwing the error to guarantee that all former warnings were
+      // properly logged.
+      return process.nextTick(() => {
+        throw warning;
+      });
+    }
   }
   process.nextTick(doEmitWarning, warning);
 }
@@ -159,7 +162,8 @@ function createWarningObject(warning, type, code, ctor, detail) {
   warning.name = String(type || 'Warning');
   if (code !== undefined) warning.code = code;
   if (detail !== undefined) warning.detail = detail;
-  ErrorCaptureStackTrace(warning, ctor || process.emitWarning);
+  // eslint-disable-next-line no-restricted-syntax
+  Error.captureStackTrace(warning, ctor || process.emitWarning);
   return warning;
 }
 
