@@ -1,9 +1,7 @@
 'use strict';
 
 const {
-  ArrayPrototypeIndexOf,
   Error,
-  StringPrototypeStartsWith,
 } = primordials;
 
 let debug = require('internal/util/debuglog').debuglog('source_map', (fn) => {
@@ -17,7 +15,6 @@ const {
   overrideStackTrace,
   maybeOverridePrepareStackTrace
 } = require('internal/errors');
-const { fileURLToPath } = require('internal/url');
 
 // Create a prettified stacktrace, inserting context from source maps
 // if possible.
@@ -43,12 +40,14 @@ const prepareStackTrace = (globalThis, error, trace) => {
   }
 
   let errorSource = '';
+  let firstSource;
   let firstLine;
   let firstColumn;
   const preparedTrace = trace.map((t, i) => {
     if (i === 0) {
       firstLine = t.getLineNumber();
       firstColumn = t.getColumnNumber();
+      firstSource = t.getFileName();
     }
     let str = i !== 0 ? '\n    at ' : '';
     str = `${str}${t}`;
@@ -64,22 +63,16 @@ const prepareStackTrace = (globalThis, error, trace) => {
         } = sm.findEntry(t.getLineNumber() - 1, t.getColumnNumber() - 1);
         if (originalSource && originalLine !== undefined &&
             originalColumn !== undefined) {
+          const originalSourceNoScheme = originalSource
+            .replace(/^file:\/\//, '');
           if (i === 0) {
             firstLine = originalLine + 1;
             firstColumn = originalColumn + 1;
-
+            firstSource = originalSourceNoScheme;
             // Show error in original source context to help user pinpoint it:
-            errorSource = getErrorSource(
-              sm.payload,
-              originalSource,
-              firstLine,
-              firstColumn
-            );
+            errorSource = getErrorSource(firstSource, firstLine, firstColumn);
           }
           // Show both original and transpiled stack trace information:
-          const originalSourceNoScheme =
-            StringPrototypeStartsWith(originalSource, 'file://') ?
-              fileURLToPath(originalSource) : originalSource;
           str += `\n        -> ${originalSourceNoScheme}:${originalLine + 1}:` +
             `${originalColumn + 1}`;
         }
@@ -95,29 +88,15 @@ const prepareStackTrace = (globalThis, error, trace) => {
 // Places a snippet of code from where the exception was originally thrown
 // above the stack trace. This logic is modeled after GetErrorSource in
 // node_errors.cc.
-function getErrorSource(payload, originalSource, firstLine, firstColumn) {
+function getErrorSource(firstSource, firstLine, firstColumn) {
   let exceptionLine = '';
-  const originalSourceNoScheme =
-    StringPrototypeStartsWith(originalSource, 'file://') ?
-      fileURLToPath(originalSource) : originalSource;
-
   let source;
-  const sourceContentIndex =
-    ArrayPrototypeIndexOf(payload.sources, originalSource);
-  if (payload.sourcesContent?.[sourceContentIndex]) {
-    // First we check if the original source content was provided in the
-    // source map itself:
-    source = payload.sourcesContent[sourceContentIndex];
-  } else {
-    // If no sourcesContent was found, attempt to load the original source
-    // from disk:
-    try {
-      source = readFileSync(originalSourceNoScheme, 'utf8');
-    } catch (err) {
-      debug(err);
-    }
+  try {
+    source = readFileSync(firstSource, 'utf8');
+  } catch (err) {
+    debug(err);
+    return exceptionLine;
   }
-
   const lines = source.split(/\r?\n/, firstLine);
   const line = lines[firstLine - 1];
   if (!line) return exceptionLine;
@@ -131,8 +110,7 @@ function getErrorSource(payload, originalSource, firstLine, firstColumn) {
   }
   prefix = prefix.slice(0, -1); // The last character is the '^'.
 
-  exceptionLine =
-              `${originalSourceNoScheme}:${firstLine}\n${line}\n${prefix}^\n\n`;
+  exceptionLine = `${firstSource}:${firstLine}\n${line}\n${prefix}^\n\n`;
   return exceptionLine;
 }
 
