@@ -3,19 +3,24 @@
 const {
   Array,
   ArrayIsArray,
-  ArrayPrototypeFilter,
-  ArrayPrototypePushApply,
+  BigInt64Array,
   BigIntPrototypeValueOf,
+  BigUint64Array,
   BooleanPrototypeValueOf,
   DatePrototypeGetTime,
   DatePrototypeToISOString,
   DatePrototypeToString,
   ErrorPrototypeToString,
+  Float32Array,
+  Float64Array,
   FunctionPrototypeCall,
   FunctionPrototypeToString,
+  Int8Array,
+  Int16Array,
+  Int32Array,
   JSONStringify,
   Map,
-  MapPrototypeGetSize,
+  MapPrototype,
   MapPrototypeEntries,
   MathFloor,
   MathMax,
@@ -41,14 +46,10 @@ const {
   ObjectPrototypePropertyIsEnumerable,
   ObjectSeal,
   ObjectSetPrototypeOf,
-  ReflectApply,
   RegExp,
-  RegExpPrototypeExec,
   RegExpPrototypeToString,
-  SafeSet,
-  SafeStringIterator,
   Set,
-  SetPrototypeGetSize,
+  SetPrototype,
   SetPrototypeValues,
   String,
   StringPrototypeValueOf,
@@ -56,10 +57,11 @@ const {
   SymbolPrototypeValueOf,
   SymbolIterator,
   SymbolToStringTag,
-  TypedArrayPrototypeGetLength,
-  TypedArrayPrototypeGetSymbolToStringTag,
+  Uint16Array,
+  Uint32Array,
   Uint8Array,
-  globalThis,
+  Uint8ArrayPrototype,
+  Uint8ClampedArray,
   uncurryThis,
 } = primordials;
 
@@ -117,19 +119,35 @@ const {
   isNumberObject,
   isBooleanObject,
   isBigIntObject,
+  isUint8Array,
+  isUint8ClampedArray,
+  isUint16Array,
+  isUint32Array,
+  isInt8Array,
+  isInt16Array,
+  isInt32Array,
+  isFloat32Array,
+  isFloat64Array,
+  isBigInt64Array,
+  isBigUint64Array
 } = require('internal/util/types');
 
 const assert = require('internal/assert');
 
 const { NativeModule } = require('internal/bootstrap/loaders');
 
+const setSizeGetter = uncurryThis(
+  ObjectGetOwnPropertyDescriptor(SetPrototype, 'size').get);
+const mapSizeGetter = uncurryThis(
+  ObjectGetOwnPropertyDescriptor(MapPrototype, 'size').get);
+const typedArraySizeGetter = uncurryThis(
+  ObjectGetOwnPropertyDescriptor(
+    ObjectGetPrototypeOf(Uint8ArrayPrototype), 'length').get);
+
 let hexSlice;
 
-const builtInObjects = new SafeSet(
-  ArrayPrototypeFilter(
-    ObjectGetOwnPropertyNames(globalThis),
-    (e) => RegExpPrototypeExec(/^[A-Z][a-zA-Z0-9]+$/, e) !== null
-  )
+const builtInObjects = new Set(
+  ObjectGetOwnPropertyNames(global).filter((e) => /^[A-Z][a-zA-Z0-9]+$/.test(e))
 );
 
 // https://tc39.es/ecma262/#sec-IsHTMLDDA-internal-slot
@@ -144,7 +162,7 @@ const inspectDefaultOptions = ObjectSeal({
   customInspect: true,
   showProxy: false,
   maxArrayLength: 100,
-  maxStringLength: Infinity,
+  maxStringLength: 10000,
   breakLength: 80,
   compact: 3,
   sorted: false,
@@ -165,7 +183,7 @@ const strEscapeSequencesReplacerSingle = /[\x00-\x1f\x5c\x7f-\x9f]/g;
 const keyStrRegExp = /^[a-zA-Z_][a-zA-Z_0-9]*$/;
 const numberRegExp = /^(0|[1-9][0-9]*)$/;
 
-const coreModuleRegExp = /^    at (?:[^/\\(]+ \(|)((?<![/\\]).+)\.js:\d+:\d+\)?$/;
+const coreModuleRegExp = /^    at (?:[^/\\(]+ \(|)node:(.+):\d+:\d+\)?$/;
 const nodeModulesRegExp = /[/\\]node_modules[/\\](.+?)(?=[/\\])/g;
 
 const classRegExp = /^(\s+[^(]*?)\s*{/;
@@ -199,7 +217,7 @@ const meta = [
 ];
 
 // Regex used for ansi escape code splitting
-// Adopted from https://github.com/chalk/ansi-regex/blob/HEAD/index.js
+// Adopted from https://github.com/chalk/ansi-regex/blob/master/index.js
 // License: MIT, authors: @sindresorhus, Qix-, arjunmehta and LitoMore
 // Matches all ansi escape code sequences in a string
 const ansiPattern = '[\\u001B\\u009B][[\\]()#;?]*' +
@@ -297,8 +315,7 @@ function inspect(value, opts) {
       ctx.showHidden = opts;
     } else if (opts) {
       const optKeys = ObjectKeys(opts);
-      for (let i = 0; i < optKeys.length; ++i) {
-        const key = optKeys[i];
+      for (const key of optKeys) {
         // TODO(BridgeAR): Find a solution what to do about stylize. Either make
         // this function public or add a new API with a similar or better
         // functionality.
@@ -344,7 +361,7 @@ inspect.colors = ObjectAssign(ObjectCreate(null), {
   italic: [3, 23],
   underline: [4, 24],
   blink: [5, 25],
-  // Swap foreground and background colors
+  // Swap forground and background colors
   inverse: [7, 27], // Alias: swapcolors, swapColors
   hidden: [8, 28], // Alias: conceal
   strikethrough: [9, 29], // Alias: strikeThrough, crossedout, crossedOut
@@ -517,14 +534,6 @@ function getEmptyFormatArray() {
   return [];
 }
 
-function isInstanceof(object, proto) {
-  try {
-    return object instanceof proto;
-  } catch {
-    return false;
-  }
-}
-
 function getConstructorName(obj, ctx, recurseTimes, protoProps) {
   let firstProto;
   const tmp = obj;
@@ -533,7 +542,7 @@ function getConstructorName(obj, ctx, recurseTimes, protoProps) {
     if (descriptor !== undefined &&
         typeof descriptor.value === 'function' &&
         descriptor.value.name !== '' &&
-        isInstanceof(tmp, descriptor.value)) {
+        tmp instanceof descriptor.value) {
       if (protoProps !== undefined &&
          (firstProto !== obj ||
          !builtInObjects.has(descriptor.value.name))) {
@@ -619,7 +628,7 @@ function addPrototypeProperties(ctx, main, obj, recurseTimes, output) {
         continue;
       }
       const value = formatProperty(
-        ctx, obj, recurseTimes, key, kObjectType, desc, main);
+        ctx, obj, recurseTimes, key, kObjectType, desc);
       if (ctx.colors) {
         // Faint!
         output.push(`\u001b[2m${value}\u001b[22m`);
@@ -635,7 +644,7 @@ function addPrototypeProperties(ctx, main, obj, recurseTimes, output) {
 
 function getPrefix(constructor, tag, fallback, size = '') {
   if (constructor === null) {
-    if (tag !== '' && fallback !== tag) {
+    if (tag !== '') {
       return `[${fallback}${size}: null prototype] [${tag}] `;
     }
     return `[${fallback}${size}: null prototype] `;
@@ -654,7 +663,7 @@ function getKeys(value, showHidden) {
   if (showHidden) {
     keys = ObjectGetOwnPropertyNames(value);
     if (symbols.length !== 0)
-      ArrayPrototypePushApply(keys, symbols);
+      keys.push(...symbols);
   } else {
     // This might throw if `value` is a Module Namespace Object from an
     // unevaluated module, but we don't want to perform the actual type
@@ -670,7 +679,7 @@ function getKeys(value, showHidden) {
     }
     if (symbols.length !== 0) {
       const filter = (key) => ObjectPrototypePropertyIsEnumerable(value, key);
-      ArrayPrototypePushApply(keys, ArrayPrototypeFilter(symbols, filter));
+      keys.push(...symbols.filter(filter));
     }
   }
   return keys;
@@ -695,13 +704,33 @@ function formatProxy(ctx, proxy, recurseTimes) {
   ctx.indentationLvl += 2;
   const res = [
     formatValue(ctx, proxy[0], recurseTimes),
-    formatValue(ctx, proxy[1], recurseTimes),
+    formatValue(ctx, proxy[1], recurseTimes)
   ];
   ctx.indentationLvl -= 2;
   return reduceToSingleString(
     ctx, res, '', ['Proxy [', ']'], kArrayExtrasType, recurseTimes);
 }
 
+function findTypedConstructor(value) {
+  for (const [check, clazz] of [
+    [isUint8Array, Uint8Array],
+    [isUint8ClampedArray, Uint8ClampedArray],
+    [isUint16Array, Uint16Array],
+    [isUint32Array, Uint32Array],
+    [isInt8Array, Int8Array],
+    [isInt16Array, Int16Array],
+    [isInt32Array, Int32Array],
+    [isFloat32Array, Float32Array],
+    [isFloat64Array, Float64Array],
+    [isBigInt64Array, BigInt64Array],
+    [isBigUint64Array, BigUint64Array]
+  ]) {
+    if (check(value)) {
+      return clazz;
+    }
+  }
+}
+
 // Note: using `formatValue` directly requires the indentation level to be
 // corrected by setting `ctx.indentationLvL += diff` and then to decrease the
 // value afterwards again.
@@ -825,7 +854,7 @@ function formatRaw(ctx, value, recurseTimes, typedArray) {
       extrasType = kArrayExtrasType;
       formatter = formatArray;
     } else if (isSet(value)) {
-      const size = SetPrototypeGetSize(value);
+      const size = setSizeGetter(value);
       const prefix = getPrefix(constructor, tag, 'Set', `(${size})`);
       keys = getKeys(value, ctx.showHidden);
       formatter = constructor !== null ?
@@ -835,7 +864,7 @@ function formatRaw(ctx, value, recurseTimes, typedArray) {
         return `${prefix}{}`;
       braces = [`${prefix}{`, '}'];
     } else if (isMap(value)) {
-      const size = MapPrototypeGetSize(value);
+      const size = mapSizeGetter(value);
       const prefix = getPrefix(constructor, tag, 'Map', `(${size})`);
       keys = getKeys(value, ctx.showHidden);
       formatter = constructor !== null ?
@@ -849,11 +878,12 @@ function formatRaw(ctx, value, recurseTimes, typedArray) {
       let bound = value;
       let fallback = '';
       if (constructor === null) {
-        fallback = TypedArrayPrototypeGetSymbolToStringTag(value);
+        const constr = findTypedConstructor(value);
+        fallback = constr.name;
         // Reconstruct the array information.
-        bound = new primordials[fallback](value);
+        bound = new constr(value);
       }
-      const size = TypedArrayPrototypeGetLength(value);
+      const size = typedArraySizeGetter(value);
       const prefix = getPrefix(constructor, tag, fallback, `(${size})`);
       braces = [`${prefix}[`, ']'];
       if (value.length === 0 && keys.length === 0 && !ctx.showHidden)
@@ -948,7 +978,7 @@ function formatRaw(ctx, value, recurseTimes, typedArray) {
       braces[0] = `${getPrefix(constructor, tag, 'WeakMap')}{`;
       formatter = ctx.showHidden ? formatWeakMap : formatWeakCollection;
     } else if (isModuleNamespaceObject(value)) {
-      braces[0] = `${getPrefix(constructor, tag, 'Module')}{`;
+      braces[0] = `[${tag}] {`;
       // Special handle keys for namespace objects.
       formatter = formatNamespaceObject.bind(null, keys);
     } else if (isBoxedPrimitive(value)) {
@@ -1347,8 +1377,7 @@ function handleMaxCallStackSize(ctx, err, constructorName, indentationLvl) {
       'special'
     );
   }
-  /* c8 ignore next */
-  assert.fail(err.stack);
+  throw err;
 }
 
 function formatNumber(fn, value) {
@@ -1399,7 +1428,9 @@ function formatNamespaceObject(keys, ctx, value, recurseTimes) {
       output[i] = formatProperty(ctx, value, recurseTimes, keys[i],
                                  kObjectType);
     } catch (err) {
-      assert(isNativeError(err) && err.name === 'ReferenceError');
+      if (!(isNativeError(err) && err.name === 'ReferenceError')) {
+        throw err;
+      }
       // Use the existing functionality. This makes sure the indentation and
       // line breaks are always correct. Otherwise it is very difficult to keep
       // this aligned, even though this is a hacky way of dealing with this.
@@ -1513,7 +1544,7 @@ function formatTypedArray(value, length, ctx, ignored, recurseTimes) {
       'length',
       'byteLength',
       'byteOffset',
-      'buffer',
+      'buffer'
     ]) {
       const str = formatValue(ctx, value[key], recurseTimes, true);
       output.push(`[${key}]: ${str}`);
@@ -1536,10 +1567,9 @@ function formatSet(value, ctx, ignored, recurseTimes) {
 function formatMap(value, ctx, ignored, recurseTimes) {
   const output = [];
   ctx.indentationLvl += 2;
-  for (const { 0: k, 1: v } of value) {
-    output.push(
-      `${formatValue(ctx, k, recurseTimes)} => ${formatValue(ctx, v, recurseTimes)}`
-    );
+  for (const [k, v] of value) {
+    output.push(`${formatValue(ctx, k, recurseTimes)} => ` +
+                formatValue(ctx, v, recurseTimes));
   }
   ctx.indentationLvl -= 2;
   return output;
@@ -1579,8 +1609,8 @@ function formatMapIterInner(ctx, recurseTimes, entries, state) {
   if (state === kWeak) {
     for (; i < maxLength; i++) {
       const pos = i * 2;
-      output[i] =
-        `${formatValue(ctx, entries[pos], recurseTimes)} => ${formatValue(ctx, entries[pos + 1], recurseTimes)}`;
+      output[i] = `${formatValue(ctx, entries[pos], recurseTimes)}` +
+        ` => ${formatValue(ctx, entries[pos + 1], recurseTimes)}`;
     }
     // Sort all entries to have a halfway reliable output (if more entries than
     // retrieved ones exist, we can not reliably return the same output) if the
@@ -1592,7 +1622,7 @@ function formatMapIterInner(ctx, recurseTimes, entries, state) {
       const pos = i * 2;
       const res = [
         formatValue(ctx, entries[pos], recurseTimes),
-        formatValue(ctx, entries[pos + 1], recurseTimes),
+        formatValue(ctx, entries[pos + 1], recurseTimes)
       ];
       output[i] = reduceToSingleString(
         ctx, res, '', ['[', ']'], kArrayExtrasType, recurseTimes);
@@ -1620,7 +1650,7 @@ function formatWeakMap(ctx, value, recurseTimes) {
 }
 
 function formatIterator(braces, ctx, value, recurseTimes) {
-  const { 0: entries, 1: isKeyValue } = previewEntries(value, true);
+  const [entries, isKeyValue] = previewEntries(value, true);
   if (isKeyValue) {
     // Mark entry iterators as such.
     braces[0] = braces[0].replace(/ Iterator] {$/, ' Entries] {');
@@ -1632,7 +1662,7 @@ function formatIterator(braces, ctx, value, recurseTimes) {
 
 function formatPromise(ctx, value, recurseTimes) {
   let output;
-  const { 0: state, 1: result } = getPromiseDetails(value);
+  const [state, result] = getPromiseDetails(value);
   if (state === kPending) {
     output = [ctx.stylize('<pending>', 'special')];
   } else {
@@ -1642,14 +1672,13 @@ function formatPromise(ctx, value, recurseTimes) {
     output = [
       state === kRejected ?
         `${ctx.stylize('<rejected>', 'special')} ${str}` :
-        str,
+        str
     ];
   }
   return output;
 }
 
-function formatProperty(ctx, value, recurseTimes, key, type, desc,
-                        original = value) {
+function formatProperty(ctx, value, recurseTimes, key, type, desc) {
   let name, str;
   let extra = ' ';
   desc = desc || ObjectGetOwnPropertyDescriptor(value, key) ||
@@ -1670,7 +1699,7 @@ function formatProperty(ctx, value, recurseTimes, key, type, desc,
           (ctx.getters === 'get' && desc.set === undefined) ||
           (ctx.getters === 'set' && desc.set !== undefined))) {
       try {
-        const tmp = ReflectApply(desc.get, original, []);
+        const tmp = value[key];
         ctx.indentationLvl += 2;
         if (tmp === null) {
           str = `${s(`[${label}:`, sp)} ${s('null', 'null')}${s(']', sp)}`;
@@ -1699,8 +1728,6 @@ function formatProperty(ctx, value, recurseTimes, key, type, desc,
   if (typeof key === 'symbol') {
     const tmp = key.toString().replace(strEscapeSequencesReplacer, escapeFn);
     name = `[${ctx.stylize(tmp, 'symbol')}]`;
-  } else if (key === '__proto__') {
-    name = "['__proto__']";
   } else if (desc.enumerable === false) {
     name = `[${key.replace(strEscapeSequencesReplacer, escapeFn)}]`;
   } else if (keyStrRegExp.test(key)) {
@@ -1739,7 +1766,7 @@ function reduceToSingleString(
   ctx, output, base, braces, extrasType, recurseTimes, value) {
   if (ctx.compact !== true) {
     if (typeof ctx.compact === 'number' && ctx.compact >= 1) {
-      // Memorize the original output length. In case the output is grouped,
+      // Memorize the original output length. In case the the output is grouped,
       // prevent lining up the entries on a single line.
       const entries = output.length;
       // Group array elements together if the array contains at least six
@@ -1850,7 +1877,7 @@ function tryStringify(arg) {
 }
 
 function format(...args) {
-  return formatWithOptionsInternal(undefined, args);
+  return formatWithOptionsInternal(undefined, ...args);
 }
 
 function formatWithOptions(inspectOptions, ...args) {
@@ -1858,10 +1885,10 @@ function formatWithOptions(inspectOptions, ...args) {
     throw new ERR_INVALID_ARG_TYPE(
       'inspectOptions', 'object', inspectOptions);
   }
-  return formatWithOptionsInternal(inspectOptions, args);
+  return formatWithOptionsInternal(inspectOptions, ...args);
 }
 
-function formatWithOptionsInternal(inspectOptions, args) {
+function formatWithOptionsInternal(inspectOptions, ...args) {
   const first = args[0];
   let a = 0;
   let str = '';
@@ -2017,7 +2044,7 @@ if (internalBinding('config').hasIntl) {
     if (removeControlChars)
       str = stripVTControlCharacters(str);
     str = str.normalize('NFC');
-    for (const char of new SafeStringIterator(str)) {
+    for (const char of str) {
       const code = char.codePointAt(0);
       if (isFullWidthCodePoint(code)) {
         width += 2;
