@@ -3,17 +3,14 @@
 const assert = require('internal/assert');
 const {
   ArrayIsArray,
-  ArrayPrototypeForEach,
-  ArrayPrototypeIndexOf,
-  ArrayPrototypeSome,
   ObjectCreate,
   ObjectDefineProperty,
   ObjectGetPrototypeOf,
   ObjectSetPrototypeOf,
-  PromiseAll,
-  SafeWeakMap,
+  SafePromise,
   Symbol,
   TypeError,
+  WeakMap,
 } = primordials;
 
 const { isContext } = internalBinding('contextify');
@@ -64,7 +61,7 @@ const STATUS_MAP = {
 
 let globalModuleId = 0;
 const defaultModuleName = 'vm:module';
-const wrapToModuleMap = new SafeWeakMap();
+const wrapToModuleMap = new WeakMap();
 
 const kWrap = Symbol('kWrap');
 const kContext = Symbol('kContext');
@@ -324,8 +321,6 @@ class SourceTextModule extends Module {
           throw new ERR_VM_MODULE_DIFFERENT_CONTEXT();
         }
         if (module.status === 'errored') {
-          // TODO(devsnek): replace with ERR_VM_MODULE_LINK_FAILURE
-          // and error cause proposal.
           throw new ERR_VM_MODULE_LINKING_ERRORED();
         }
         if (module.status === 'unlinked') {
@@ -336,7 +331,7 @@ class SourceTextModule extends Module {
 
       try {
         if (promises !== undefined) {
-          await PromiseAll(promises);
+          await SafePromise.all(promises);
         }
       } catch (e) {
         this.#error = e;
@@ -396,13 +391,13 @@ class SourceTextModule extends Module {
 class SyntheticModule extends Module {
   constructor(exportNames, evaluateCallback, options = {}) {
     if (!ArrayIsArray(exportNames) ||
-      ArrayPrototypeSome(exportNames, (e) => typeof e !== 'string')) {
+      exportNames.some((e) => typeof e !== 'string')) {
       throw new ERR_INVALID_ARG_TYPE('exportNames',
                                      'Array of unique strings',
                                      exportNames);
     } else {
-      ArrayPrototypeForEach(exportNames, (name, i) => {
-        if (ArrayPrototypeIndexOf(exportNames, name, i + 1) !== -1) {
+      exportNames.forEach((name, i) => {
+        if (exportNames.indexOf(name, i + 1) !== -1) {
           throw new ERR_INVALID_ARG_VALUE(`exportNames.${name}`,
                                           name,
                                           'is duplicated');
