@@ -23,7 +23,6 @@
 
 const {
   ArrayIsArray,
-  ArrayPrototypeIndexOf,
   Boolean,
   Error,
   Number,
@@ -39,7 +38,9 @@ const stream = require('stream');
 let debug = require('internal/util/debuglog').debuglog('net', (fn) => {
   debug = fn;
 });
-const { deprecate } = require('internal/util');
+const {
+  assertCrypto,
+} = require('internal/util');
 const {
   isIP,
   isIPv4,
@@ -91,10 +92,10 @@ const {
     ERR_INVALID_ARG_VALUE,
     ERR_INVALID_FD_TYPE,
     ERR_INVALID_IP_ADDRESS,
-    ERR_INVALID_OPT_VALUE,
     ERR_SERVER_ALREADY_LISTEN,
     ERR_SERVER_NOT_RUNNING,
-    ERR_SOCKET_CLOSED
+    ERR_SOCKET_CLOSED,
+    ERR_MISSING_ARGS,
   },
   errnoException,
   exceptionWithHostPort,
@@ -115,6 +116,7 @@ const {
 // Lazy loaded to improve startup performance.
 let cluster;
 let dns;
+let BlockList;
 
 const { clearTimeout } = require('timers');
 const { kTimeout } = require('internal/timers');
@@ -124,7 +126,7 @@ const DEFAULT_IPV6_ADDR = '::';
 
 const isWindows = process.platform === 'win32';
 
-const noop = () => {};
+function noop() {}
 
 function getFlags(ipv6Only) {
   return ipv6Only === true ? TCPConstants.UV_TCP_IPV6ONLY : 0;
@@ -290,20 +292,15 @@ function Socket(options) {
   else
     options = { ...options };
 
-  const { allowHalfOpen } = options;
-
-  // Prevent the "no-half-open enforcer" from being inherited from `Duplex`.
-  options.allowHalfOpen = true;
+  // Default to *not* allowing half open sockets.
+  options.allowHalfOpen = Boolean(options.allowHalfOpen);
   // For backwards compat do not emit close on destroy.
   options.emitClose = false;
-  options.autoDestroy = false;
+  options.autoDestroy = true;
   // Handle strings directly.
   options.decodeStrings = false;
   stream.Duplex.call(this, options);
 
-  // Default to *not* allowing half open sockets.
-  this.allowHalfOpen = Boolean(allowHalfOpen);
-
   if (options.handle) {
     this._handle = options.handle; // private
     this[async_id_symbol] = getNewAsyncId(this._handle);
@@ -420,28 +417,18 @@ Socket.prototype._final = function(cb) {
   const err = this._handle.shutdown(req);
 
   if (err === 1 || err === UV_ENOTCONN)  // synchronous finish
-    return afterShutdown.call(req, 0);
+    return cb();
   else if (err !== 0)
-    return this.destroy(errnoException(err, 'shutdown'));
+    return cb(errnoException(err, 'shutdown'));
 };
 
-
-function afterShutdown(status) {
+function afterShutdown() {
   const self = this.handle[owner_symbol];
 
   debug('afterShutdown destroyed=%j', self.destroyed,
         self._readableState);
 
   this.callback();
-
-  // Callback may come after call to destroy.
-  if (self.destroyed)
-    return;
-
-  if (!self.readable || self.readableEnded) {
-    debug('readableState ended, destroying');
-    self.destroy();
-  }
 }
 
 // Provide a better error message when we call end() as a result
@@ -456,10 +443,10 @@ function writeAfterFIN(chunk, encoding, cb) {
   // eslint-disable-next-line no-restricted-syntax
   const er = new Error('This socket has been ended by the other party');
   er.code = 'EPIPE';
-  process.nextTick(emitErrorNT, this, er);
   if (typeof cb === 'function') {
     defaultTriggerAsyncIdScope(this[async_id_symbol], process.nextTick, cb, er);
   }
+  this.destroy(er);
 
   return false;
 }
@@ -589,8 +576,7 @@ Socket.prototype._read = function(n) {
 
 
 Socket.prototype.end = function(data, encoding, callback) {
-  stream.Duplex.prototype.end.call(this,
-                                   data, encoding, callback);
+  stream.Duplex.prototype.end.call(this, data, encoding, callback);
   DTRACE_NET_STREAM_END(this);
   return this;
 };
@@ -632,12 +618,7 @@ Socket.prototype.read = function(n) {
 function onReadableStreamEnd() {
   if (!this.allowHalfOpen) {
     this.write = writeAfterFIN;
-    if (this.writable)
-      this.end();
-    else if (!this.writableLength)
-      this.destroy();
-  } else if (!this.destroyed && !this.writable && !this.writableLength)
-    this.destroy();
+  }
 }
 
 
@@ -693,14 +674,12 @@ Socket.prototype._destroy = function(exception, cb) {
 };
 
 Socket.prototype._getpeername = function() {
-  if (!this._peername) {
-    if (!this._handle || !this._handle.getpeername) {
-      return {};
-    }
-    const out = {};
-    const err = this._handle.getpeername(out);
-    if (err) return {};  // FIXME(bnoordhuis) Throw?
-    this._peername = out;
+  if (!this._handle || !this._handle.getpeername) {
+    return this._peername || {};
+  } else if (!this._peername) {
+    this._peername = {};
+    // FIXME(bnoordhuis) Throw when the return value is not 0?
+    this._handle.getpeername(this._peername);
   }
   return this._peername;
 };
@@ -733,12 +712,10 @@ protoGetter('remotePort', function remotePort() {
 Socket.prototype._getsockname = function() {
   if (!this._handle || !this._handle.getsockname) {
     return {};
-  }
-  if (!this._sockname) {
-    const out = {};
-    const err = this._handle.getsockname(out);
-    if (err) return {};  // FIXME(bnoordhuis) Throw?
-    this._sockname = out;
+  } else if (!this._sockname) {
+    this._sockname = {};
+    // FIXME(bnoordhuis) Throw when the return value is not 0?
+    this._handle.getsockname(this._sockname);
   }
   return this._sockname;
 };
@@ -940,6 +917,10 @@ Socket.prototype.connect = function(...args) {
   const options = normalized[0];
   const cb = normalized[1];
 
+  // options.port === null will be checked later.
+  if (options.port === undefined && options.path == null)
+    throw new ERR_MISSING_ARGS(['options', 'port', 'path']);
+
   if (this.write !== Socket.prototype.write)
     this.write = Socket.prototype.write;
 
@@ -1181,21 +1162,6 @@ function Server(options, connectionListener) {
 
   this._connections = 0;
 
-  ObjectDefineProperty(this, 'connections', {
-    get: deprecate(() => {
-
-      if (this._usingWorkers) {
-        return null;
-      }
-      return this._connections;
-    }, 'Server.connections property is deprecated. ' +
-       'Use Server.getConnections method instead.', 'DEP0020'),
-    set: deprecate((val) => (this._connections = val),
-                   'Server.connections property is deprecated.',
-                   'DEP0020'),
-    configurable: true, enumerable: false
-  });
-
   this[async_id_symbol] = -1;
   this._handle = null;
   this._usingWorkers = false;
@@ -1492,7 +1458,7 @@ Server.prototype.listen = function(...args) {
                                     'must have the property "port" or "path"');
   }
 
-  throw new ERR_INVALID_OPT_VALUE('options', options);
+  throw new ERR_INVALID_ARG_VALUE('options', options);
 };
 
 function lookupAndListen(self, port, address, backlog, exclusive, flags) {
@@ -1691,7 +1657,7 @@ Server.prototype._setupWorker = function(socketList) {
   this._usingWorkers = true;
   this._workers.push(socketList);
   socketList.once('exit', (socketList) => {
-    const index = ArrayPrototypeIndexOf(this._workers, socketList);
+    const index = this._workers.indexOf(socketList);
     this._workers.splice(index, 1);
   });
 };
@@ -1756,6 +1722,11 @@ module.exports = {
   _createServerHandle: createServerHandle,
   _normalizeArgs: normalizeArgs,
   _setSimultaneousAccepts,
+  get BlockList() {
+    if (BlockList === undefined)
+      BlockList = require('internal/blocklist');
+    return BlockList;
+  },
   connect,
   createConnection: connect,
   createServer,
@@ -1766,3 +1737,25 @@ module.exports = {
   Socket,
   Stream: Socket, // Legacy naming
 };
+
+if (process.versions.ngtcp2 !== undefined) {
+  let quic;
+
+  function lazyQuic() {
+    if (quic === undefined) {
+      assertCrypto();
+      quic = require('internal/quic/core');
+      process.emitWarning(
+        'The QUIC protocol is experimental and not yet ' +
+        'supported for production use',
+        'ExperimentalWarning');
+    }
+    return quic;
+  }
+
+  function createQuicSocket(...args) {
+    return lazyQuic().createSocket(...args);
+  }
+
+  module.exports.createQuicSocket = createQuicSocket;
+}
