@@ -4,6 +4,7 @@ const {
   ArrayPrototypePush,
   FunctionPrototypeBind,
   ObjectEntries,
+  String,
   Symbol,
 } = primordials;
 
@@ -20,13 +21,36 @@ const {
   validateObject,
 } = require('internal/validators');
 const { WASI: _WASI } = internalBinding('wasi');
-const kExitCode = Symbol('exitCode');
-const kSetMemory = Symbol('setMemory');
-const kStarted = Symbol('started');
+const kExitCode = Symbol('kExitCode');
+const kSetMemory = Symbol('kSetMemory');
+const kStarted = Symbol('kStarted');
+const kInstance = Symbol('kInstance');
 
 emitExperimentalWarning('WASI');
 
 
+function setupInstance(self, instance) {
+  validateObject(instance, 'instance');
+  validateObject(instance.exports, 'instance.exports');
+
+  // WASI::_SetMemory() in src/node_wasi.cc only expects that |memory| is
+  // an object. It will try to look up the .buffer property when needed
+  // and fail with UVWASI_EINVAL when the property is missing or is not
+  // an ArrayBuffer. Long story short, we don't need much validation here
+  // but we type-check anyway because it helps catch bugs in the user's
+  // code early.
+  validateObject(instance.exports.memory, 'instance.exports.memory');
+  if (!isArrayBuffer(instance.exports.memory.buffer)) {
+    throw new ERR_INVALID_ARG_TYPE(
+      'instance.exports.memory.buffer',
+      ['WebAssembly.Memory'],
+      instance.exports.memory.buffer);
+  }
+
+  self[kInstance] = instance;
+  self[kSetMemory](instance.exports.memory);
+}
+
 class WASI {
   constructor(options = {}) {
     validateObject(options, 'options');
@@ -75,51 +99,31 @@ class WASI {
     this.wasiImport = wrap;
     this[kStarted] = false;
     this[kExitCode] = 0;
+    this[kInstance] = undefined;
   }
 
+  // Must not export _initialize, must export _start
   start(instance) {
-    validateObject(instance, 'instance');
+    if (this[kStarted]) {
+      throw new ERR_WASI_ALREADY_STARTED();
+    }
+    this[kStarted] = true;
 
-    const exports = instance.exports;
+    setupInstance(this, instance);
 
-    validateObject(exports, 'instance.exports');
-
-    const { _initialize, _start, memory } = exports;
+    const { _start, _initialize } = this[kInstance].exports;
 
     if (typeof _start !== 'function') {
       throw new ERR_INVALID_ARG_TYPE(
         'instance.exports._start', 'function', _start);
     }
-
     if (_initialize !== undefined) {
       throw new ERR_INVALID_ARG_TYPE(
         'instance.exports._initialize', 'undefined', _initialize);
     }
 
-    // WASI::_SetMemory() in src/node_wasi.cc only expects that |memory| is
-    // an object. It will try to look up the .buffer property when needed
-    // and fail with UVWASI_EINVAL when the property is missing or is not
-    // an ArrayBuffer. Long story short, we don't need much validation here
-    // but we type-check anyway because it helps catch bugs in the user's
-    // code early.
-    validateObject(memory, 'instance.exports.memory');
-
-    if (!isArrayBuffer(memory.buffer)) {
-      throw new ERR_INVALID_ARG_TYPE(
-        'instance.exports.memory.buffer',
-        ['WebAssembly.Memory'],
-        memory.buffer);
-    }
-
-    if (this[kStarted]) {
-      throw new ERR_WASI_ALREADY_STARTED();
-    }
-
-    this[kStarted] = true;
-    this[kSetMemory](memory);
-
     try {
-      exports._start();
+      _start();
     } catch (err) {
       if (err !== kExitCode) {
         throw err;
@@ -128,6 +132,31 @@ class WASI {
 
     return this[kExitCode];
   }
+
+  // Must not export _start, may optionally export _initialize
+  initialize(instance) {
+    if (this[kStarted]) {
+      throw new ERR_WASI_ALREADY_STARTED();
+    }
+    this[kStarted] = true;
+
+    setupInstance(this, instance);
+
+    const { _start, _initialize } = this[kInstance].exports;
+
+    if (typeof _initialize !== 'function' && _initialize !== undefined) {
+      throw new ERR_INVALID_ARG_TYPE(
+        'instance.exports._initialize', 'function', _initialize);
+    }
+    if (_start !== undefined) {
+      throw new ERR_INVALID_ARG_TYPE(
+        'instance.exports._start', 'undefined', _initialize);
+    }
+
+    if (_initialize !== undefined) {
+      _initialize();
+    }
+  }
 }
 
 
