@@ -1,6 +1,7 @@
 #include "node_url.h"
 #include "base_object-inl.h"
 #include "node_errors.h"
+#include "node_external_reference.h"
 #include "node_i18n.h"
 #include "util-inl.h"
 
@@ -206,7 +207,8 @@ CHAR_TEST(8, IsForbiddenHostCodePoint,
           ch == '\0' || ch == '\t' || ch == '\n' || ch == '\r' ||
           ch == ' ' || ch == '#' || ch == '%' || ch == '/' ||
           ch == ':' || ch == '?' || ch == '@' || ch == '[' ||
-          ch == '\\' || ch == ']')
+          ch == '<' || ch == '>' || ch == '\\' || ch == ']' ||
+          ch == '^')
 
 // https://url.spec.whatwg.org/#windows-drive-letter
 TWO_CHAR_STRING_TEST(8, IsWindowsDriveLetter,
@@ -725,7 +727,6 @@ std::string PercentDecode(const char* input, size_t len) {
 #define SPECIALS(XX)                                                          \
   XX(ftp, 21, "ftp:")                                                         \
   XX(file, -1, "file:")                                                       \
-  XX(gopher, 70, "gopher:")                                                   \
   XX(http, 80, "http:")                                                       \
   XX(https, 443, "https:")                                                    \
   XX(ws, 80, "ws:")                                                           \
@@ -778,6 +779,8 @@ bool ToASCII(const std::string& input, std::string* output) {
   MaybeStackBuffer<char> buf;
   if (i18n::ToASCII(&buf, input.c_str(), input.length()) < 0)
     return false;
+  if (buf.length() == 0)
+    return false;
   output->assign(*buf, buf.length());
   return true;
 }
@@ -1426,7 +1429,7 @@ void URL::Parse(const char* input,
     const char ch = p < end ? p[0] : kEOL;
     bool special = (url->flags & URL_FLAGS_SPECIAL);
     bool cannot_be_base;
-    bool special_back_slash = (special && ch == '\\');
+    const bool special_back_slash = (special && ch == '\\');
 
     switch (state) {
       case kSchemeStart:
@@ -1458,13 +1461,11 @@ void URL::Parse(const char* input,
                 ((buffer == "file:") &&
                  ((url->flags & URL_FLAGS_HAS_USERNAME) ||
                   (url->flags & URL_FLAGS_HAS_PASSWORD) ||
-                  (url->port != -1)))) {
+                  (url->port != -1))) ||
+                  (url->scheme == "file:" && url->host.empty())) {
               url->flags |= URL_FLAGS_TERMINATED;
               return;
             }
-
-            // File scheme && (host == empty or null) check left to JS-land
-            // as it can be done before even entering C++ binding.
           }
 
           url->scheme = std::move(buffer);
@@ -1476,7 +1477,6 @@ void URL::Parse(const char* input,
             url->flags &= ~URL_FLAGS_SPECIAL;
             special = false;
           }
-          special_back_slash = (special && ch == '\\');
           buffer.clear();
           if (has_state_override)
             return;
@@ -1521,7 +1521,6 @@ void URL::Parse(const char* input,
             url->flags &= ~URL_FLAGS_SPECIAL;
             special = false;
           }
-          special_back_slash = (special && ch == '\\');
           if (base->flags & URL_FLAGS_HAS_PATH) {
             url->flags |= URL_FLAGS_HAS_PATH;
             url->path = base->path;
@@ -1545,7 +1544,6 @@ void URL::Parse(const char* input,
           url->flags |= URL_FLAGS_SPECIAL;
           special = true;
           state = kFile;
-          special_back_slash = (special && ch == '\\');
           continue;
         }
         break;
@@ -1575,7 +1573,6 @@ void URL::Parse(const char* input,
           url->flags &= ~URL_FLAGS_SPECIAL;
           special = false;
         }
-        special_back_slash = (special && ch == '\\');
         switch (ch) {
           case kEOL:
             if (base->flags & URL_FLAGS_HAS_USERNAME) {
@@ -1856,13 +1853,14 @@ void URL::Parse(const char* input,
         break;
       case kFile:
         url->scheme = "file:";
+        url->host.clear();
+        url->flags |= URL_FLAGS_HAS_HOST;
         if (ch == '/' || ch == '\\') {
           state = kFileSlash;
         } else if (has_base && base->scheme == "file:") {
           switch (ch) {
             case kEOL:
               if (base->flags & URL_FLAGS_HAS_HOST) {
-                url->flags |= URL_FLAGS_HAS_HOST;
                 url->host = base->host;
               }
               if (base->flags & URL_FLAGS_HAS_PATH) {
@@ -1876,7 +1874,6 @@ void URL::Parse(const char* input,
               break;
             case '?':
               if (base->flags & URL_FLAGS_HAS_HOST) {
-                url->flags |= URL_FLAGS_HAS_HOST;
                 url->host = base->host;
               }
               if (base->flags & URL_FLAGS_HAS_PATH) {
@@ -1889,7 +1886,6 @@ void URL::Parse(const char* input,
               break;
             case '#':
               if (base->flags & URL_FLAGS_HAS_HOST) {
-                url->flags |= URL_FLAGS_HAS_HOST;
                 url->host = base->host;
               }
               if (base->flags & URL_FLAGS_HAS_PATH) {
@@ -1905,16 +1901,18 @@ void URL::Parse(const char* input,
               state = kFragment;
               break;
             default:
+              url->query.clear();
+              if (base->flags & URL_FLAGS_HAS_HOST) {
+                url->host = base->host;
+              }
+              if (base->flags & URL_FLAGS_HAS_PATH) {
+                url->flags |= URL_FLAGS_HAS_PATH;
+                url->path = base->path;
+              }
               if (!StartsWithWindowsDriveLetter(p, end)) {
-                if (base->flags & URL_FLAGS_HAS_HOST) {
-                  url->flags |= URL_FLAGS_HAS_HOST;
-                  url->host = base->host;
-                }
-                if (base->flags & URL_FLAGS_HAS_PATH) {
-                  url->flags |= URL_FLAGS_HAS_PATH;
-                  url->path = base->path;
-                }
                 ShortenUrlPath(url);
+              } else {
+                url->path.clear();
               }
               state = kPath;
               continue;
@@ -1928,20 +1926,13 @@ void URL::Parse(const char* input,
         if (ch == '/' || ch == '\\') {
           state = kFileHost;
         } else {
-          if (has_base &&
-              base->scheme == "file:" &&
-              !StartsWithWindowsDriveLetter(p, end)) {
-            if (IsNormalizedWindowsDriveLetter(base->path[0])) {
+          if (has_base && base->scheme == "file:") {
+            url->flags |= URL_FLAGS_HAS_HOST;
+            url->host = base->host;
+            if (!StartsWithWindowsDriveLetter(p, end) &&
+                IsNormalizedWindowsDriveLetter(base->path[0])) {
               url->flags |= URL_FLAGS_HAS_PATH;
               url->path.push_back(base->path[0]);
-            } else {
-              if (base->flags & URL_FLAGS_HAS_HOST) {
-                url->flags |= URL_FLAGS_HAS_HOST;
-                url->host = base->host;
-              } else {
-                url->flags &= ~URL_FLAGS_HAS_HOST;
-                url->host.clear();
-              }
             }
           }
           state = kPath;
@@ -2025,29 +2016,19 @@ void URL::Parse(const char* input,
                 url->path.empty() &&
                 buffer.size() == 2 &&
                 IsWindowsDriveLetter(buffer)) {
-              if ((url->flags & URL_FLAGS_HAS_HOST) &&
-                  !url->host.empty()) {
-                url->host.clear();
-                url->flags |= URL_FLAGS_HAS_HOST;
-              }
               buffer[1] = ':';
             }
             url->flags |= URL_FLAGS_HAS_PATH;
             url->path.emplace_back(std::move(buffer));
           }
           buffer.clear();
-          if (url->scheme == "file:" &&
-              (ch == kEOL ||
-               ch == '?' ||
-               ch == '#')) {
-            while (url->path.size() > 1 && url->path[0].empty()) {
-              url->path.erase(url->path.begin());
-            }
-          }
           if (ch == '?') {
             url->flags |= URL_FLAGS_HAS_QUERY;
+            url->query.clear();
             state = kQuery;
           } else if (ch == '#') {
+            url->flags |= URL_FLAGS_HAS_FRAGMENT;
+            url->fragment.clear();
             state = kFragment;
           }
         } else {
@@ -2087,8 +2068,6 @@ void URL::Parse(const char* input,
             url->flags |= URL_FLAGS_HAS_FRAGMENT;
             url->fragment = std::move(buffer);
             break;
-          case 0:
-            break;
           default:
             AppendOrEscape(&buffer, ch, FRAGMENT_ENCODE_SET);
         }
@@ -2329,6 +2308,15 @@ void Initialize(Local<Object> target,
 }
 }  // namespace
 
+void RegisterExternalReferences(ExternalReferenceRegistry* registry) {
+  registry->Register(Parse);
+  registry->Register(EncodeAuthSet);
+  registry->Register(ToUSVString);
+  registry->Register(DomainToASCII);
+  registry->Register(DomainToUnicode);
+  registry->Register(SetURLConstructor);
+}
+
 std::string URL::ToFilePath() const {
   if (context_.scheme != "file:") {
     return "";
@@ -2450,3 +2438,4 @@ MaybeLocal<Value> URL::ToObject(Environment* env) const {
 }  // namespace node
 
 NODE_MODULE_CONTEXT_AWARE_INTERNAL(url, node::url::Initialize)
+NODE_MODULE_EXTERNAL_REFERENCE(url, node::url::RegisterExternalReferences)
