@@ -50,6 +50,7 @@ const noop = () => {};
 
 const hasCrypto = Boolean(process.versions.openssl) &&
                   !process.env.NODE_SKIP_CRYPTO;
+const hasQuic = hasCrypto && Boolean(process.versions.ngtcp2);
 
 // Check for flags. Skip this for workers (both, the `cluster` module and
 // `worker_threads`) and child processes.
@@ -275,8 +276,8 @@ if (global.gc) {
   knownGlobals.push(global.gc);
 }
 
-function allowGlobals(...allowlist) {
-  knownGlobals = knownGlobals.concat(allowlist);
+function allowGlobals(...whitelist) {
+  knownGlobals = knownGlobals.concat(whitelist);
 }
 
 if (process.env.NODE_TEST_KNOWN_GLOBALS !== '0') {
@@ -612,11 +613,8 @@ function getBufferSources(buf) {
   return [...getArrayBufferViews(buf), new Uint8Array(buf).buffer];
 }
 
-// Crash the process on unhandled rejections.
-const crashOnUnhandledRejection = (err) => { throw err; };
-process.on('unhandledRejection', crashOnUnhandledRejection);
 function disableCrashOnUnhandledRejection() {
-  process.removeListener('unhandledRejection', crashOnUnhandledRejection);
+  process.on('unhandledRejection', () => {});
 }
 
 function getTTYfd() {
@@ -699,20 +697,6 @@ function gcUntil(name, condition) {
   });
 }
 
-function requireNoPackageJSONAbove() {
-  let possiblePackage = path.join(__dirname, '..', 'package.json');
-  let lastPackage = null;
-  while (possiblePackage !== lastPackage) {
-    if (fs.existsSync(possiblePackage)) {
-      assert.fail(
-        'This test shouldn\'t load properties from a package.json above ' +
-        `its file location. Found package.json at ${possiblePackage}.`);
-    }
-    lastPackage = possiblePackage;
-    possiblePackage = path.join(possiblePackage, '..', '..', 'package.json');
-  }
-}
-
 const common = {
   allowGlobals,
   buildType,
@@ -729,6 +713,7 @@ const common = {
   getTTYfd,
   hasIntl,
   hasCrypto,
+  hasQuic,
   hasMultiLocalhost,
   invalidArgTypeHelper,
   isAIX,
@@ -751,7 +736,6 @@ const common = {
   platformTimeout,
   printSkipMessage,
   pwdCommand,
-  requireNoPackageJSONAbove,
   runWithInvalidFD,
   skip,
   skipIf32Bits,
