@@ -17,7 +17,6 @@ const expectedModules = new Set([
   'Internal Binding credentials',
   'Internal Binding fs',
   'Internal Binding fs_dir',
-  'Internal Binding fs_event_wrap',
   'Internal Binding messaging',
   'Internal Binding module_wrap',
   'Internal Binding native_module',
@@ -32,11 +31,11 @@ const expectedModules = new Set([
   'Internal Binding types',
   'Internal Binding url',
   'Internal Binding util',
-  'Internal Binding uv',
-  'NativeModule async_hooks',
+  'Internal Binding worker',
   'NativeModule buffer',
   'NativeModule events',
   'NativeModule fs',
+  'NativeModule internal/abort_controller',
   'NativeModule internal/assert',
   'NativeModule internal/async_hooks',
   'NativeModule internal/bootstrap/pre_execution',
@@ -46,12 +45,10 @@ const expectedModules = new Set([
   'NativeModule internal/constants',
   'NativeModule internal/encoding',
   'NativeModule internal/errors',
+  'NativeModule internal/event_target',
   'NativeModule internal/fixed_queue',
   'NativeModule internal/fs/dir',
   'NativeModule internal/fs/utils',
-  'NativeModule internal/fs/promises',
-  'NativeModule internal/fs/rimraf',
-  'NativeModule internal/fs/watchers',
   'NativeModule internal/idna',
   'NativeModule internal/linkedlist',
   'NativeModule internal/modules/run_main',
@@ -67,7 +64,6 @@ const expectedModules = new Set([
   'NativeModule internal/modules/esm/resolve',
   'NativeModule internal/modules/esm/transform_source',
   'NativeModule internal/modules/esm/translators',
-  'NativeModule internal/modules/esm/handle_process_exit',
   'NativeModule internal/process/esm_loader',
   'NativeModule internal/options',
   'NativeModule internal/priority_queue',
@@ -80,28 +76,38 @@ const expectedModules = new Set([
   'NativeModule internal/process/warning',
   'NativeModule internal/querystring',
   'NativeModule internal/source_map/source_map_cache',
-  'NativeModule internal/streams/utils',
+  'NativeModule internal/streams/buffer_list',
+  'NativeModule internal/streams/destroy',
+  'NativeModule internal/streams/duplex',
+  'NativeModule internal/streams/end-of-stream',
+  'NativeModule internal/streams/legacy',
+  'NativeModule internal/streams/passthrough',
+  'NativeModule internal/streams/pipeline',
+  'NativeModule internal/streams/readable',
+  'NativeModule internal/streams/state',
+  'NativeModule internal/streams/transform',
+  'NativeModule internal/streams/writable',
   'NativeModule internal/timers',
   'NativeModule internal/url',
   'NativeModule internal/util',
   'NativeModule internal/util/debuglog',
   'NativeModule internal/util/inspect',
-  'NativeModule internal/util/iterable_weak_map',
   'NativeModule internal/util/types',
   'NativeModule internal/validators',
   'NativeModule internal/vm/module',
+  'NativeModule internal/worker/io',
   'NativeModule internal/worker/js_transferable',
-  'NativeModule internal/blob',
   'NativeModule path',
+  'NativeModule stream',
   'NativeModule timers',
   'NativeModule url',
+  'NativeModule util',
   'NativeModule vm',
 ]);
 
 if (!common.isMainThread) {
   [
     'Internal Binding messaging',
-    'Internal Binding performance',
     'Internal Binding symbols',
     'Internal Binding worker',
     'NativeModule internal/streams/duplex',
@@ -110,7 +116,6 @@ if (!common.isMainThread) {
     'NativeModule internal/streams/transform',
     'NativeModule internal/streams/writable',
     'NativeModule internal/error_serdes',
-    'NativeModule internal/event_target',
     'NativeModule internal/process/worker_thread_only',
     'NativeModule internal/streams/buffer_list',
     'NativeModule internal/streams/destroy',
@@ -121,7 +126,6 @@ if (!common.isMainThread) {
     'NativeModule internal/worker',
     'NativeModule internal/worker/io',
     'NativeModule stream',
-    'NativeModule util',
     'NativeModule worker_threads',
   ].forEach(expectedModules.add.bind(expectedModules));
 }
