@@ -91,14 +91,14 @@ const bobPublicKey = crypto.createPublicKey({
 
 assert.throws(() => crypto.diffieHellman({ privateKey: alicePrivateKey }), {
   name: 'TypeError',
-  code: 'ERR_INVALID_OPT_VALUE',
-  message: 'The value "undefined" is invalid for option "publicKey"'
+  code: 'ERR_INVALID_ARG_VALUE',
+  message: "The property 'options.publicKey' is invalid. Received undefined"
 });
 
 assert.throws(() => crypto.diffieHellman({ publicKey: alicePublicKey }), {
   name: 'TypeError',
-  code: 'ERR_INVALID_OPT_VALUE',
-  message: 'The value "undefined" is invalid for option "privateKey"'
+  code: 'ERR_INVALID_ARG_VALUE',
+  message: "The property 'options.privateKey' is invalid. Received undefined"
 });
 
 const privateKey = Buffer.from(
@@ -196,9 +196,10 @@ for (const [params1, params2] of [
 test(crypto.generateKeyPairSync('ec', { namedCurve: 'secp256k1' }),
      crypto.generateKeyPairSync('ec', { namedCurve: 'secp256k1' }));
 
+const not256k1 = crypto.getCurves().find((c) => /^sec.*(224|384|512)/.test(c));
 assert.throws(() => {
   test(crypto.generateKeyPairSync('ec', { namedCurve: 'secp256k1' }),
-       crypto.generateKeyPairSync('ec', { namedCurve: 'secp224k1' }));
+       crypto.generateKeyPairSync('ec', { namedCurve: not256k1 }));
 }, {
   name: 'Error',
   code: 'ERR_OSSL_EVP_DIFFERENT_PARAMETERS'
