@@ -14,10 +14,9 @@ function testPBKDF2(password, salt, iterations, keylen, expected) {
     crypto.pbkdf2Sync(password, salt, iterations, keylen, 'sha256');
   assert.strictEqual(actual.toString('latin1'), expected);
 
-  crypto.pbkdf2(password, salt, iterations, keylen, 'sha256',
-                (err, actual) => {
-                  assert.strictEqual(actual.toString('latin1'), expected);
-                });
+  crypto.pbkdf2(password, salt, iterations, keylen, 'sha256', (err, actual) => {
+    assert.strictEqual(actual.toString('latin1'), expected);
+  });
 }
 
 
@@ -49,9 +48,7 @@ const expected =
 const key = crypto.pbkdf2Sync('password', 'salt', 32, 32, 'sha256');
 assert.strictEqual(key.toString('hex'), expected);
 
-crypto.pbkdf2('password', 'salt', 32, 32, 'sha256',
-              common.mustSucceed(ondone));
-
+crypto.pbkdf2('password', 'salt', 32, 32, 'sha256', common.mustSucceed(ondone));
 function ondone(key) {
   assert.strictEqual(key.toString('hex'), expected);
 }
@@ -146,14 +143,11 @@ assert.throws(
              'Received null'
   });
 [1, {}, [], true, undefined, null].forEach((input) => {
-  const msgPart2 = 'an instance of Buffer, TypedArray, or DataView.' +
-                   common.invalidArgTypeHelper(input);
   assert.throws(
     () => crypto.pbkdf2(input, 'salt', 8, 8, 'sha256', common.mustNotCall()),
     {
       code: 'ERR_INVALID_ARG_TYPE',
       name: 'TypeError',
-      message: `The "password" argument must be of type string or ${msgPart2}`
     }
   );
 
@@ -162,7 +156,6 @@ assert.throws(
     {
       code: 'ERR_INVALID_ARG_TYPE',
       name: 'TypeError',
-      message: `The "salt" argument must be of type string or ${msgPart2}`
     }
   );
 
@@ -171,7 +164,6 @@ assert.throws(
     {
       code: 'ERR_INVALID_ARG_TYPE',
       name: 'TypeError',
-      message: `The "password" argument must be of type string or ${msgPart2}`
     }
   );
 
@@ -180,7 +172,6 @@ assert.throws(
     {
       code: 'ERR_INVALID_ARG_TYPE',
       name: 'TypeError',
-      message: `The "salt" argument must be of type string or ${msgPart2}`
     }
   );
 });
@@ -207,25 +198,21 @@ assert.throws(
 });
 
 // Any TypedArray should work for password and salt
-crypto.pbkdf2(new Uint8Array(10), 'salt', 8, 8, 'sha256',
+crypto.pbkdf2(new Uint8Array(10), 'salt', 8, 8, 'sha256', common.mustSucceed());
+crypto.pbkdf2('pass', new Uint8Array(10), 8, 8, 'sha256', common.mustSucceed());
+crypto.pbkdf2(new Uint16Array(10), 'salt', 8, 8, 'sha1', common.mustSucceed());
+crypto.pbkdf2('pass', new Uint16Array(10), 8, 8, 'sha1', common.mustSucceed());
+crypto.pbkdf2(new Uint32Array(10), 'salt', 8, 8, 'sha1', common.mustSucceed());
+crypto.pbkdf2('pass', new Uint32Array(10), 8, 8, 'sha1', common.mustSucceed());
+crypto.pbkdf2(new Float32Array(10), 'salt', 8, 8, 'sha1', common.mustSucceed());
+crypto.pbkdf2('pass', new Float32Array(10), 8, 8, 'sha1', common.mustSucceed());
+crypto.pbkdf2(new Float64Array(10), 'salt', 8, 8, 'sha1', common.mustSucceed());
+crypto.pbkdf2('pass', new Float64Array(10), 8, 8, 'sha1', common.mustSucceed());
+crypto.pbkdf2(new ArrayBuffer(10), 'salt', 8, 8, 'sha1', common.mustSucceed());
+crypto.pbkdf2('pass', new ArrayBuffer(10), 8, 8, 'sha1', common.mustSucceed());
+crypto.pbkdf2(new SharedArrayBuffer(10), 'salt', 8, 8, 'sha256',
               common.mustSucceed());
-crypto.pbkdf2('pass', new Uint8Array(10), 8, 8, 'sha256',
-              common.mustSucceed());
-crypto.pbkdf2(new Uint16Array(10), 'salt', 8, 8, 'sha256',
-              common.mustSucceed());
-crypto.pbkdf2('pass', new Uint16Array(10), 8, 8, 'sha256',
-              common.mustSucceed());
-crypto.pbkdf2(new Uint32Array(10), 'salt', 8, 8, 'sha256',
-              common.mustSucceed());
-crypto.pbkdf2('pass', new Uint32Array(10), 8, 8, 'sha256',
-              common.mustSucceed());
-crypto.pbkdf2(new Float32Array(10), 'salt', 8, 8, 'sha256',
-              common.mustSucceed());
-crypto.pbkdf2('pass', new Float32Array(10), 8, 8, 'sha256',
-              common.mustSucceed());
-crypto.pbkdf2(new Float64Array(10), 'salt', 8, 8, 'sha256',
-              common.mustSucceed());
-crypto.pbkdf2('pass', new Float64Array(10), 8, 8, 'sha256',
+crypto.pbkdf2('pass', new SharedArrayBuffer(10), 8, 8, 'sha256',
               common.mustSucceed());
 
 crypto.pbkdf2Sync(new Uint8Array(10), 'salt', 8, 8, 'sha256');
@@ -238,6 +225,10 @@ crypto.pbkdf2Sync(new Float32Array(10), 'salt', 8, 8, 'sha256');
 crypto.pbkdf2Sync('pass', new Float32Array(10), 8, 8, 'sha256');
 crypto.pbkdf2Sync(new Float64Array(10), 'salt', 8, 8, 'sha256');
 crypto.pbkdf2Sync('pass', new Float64Array(10), 8, 8, 'sha256');
+crypto.pbkdf2Sync(new ArrayBuffer(10), 'salt', 8, 8, 'sha256');
+crypto.pbkdf2Sync('pass', new ArrayBuffer(10), 8, 8, 'sha256');
+crypto.pbkdf2Sync(new SharedArrayBuffer(10), 'salt', 8, 8, 'sha256');
+crypto.pbkdf2Sync('pass', new SharedArrayBuffer(10), 8, 8, 'sha256');
 
 assert.throws(
   () => crypto.pbkdf2('pass', 'salt', 8, 8, 'md55', common.mustNotCall()),
@@ -256,3 +247,11 @@ assert.throws(
     message: 'Invalid digest: md55'
   }
 );
+
+const kNotPBKDF2Supported = ['shake128', 'shake256'];
+crypto.getHashes()
+  .filter((hash) => !kNotPBKDF2Supported.includes(hash))
+  .forEach((hash) => {
+    crypto.pbkdf2Sync(new Uint8Array(10), 'salt', 8, 8, hash);
+    crypto.pbkdf2(new Uint8Array(10), 'salt', 8, 8, hash, common.mustCall());
+  });
