@@ -13,6 +13,7 @@ const assert = require('assert');
 const EE = require('events');
 const fs = require('fs');
 const { promisify } = require('util');
+const http = require('http');
 
 {
   const rs = new Readable({
@@ -475,23 +476,19 @@ testClosed((opts) => new Writable({ write() {}, ...opts }));
 }
 
 {
-  const w = new Writable({
-    write(chunk, encoding, callback) {
-      process.nextTick(callback);
-    }
+  const server = http.createServer((req, res) => {
+    res.on('close', () => {
+      finished(res, common.mustCall(() => {
+        server.close();
+      }));
+    });
+    res.end();
+  })
+  .listen(0, function() {
+    http.request({
+      method: 'GET',
+      port: this.address().port
+    }).end()
+      .on('response', common.mustCall());
   });
-  w.aborted = false;
-  w.end();
-  let closed = false;
-  w.on('finish', () => {
-    assert.strictEqual(closed, false);
-    w.emit('aborted');
-  });
-  w.on('close', common.mustCall(() => {
-    closed = true;
-  }));
-
-  finished(w, common.mustCall(() => {
-    assert.strictEqual(closed, true);
-  }));
 }
