@@ -627,7 +627,9 @@ const net = require('net');
     await Promise.resolve();
     yield 'hello';
   }, async function*(source) {
-    for await (const chunk of source) {}
+    for await (const chunk of source) {
+      chunk;
+    }
   }, common.mustCall((err) => {
     assert.strictEqual(err, undefined);
   }));
@@ -643,7 +645,9 @@ const net = require('net');
     await Promise.resolve();
     throw new Error('kaboom');
   }, async function*(source) {
-    for await (const chunk of source) {}
+    for await (const chunk of source) {
+      chunk;
+    }
   }, common.mustCall((err) => {
     assert.strictEqual(err.message, 'kaboom');
   }));
@@ -699,6 +703,7 @@ const net = require('net');
     yield 'world';
   }, s, async function(source) {
     for await (const chunk of source) {
+      chunk;
       throw new Error('kaboom');
     }
   }, common.mustCall((err, val) => {
@@ -713,6 +718,7 @@ const net = require('net');
     return ['hello', 'world'];
   }, s, async function*(source) {
     for await (const chunk of source) {
+      chunk;
       throw new Error('kaboom');
     }
   }, common.mustCall((err) => {
@@ -1212,38 +1218,25 @@ const net = require('net');
 }
 
 {
-  pipeline([1, 2, 3], PassThrough({ objectMode: true }),
-           common.mustSucceed(() => {}));
+  let flushed = false;
+  const makeStream = () =>
+    new Transform({
+      transform: (chunk, enc, cb) => cb(null, chunk),
+      flush: (cb) =>
+        setTimeout(() => {
+          flushed = true;
+          cb(null);
+        }, 1),
+    });
 
-  let res = '';
-  const w = new Writable({
-    write(chunk, encoding, callback) {
-      res += chunk;
-      callback();
-    },
-  });
-  pipeline(['1', '2', '3'], w, common.mustSucceed(() => {
-    assert.strictEqual(res, '123');
-  }));
-}
-{
-  function createThenable() {
-    let counter = 0;
-    return {
-      get then() {
-        if (counter++) {
-          throw new Error('Cannot access `then` more than once');
-        }
-        return Function.prototype;
-      },
-    };
-  }
+  const input = new Readable();
+  input.push(null);
 
   pipeline(
-    function* () {
-      yield 0;
-    },
-    createThenable,
-    () => common.mustNotCall(),
+    input,
+    makeStream(),
+    common.mustCall(() => {
+      assert.strictEqual(flushed, true);
+    }),
   );
 }
