@@ -1,8 +1,6 @@
-'use strict';
-
 // Flags: --experimental-vm-modules
-
-require('../common');
+'use strict';
+const common = require('../common');
 const assert = require('assert');
 
 const { SourceTextModule } = require('vm');
@@ -13,11 +11,7 @@ const { inspect } = require('util');
   await m.link(() => 0);
   assert.strictEqual(
     inspect(m.namespace),
-    '[Module: null prototype] { a: <uninitialized>, b: undefined }'
-  );
+    '[Module] { a: <uninitialized>, b: undefined }');
   await m.evaluate();
-  assert.strictEqual(
-    inspect(m.namespace),
-    '[Module: null prototype] { a: 1, b: 2 }'
-  );
-})();
+  assert.strictEqual(inspect(m.namespace), '[Module] { a: 1, b: 2 }');
+})().then(common.mustCall());
